package com.deepclone.lw.beans.acm;


import java.util.List;

import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import com.deepclone.lw.interfaces.acm.UsersDAO;
import com.deepclone.lw.interfaces.mailer.Mailer;
import com.deepclone.lw.sqld.accounts.QuittingAccount;



class AccountCleanupTask
		implements Runnable
{

	private TransactionTemplate tTemplate;
	private UsersDAO usersDao;
	private Mailer mailer;


	public AccountCleanupTask( TransactionTemplate tTemplate , UsersDAO usersDao , Mailer mailer )
	{
		this.tTemplate = tTemplate;
		this.usersDao = usersDao;
		this.mailer = mailer;
	}


	@Override
	public void run( )
	{
		this.sendInactivityWarnings( );
		this.disableInactiveAccounts( );
		this.dropOldAccounts( );
	}


	private void sendInactivityWarnings( )
	{
		List< QuittingAccount > toWarn;
		toWarn = this.tTemplate.execute( new TransactionCallback< List< QuittingAccount >>( ) {
			@Override
			public List< QuittingAccount > doInTransaction( TransactionStatus status )
			{
				return usersDao.getInactivesToWarn( );
			}
		} );

		for ( QuittingAccount account : toWarn ) {
			try {
				this.mailer.createMail( account.getLanguage( ) , "inactivityWarningMail" , account.getAddress( ) )
						.queue( );
			} catch ( Exception e ) {
				continue;
			}
		}
	}


	private void disableInactiveAccounts( )
	{
		List< QuittingAccount > toDisable;
		toDisable = this.tTemplate.execute( new TransactionCallback< List< QuittingAccount >>( ) {
			@Override
			public List< QuittingAccount > doInTransaction( TransactionStatus status )
			{
				return usersDao.getInactivesToDisable( );
			}
		} );

		for ( QuittingAccount account : toDisable ) {
			try {
				this.mailer.createMail( account.getLanguage( ) , "inactivityQuitMail" , account.getAddress( ) ).queue( );
			} catch ( Exception e ) {
				continue;
			}
		}
	}


	private void dropOldAccounts( )
	{
		this.tTemplate.execute( new TransactionCallbackWithoutResult( ) {
			@Override
			protected void doInTransactionWithoutResult( TransactionStatus status )
			{
				usersDao.deleteOldAccounts( );
			}
		} );
	}
}
