package com.deepclone.lw.beans.acm;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.cmd.admin.users.UserSession;
import com.deepclone.lw.interfaces.acm.UserSessionDAO;
import com.deepclone.lw.utils.StoredProc;



public class UserSessionDAOBean
		implements UserSessionDAO , InitializingBean
{

	private JdbcTemplate dTemplate;

	private final RowMapper< UserSession > mSession;

	private StoredProc fSessionStart;
	private StoredProc fSessionEnd;


	public UserSessionDAOBean( )
	{
		this.mSession = new RowMapper< UserSession >( ) {
			@Override
			public UserSession mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				UserSession session = new UserSession( );

				session.setId( rs.getLong( "id" ) );
				session.setCredentialsId( rs.getInt( "credentials_id" ) );
				session.setSessionName( rs.getString( "session" ) );
				session.setClientName( rs.getString( "client_name" ) );
				session.setExclusive( rs.getBoolean( "exclusive" ) );
				session.setFromAddress( rs.getString( "from_address" ) );
				if ( session.getFromAddress( ).startsWith( "/" )) {
					session.setFromAddress( session.getFromAddress( ).substring( 1 ) );
				}
				session.setStarted( rs.getTimestamp( "started" ) );
				session.setEnded( rs.getTimestamp( "ended" ) );
				if ( session.getEnded( ) != null ) {
					session.setEndType( SessionTerminationType.valueOf( rs.getString( "end_type" ) ) );
				}

				return session;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fSessionStart = new StoredProc( dataSource , "users" , "sessions_login" );
		this.fSessionStart.addParameter( "credentials_id" , Types.INTEGER );
		this.fSessionStart.addParameter( "session_name" , Types.VARCHAR );
		this.fSessionStart.addParameter( "client_type" , Types.VARCHAR );
		this.fSessionStart.addParameter( "ip_address" , Types.VARCHAR );
		this.fSessionStart.addOutput( "s_id" , Types.BIGINT );

		this.fSessionEnd = new StoredProc( dataSource , "users" , "sessions_terminate" );
		this.fSessionEnd.addParameter( "session_id" , Types.BIGINT );
		this.fSessionEnd.addParameter( "end_type" , "session_termination_type" );
	}


	@Override
	public void afterPropertiesSet( )
	{
		this.dTemplate.execute( "SELECT users.sessions_server_restart()" );
	}


	@Override
	public long startSession( int id , String sName , String client , String address )
	{
		return (Long) this.fSessionStart.execute( id , sName , client , address ).get( "s_id" );
	}


	@Override
	public void endSession( long session , SessionTerminationType termination )
	{
		this.fSessionEnd.execute( session , termination.toString( ) );
	}


	@Override
	public UserSession getSession( long id )
	{
		String sql = "SELECT * FROM users.sessions WHERE id = ?";
		try {
			return this.dTemplate.queryForObject( sql , this.mSession , id );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public boolean isOnline( int id )
	{
		String sql = "SELECT credentials_id FROM users.sessions WHERE credentials_id = ? AND end_type IS NULL AND exclusive";
		try {
			this.dTemplate.queryForInt( sql , id );
			return true;
		} catch ( EmptyResultDataAccessException e ) {
			return false;
		}
	}


	@Override
	public List< UserSession > getSessions( int id )
	{
		String sql = "SELECT * FROM users.sessions WHERE credentials_id = ?";
		return this.dTemplate.query( sql , this.mSession , id );
	}

}
