package com.deepclone.lw.beans.admin;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.admin.bans.ArchivedBanRequest;
import com.deepclone.lw.cmd.admin.bans.BanRequest;
import com.deepclone.lw.cmd.admin.bans.BanType;
import com.deepclone.lw.cmd.admin.bans.SummaryEntry;
import com.deepclone.lw.cmd.admin.bans.ValidatedBanRequest;
import com.deepclone.lw.interfaces.admin.BanMailData;
import com.deepclone.lw.interfaces.admin.BansDAO;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.StoredProc;



public class BansDAOBean
		implements BansDAO
{
	private static final String sGetPending = "SELECT * FROM admin.pending_bans ORDER BY requested DESC";
	private static final String sGetArchived = "SELECT * FROM admin.cancelled_bans ORDER BY updated DESC";
	private static final String sGetValidated = "SELECT * FROM admin.active_bans ORDER BY updated DESC";
	private static final String sGetBan = "SELECT * FROM admin.active_bans WHERE account_mail = ?";

	private static abstract class BanRequestMapper< T extends BanRequest >
			implements RowMapper< T >
	{
		protected void getCommonData( BanRequest br , ResultSet rs )
				throws SQLException
		{
			br.setId( rs.getInt( "id" ) );
			br.setRequestedById( rs.getInt( "requested_by_id" ) );
			br.setRequestedByName( rs.getString( "requested_by_name" ) );
			br.setAccountId( rs.getInt( "account_id" ) );
			br.setAccountMail( rs.getString( "account_mail" ) );
			br.setTimestamp( rs.getTimestamp( "requested" ) );
			br.setReason( rs.getString( "reason" ) );
		}
	}

	private JdbcTemplate dTemplate;

	private final BanRequestMapper< BanRequest > mPending;
	private final BanRequestMapper< BanRequest > mArchived;
	private final BanRequestMapper< BanRequest > mValidated;
	private final RowMapper< Boolean > mBoolean;

	private StoredProc fRequestBanAddress;
	private StoredProc fRequestBanEmpire;
	private StoredProc fRejectBan;
	private StoredProc fValidateBan;
	private StoredProc fLiftBan;


	public BansDAOBean( )
	{
		this.mPending = new BanRequestMapper< BanRequest >( ) {
			@Override
			public BanRequest mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BanRequest br = new BanRequest( );
				this.getCommonData( br , rs );
				return br;
			}
		};
		this.mArchived = new BanRequestMapper< BanRequest >( ) {
			@Override
			public BanRequest mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				ArchivedBanRequest br = new ArchivedBanRequest( );
				this.getCommonData( br , rs );
				br.setUpdate( rs.getTimestamp( "updated" ) );
				br.setExpired( rs.getBoolean( "expired" ) );
				if ( !br.isExpired( ) ) {
					br.setRejectedById( rs.getInt( "rejected_by_id" ) );
					br.setRejectedByName( rs.getString( "rejected_by_name" ) );
					br.setRejectionReason( rs.getString( "rejection_reason" ) );
				}
				return br;
			}
		};
		this.mValidated = new BanRequestMapper< BanRequest >( ) {
			@Override
			public BanRequest mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				ValidatedBanRequest br = new ValidatedBanRequest( );
				this.getCommonData( br , rs );
				br.setUpdate( rs.getTimestamp( "updated" ) );
				br.setRedeemable( rs.getBoolean( "redeemable" ) );
				br.setValidatedById( rs.getInt( "validated_by_id" ) );
				br.setValidatedByName( rs.getString( "validated_by_name" ) );
				return br;
			}
		};
		this.mBoolean = new RowMapper< Boolean >( ) {
			@Override
			public Boolean mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return rs.getBoolean( 1 );
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fRequestBanAddress = new StoredProc( dataSource , "admin" , "request_ban_on_address" );
		this.fRequestBanAddress.addParameter( "admin_id" , Types.INTEGER );
		this.fRequestBanAddress.addParameter( "user_address" , Types.VARCHAR );
		this.fRequestBanAddress.addParameter( "reason" , Types.VARCHAR );
		this.fRequestBanAddress.addOutput( "err_code" , Types.INTEGER );

		this.fRequestBanEmpire = new StoredProc( dataSource , "admin" , "request_ban_on_empire" );
		this.fRequestBanEmpire.addParameter( "admin_id" , Types.INTEGER );
		this.fRequestBanEmpire.addParameter( "user_address" , Types.VARCHAR );
		this.fRequestBanEmpire.addParameter( "reason" , Types.VARCHAR );
		this.fRequestBanEmpire.addOutput( "err_code" , Types.INTEGER );

		this.fRejectBan = new StoredProc( dataSource , "admin" , "reject_ban_request" );
		this.fRejectBan.addParameter( "admin_id" , Types.INTEGER );
		this.fRejectBan.addParameter( "request_id" , Types.INTEGER );
		this.fRejectBan.addParameter( "reason" , Types.VARCHAR );

		this.fValidateBan = new StoredProc( dataSource , "admin" , "confirm_ban_request" );
		this.fValidateBan.addParameter( "admin_id" , Types.INTEGER );
		this.fValidateBan.addParameter( "request_id" , Types.INTEGER );
		this.fValidateBan.addOutput( "success" , Types.BOOLEAN );
		this.fValidateBan.addOutput( "addr" , Types.VARCHAR );
		this.fValidateBan.addOutput( "lang" , Types.VARCHAR );
		this.fValidateBan.addOutput( "r_txt" , Types.VARCHAR );

		this.fLiftBan = new StoredProc( dataSource , "admin" , "lift_ban" );
		this.fLiftBan.addParameter( "admin_id" , Types.INTEGER );
		this.fLiftBan.addParameter( "request_id" , Types.INTEGER );
		this.fLiftBan.addOutput( "success" , Types.BOOLEAN );
		this.fLiftBan.addOutput( "addr" , Types.VARCHAR );
		this.fLiftBan.addOutput( "lang" , Types.VARCHAR );
	}


	@Override
	public List< SummaryEntry > getSummary( )
	{
		List< SummaryEntry > result = new ArrayList< SummaryEntry >( 3 );
		final String names[] = {
				"pending" , "cancelled" , "active"
		};

		for ( int i = 0 ; i < 3 ; i++ ) {
			long count = this.dTemplate.queryForLong( "SELECT count(*) FROM admin." + names[ i ] + "_bans" );
			result.add( new SummaryEntry( BanType.values( )[ i ] , count ) );
		}

		return result;
	}


	@Override
	public List< BanRequest > getPending( )
	{
		return this.dTemplate.query( sGetPending , this.mPending );
	}


	@Override
	public List< BanRequest > getArchived( )
	{
		return this.dTemplate.query( sGetArchived , this.mArchived );
	}


	@Override
	public List< BanRequest > getActive( )
	{
		return this.dTemplate.query( sGetValidated , this.mValidated );
	}


	private int requestBan( StoredProc fRequestBan , int administrator , String address , String reason )
	{
		return (Integer) fRequestBan.execute( administrator , address , reason ).get( "err_code" );
	}


	@Override
	public int requestBan( int administrator , EmailAddress address , String reason )
	{
		return this.requestBan( this.fRequestBanAddress , administrator , address.getAddress( ) , reason );
	}


	@Override
	public int requestBan( int administrator , String empire , String reason )
	{
		return this.requestBan( this.fRequestBanEmpire , administrator , empire , reason );
	}


	@Override
	public void rejectBan( int administrator , int requestId , String reason )
	{
		this.fRejectBan.execute( administrator , requestId , reason );
	}


	@Override
	public BanMailData validateBan( int administrator , int requestId )
	{
		Map< String , Object > result = this.fValidateBan.execute( administrator , requestId );
		boolean success = (Boolean) result.get( "success" );
		if ( !success ) {
			return null;
		}
		return new BanMailData( (String) result.get( "addr" ) , (String) result.get( "lang" ) , (String) result
				.get( "r_txt" ) );
	}


	@Override
	public BanMailData liftBan( int administrator , int requestId )
	{
		Map< String , Object > result = this.fLiftBan.execute( administrator , requestId );
		boolean success = (Boolean) result.get( "success" );
		if ( !success ) {
			return null;
		}
		return new BanMailData( (String) result.get( "addr" ) , (String) result.get( "lang" ) );
	}


	@Override
	public void expireBanRequests( )
	{
		this.dTemplate.execute( "SELECT admin.expire_ban_requests( )" );
	}


	@Override
	public void expireWarnings( )
	{
		this.dTemplate.execute( "SELECT admin.expire_warnings( )" );
	}


	@Override
	public boolean finaliseBan( )
	{
		String sql = "SELECT admin.delete_banned_empires( )";
		return this.dTemplate.queryForObject( sql , this.mBoolean );
	}


	@Override
	public ValidatedBanRequest getActiveBan( EmailAddress address )
	{
		try {
			return (ValidatedBanRequest) this.dTemplate.queryForObject( sGetBan , this.mValidated , address
					.getAddress( ) );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}

}
