package com.deepclone.lw.beans.prefs;


import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.cmd.player.gdata.MailPreference;
import com.deepclone.lw.cmd.player.gdata.MapSize;
import com.deepclone.lw.interfaces.prefs.PreferenceDefinitions;



public class DefaultPreferencesBean
		implements InitializingBean
{

	private PreferenceDefinitions pDefs;
	private PreferenceTypesRegistryBean registry;


	@Autowired( required = true )
	public void setTypesRegistry( PreferenceTypesRegistryBean registry )
	{
		this.registry = registry;
	}


	@Autowired( required = true )
	public void setPreferenceDefinitions( PreferenceDefinitions preferenceDefinitions )
	{
		this.pDefs = preferenceDefinitions;
	}


	@Override
	public void afterPropertiesSet( )
			throws Exception
	{
		this.createTypes( );

		this.pDefs.registerGroup( "display" , "pgDisplay" );
		this.pDefs.registerPreference( "useRLTime" , "display" , "pUseRLTime" , "pUseRLTimeDescription" ,
				(Boolean) false );

		this.pDefs.registerGroup( "map" , "pgMap" );
		this.pDefs.registerPreference( "mapX" , "map" , "pMapX" , "pMapXDescription" , (Integer) 0 );
		this.pDefs.registerPreference( "mapY" , "map" , "pMapY" , "pMapYDescription" , (Integer) 0 );
		this.pDefs.registerPreference( "mapSize" , "map" , "pMapSize" , "pMapSizeDescription" , MapSize.MEDIUM );

		this.pDefs.registerGroup( "mail" , "pgMail" );
		this.pDefs.registerPreference( "mailOnPM" , "mail" , "pMailOnPM" , "pMailOnPMDescription" ,
				MailPreference.INSTANT );
		this.pDefs.registerPreference( "mailOnAlliance" , "mail" , "pMailOnAlliance" , "pMailOnAllianceDescription" ,
				MailPreference.DAILY_RECAP );
		this.pDefs.registerPreference( "mailOnIM" , "mail" , "pMailOnIM" , "pMailOnIMDescription" ,
				MailPreference.DAILY_RECAP );
		this.pDefs.registerPreference( "mailOnAdmin" , "mail" , "pMailOnAdmin" , "pMailOnAdminDescription" ,
				MailPreference.INSTANT );
	}


	private void createTypes( )
	{
		EnumPreferenceType< MapSize > mapSizeType;
		mapSizeType = new EnumPreferenceType< MapSize >( MapSize.class );
		mapSizeType.setTranslation( MapSize.SMALL , "mapSizeSmall" );
		mapSizeType.setTranslation( MapSize.MEDIUM , "mapSizeMedium" );
		mapSizeType.setTranslation( MapSize.LARGE , "mapSizeLarge" );
		this.registry.register( mapSizeType );

		EnumPreferenceType< MailPreference > mailPrefType;
		mailPrefType = new EnumPreferenceType< MailPreference >( MailPreference.class );
		mailPrefType.setTranslation( MailPreference.NO_MAIL , "mailPrefNo" );
		mailPrefType.setTranslation( MailPreference.DAILY_RECAP , "mailPrefRecap" );
		mailPrefType.setTranslation( MailPreference.INSTANT , "mailPrefInstant" );
		this.registry.register( mailPrefType );
	}

}
