package com.deepclone.lw.beans.prefs;


import java.util.HashMap;
import java.util.Map;

import com.deepclone.lw.interfaces.prefs.PreferenceType;
import com.deepclone.lw.interfaces.prefs.PreferenceTypesRegistry;



public class PreferenceTypesRegistryBean
		implements PreferenceTypesRegistry
{

	private final Map< String , PreferenceType< ? >> types = new HashMap< String , PreferenceType< ? > >( );


	public PreferenceTypesRegistryBean( )
	{
		this.register( new StringPreferenceType( ) );
		this.register( new IntPreferenceType( ) );
		this.register( new BoolPreferenceType( ) );
	}


	void register( PreferenceType< ? > type )
	{
		String name = type.getType( ).getSimpleName( );
		synchronized ( this.types ) {
			if ( this.types.containsKey( name ) ) {
				throw new IllegalArgumentException( "Type '" + name + "' already registered." );
			}
			this.types.put( name , type );
		}
	}


	@Override
	@SuppressWarnings( "unchecked" )
	public < T > PreferenceType< T > getType( Class< T > jType )
	{
		PreferenceType< T > type;
		synchronized ( this.types ) {
			type = (PreferenceType< T >) this.types.get( jType.getSimpleName( ) );
		}
		if ( type == null ) {
			throw new IllegalArgumentException( "Type '" + jType.getSimpleName( ) + "' is not registered." );
		}
		return type;
	}


	@Override
	public PreferenceType< ? > getType( String typeName )
	{
		PreferenceType< ? > type;
		synchronized ( this.types ) {
			type = (PreferenceType< ? >) this.types.get( typeName );
		}
		if ( type == null ) {
			throw new IllegalArgumentException( "Type '" + typeName + "' is not registered." );
		}
		return type;
	}
}
