package com.deepclone.lw.beans.prefs;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.interfaces.prefs.AccountPreferences;
import com.deepclone.lw.interfaces.prefs.Preference;
import com.deepclone.lw.interfaces.prefs.PreferenceGroup;
import com.deepclone.lw.interfaces.prefs.PreferenceType;
import com.deepclone.lw.interfaces.prefs.PreferenceTypesRegistry;
import com.deepclone.lw.interfaces.prefs.PreferencesDAO;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.utils.StoredProc;



public class PreferencesDAOBean
		implements PreferencesDAO
{

	private JdbcTemplate dTemplate;
	private PreferenceTypesRegistry registry;
	private StoredProc fReset;
	private StoredProc fSetPref;


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fReset = new StoredProc( dataSource , "users" , "reset_preferences" );
		this.fReset.addParameter( "a_id" , Types.INTEGER );

		this.fSetPref = new StoredProc( dataSource , "users" , "set_preference" );
		this.fSetPref.addParameter( "a_id" , Types.INTEGER );
		this.fSetPref.addParameter( "p_name" , Types.VARCHAR );
		this.fSetPref.addParameter( "p_value" , Types.VARCHAR );
	}


	@Autowired( required = true )
	public void setTypesRegistry( PreferenceTypesRegistry registry )
	{
		this.registry = registry;
	}


	@Override
	public AccountPreferences getPreferences( int accountId )
	{
		String sql = "SELECT * FROM users.preferences_view WHERE account_id = ?";
		RowMapper< Map< String , String > > mapper = new RowMapper< Map< String , String > >( ) {

			@Override
			public Map< String , String > mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				final String[] names = {
						"group_name" , "group_i18n_name" , "pref_name" , "pref_i18n_name" , "pref_i18n_description" ,
						"pref_type" , "value"
				};
				Map< String , String > data = new HashMap< String , String >( );
				for ( String n : names ) {
					data.put( n , rs.getString( n ) );
				}
				return data;
			}

		};

		List< PreferenceGroup > groups = new LinkedList< PreferenceGroup >( );
		Map< String , String > prefGroups = new HashMap< String , String >( );
		String last = null;
		PreferenceGroup lg = null;
		for ( Map< String , String > data : this.dTemplate.query( sql , mapper , accountId ) ) {
			String gName = data.get( "group_name" );
			if ( last == null || !last.equals( gName ) ) {
				lg = new PreferenceGroup( gName , data.get( "group_i18n_name" ) );
				groups.add( lg );
				last = gName;
			}

			String pName = data.get( "pref_name" );
			prefGroups.put( pName , gName );

			PreferenceType< ? > prefType = this.registry.getType( data.get( "pref_type" ) );

			new Preference( pName , lg , data.get( "pref_i18n_name" ) , data.get( "pref_i18n_description" ) , prefType ,
					data.get( "value" ) );
		}

		return new AccountPreferences( groups , prefGroups );
	}


	@Override
	public AccountPreferences getPreferences( Account account )
	{
		return this.getPreferences( account.getId( ) );
	}


	@Override
	public void setPreferences( Account account , Map< String , String > values )
	{
		String sql = "SELECT pref_name , pref_type FROM users.preferences_view WHERE account_id = ?";
		RowMapper< Map< String , String > > mapper = new RowMapper< Map< String , String > >( ) {

			@Override
			public Map< String , String > mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				final String[] names = {
						"pref_name" , "pref_type"
				};
				Map< String , String > data = new HashMap< String , String >( );
				for ( String n : names ) {
					data.put( n , rs.getString( n ) );
				}
				return data;
			}

		};

		for ( Map< String , String > def : this.dTemplate.query( sql , mapper , account.getId( ) ) ) {
			String pName = def.get( "pref_name" );
			PreferenceType< ? > pType = this.registry.getType( def.get( "pref_type" ) );

			String inValue = values.get( pName );
			if ( inValue == null ) {
				continue;
			}

			Object inObject = pType.valueOf( inValue );
			if ( inObject == null ) {
				continue;
			}

			String dbValue = pType.convert( inObject );
			this.fSetPref.execute( account.getId( ) , pName , dbValue );
		}
	}


	@Override
	public void resetPreferences( Account account )
	{
		this.fReset.execute( account.getId( ) );
	}

}
