package com.deepclone.lw.beans.bt;


import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.bt.BugsSummaryResponse;
import com.deepclone.lw.cmd.admin.bt.GetSnapshotResponse;
import com.deepclone.lw.cmd.admin.bt.ListBugsResponse;
import com.deepclone.lw.cmd.admin.bt.MergeError;
import com.deepclone.lw.cmd.admin.bt.MergeReportsResponse;
import com.deepclone.lw.cmd.admin.bt.PostCommentResponse;
import com.deepclone.lw.cmd.admin.bt.ReportBugResponse;
import com.deepclone.lw.cmd.admin.bt.ViewBugResponse;
import com.deepclone.lw.cmd.bt.data.BugEvent;
import com.deepclone.lw.cmd.bt.data.BugEventType;
import com.deepclone.lw.cmd.bt.data.BugReport;
import com.deepclone.lw.cmd.bt.data.BugStatus;
import com.deepclone.lw.interfaces.bt.AdminBugs;
import com.deepclone.lw.interfaces.bt.BugsDAO;



@Transactional
public class AdminBugsBean
		implements AdminBugs
{

	private BugsDAO bugsDao;


	@Autowired( required = true )
	public void setBugsDao( BugsDAO bugsDao )
	{
		this.bugsDao = bugsDao;
	}


	private List< BugEvent > getBugEvents( long bugId )
	{
		List< BugEvent > events = this.bugsDao.getEvents( bugId );
		Iterator< BugEvent > it = events.iterator( );
		BugStatus status = BugStatus.PENDING;
		boolean pub = false;
		while ( it.hasNext( ) ) {
			BugEvent event = it.next( );
			if ( event.getType( ) == BugEventType.STATUS ) {
				if ( event.getStatus( ) == status ) {
					it.remove( );
				} else {
					status = event.getStatus( );
				}
			} else if ( event.getType( ) == BugEventType.VISIBILITY ) {
				if ( event.getVisible( ) == pub ) {
					it.remove( );
				} else {
					pub = event.getVisible( );
				}
			}
		}

		return events;
	}


	@Override
	public BugsSummaryResponse getSummary( Administrator admin )
	{
		long pending = this.bugsDao.countReports( admin , BugStatus.PENDING , false );
		long open = this.bugsDao.countReports( admin , BugStatus.OPEN , false );
		long own = this.bugsDao.countReports( admin , null , true );
		long updated = this.bugsDao.countUpdatedReports( admin );
		long total = this.bugsDao.countReports( admin , null , false );
		return new BugsSummaryResponse( admin , pending , open , own , updated , total );
	}


	@Override
	public ListBugsResponse getBugs( Administrator admin , BugStatus status , boolean ownOnly , long first , int count )
	{
		long nBugs = this.bugsDao.countReports( admin , status , ownOnly );
		List< BugReport > bugs = this.bugsDao.getReports( admin , status , ownOnly , first , count );
		return new ListBugsResponse( admin , status , ownOnly , first , count , nBugs , bugs );
	}


	@Override
	public ReportBugResponse postReport( Administrator admin , String title , String contents , boolean publicReport )
	{
		long bugId = this.bugsDao.postReport( admin , title , contents , publicReport );
		return new ReportBugResponse( admin , bugId );
	}


	@Override
	public ViewBugResponse getReport( Administrator admin , long bugId )
	{
		BugReport report = this.bugsDao.getReport( admin , bugId );
		if ( report == null ) {
			return new ViewBugResponse( admin , false );
		}

		return new ViewBugResponse( admin , report , this.getBugEvents( bugId ) );
	}


	@Override
	public PostCommentResponse postComment( Administrator admin , long bugId , String comment , boolean publicComment )
	{
		this.bugsDao.postComment( admin , bugId , comment , publicComment );
		return new PostCommentResponse( admin );
	}


	@Override
	public void moderateComment( Administrator admin , long commentId , boolean validation )
	{
		if ( validation ) {
			this.bugsDao.showComment( admin , commentId );
		} else {
			this.bugsDao.deleteComment( admin , commentId );
		}
	}


	@Override
	public void validateReport( Administrator admin , long bugId , BugStatus status , boolean publicReport ,
			int credits , boolean snapshot )
	{
		this.bugsDao.validateReport( admin , bugId , status , publicReport , credits , snapshot );
	}


	@Override
	public void setStatus( Administrator admin , long bugId , BugStatus status )
	{
		this.bugsDao.setStatus( admin , bugId , status );
	}


	@Override
	public void toggleVisibility( Administrator admin , long bugId )
	{
		this.bugsDao.toggleVisibility( admin , bugId );
	}


	@Override
	public MergeReportsResponse mergeReports( Administrator admin , long id , long mergeId )
	{
		int errCode = this.bugsDao.mergeReports( admin , id , mergeId );
		if ( errCode == 0 ) {
			return new MergeReportsResponse( admin );
		}

		BugReport report = this.bugsDao.getReport( admin , id );
		if ( report == null ) {
			return new MergeReportsResponse( admin , false );
		}

		return new MergeReportsResponse( admin , report , this.getBugEvents( id ) ,
				MergeError.values( )[ errCode - 1 ] , mergeId );
	}


	@Override
	public GetSnapshotResponse getSnapshot( Administrator admin , long bugId )
	{
		String snapshot = this.bugsDao.getSnapshot( bugId );
		if ( snapshot == null ) {
			return new GetSnapshotResponse( admin , false );
		}
		return new GetSnapshotResponse( admin , snapshot );
	}
}
