package com.deepclone.lw.beans.bt;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.bt.data.*;
import com.deepclone.lw.interfaces.bt.BugsDAO;
import com.deepclone.lw.utils.StoredProc;



public class BugsDAOBean
		implements BugsDAO
{

	private JdbcTemplate dTemplate;

	private final RowMapper< BugReport > mBugReport;
	private final RowMapper< BugEvent > mBugEvent;
	private final RowMapper< String > mString;

	private StoredProc fPostUserReport;
	private StoredProc fPostAdminReport;
	private StoredProc fPostUserComment;
	private StoredProc fPostAdminComment;
	private StoredProc fShowComment;
	private StoredProc fDeleteComment;
	private StoredProc fValidateReport;
	private StoredProc fSetStatus;
	private StoredProc fToggleVisibility;
	private StoredProc fMerge;


	public BugsDAOBean( )
	{
		this.mBugReport = new RowMapper< BugReport >( ) {
			@Override
			public BugReport mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BugSubmitter sub;
				BugReport br = new BugReport( );

				br.setReportId( rs.getLong( "bug_report_id" ) );
				br.setTitle( rs.getString( "title" ) );
				br.setPosted( rs.getTimestamp( "posted" ) );
				br.setVisible( rs.getBoolean( "visible" ) );
				br.setStatus( BugStatus.valueOf( rs.getString( "status" ) ) );
				br.setLastUpdate( rs.getTimestamp( "last_update" ) );
				br.setUpdated( rs.getBoolean( "updated" ) );

				sub = new BugSubmitter( );
				sub.setAdmin( rs.getBoolean( "initial_submitter_admin" ) );
				sub.setName( rs.getString( "initial_submitter_name" ) );
				sub.setUserId( (Integer) rs.getObject( "initial_submitter_uid" ) );
				br.setInitialSubmitter( sub );

				sub = new BugSubmitter( );
				sub.setAdmin( rs.getBoolean( "last_submitter_admin" ) );
				sub.setName( rs.getString( "last_submitter_name" ) );
				sub.setUserId( (Integer) rs.getObject( "last_submitter_uid" ) );
				br.setLatestSubmitter( sub );

				return br;
			}
		};
		this.mBugEvent = new RowMapper< BugEvent >( ) {
			@Override
			public BugEvent mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BugSubmitter sub;
				BugEvent be = new BugEvent( );

				be.setId( rs.getLong( "event_id" ) );
				be.setType( BugEventType.valueOf( rs.getString( "event_type" ) ) );
				be.setTimestamp( rs.getTimestamp( "event_timestamp" ) );
				be.setTitle( rs.getString( "title" ) );
				be.setContents( rs.getString( "contents" ) );
				BugEventType type = be.getType( );
				switch ( type ) {
					case STATUS:
						be.setStatus( BugStatus.valueOf( rs.getString( "status" ) ) );
						break;

					case COMMENT:
					case VISIBILITY:
					case INIT:
						be.setVisible( rs.getBoolean( "visible" ) );
						break;

					case MERGE:
						be.setMergedId( rs.getLong( "merged_report_id" ) );
						break;
				}

				sub = new BugSubmitter( );
				sub.setAdmin( rs.getBoolean( "submitter_admin" ) );
				sub.setName( rs.getString( "submitter_name" ) );
				sub.setUserId( (Integer) rs.getObject( "submitter_uid" ) );
				be.setSubmitter( sub );

				return be;
			}
		};
		this.mString = new RowMapper< String >( ) {
			@Override
			public String mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return rs.getString( 1 );
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fPostUserReport = new StoredProc( dataSource , "bugs" , "post_player_report" );
		this.fPostUserReport.addParameter( "empire_id" , Types.INTEGER );
		this.fPostUserReport.addParameter( "title" , Types.VARCHAR );
		this.fPostUserReport.addParameter( "description" , Types.VARCHAR );
		this.fPostUserReport.addParameter( "extra" , Types.VARCHAR );
		this.fPostUserReport.addOutput( "report_id" , Types.BIGINT );
		this.fPostUserReport.addOutput( "group_id" , Types.BIGINT );

		this.fPostAdminReport = new StoredProc( dataSource , "bugs" , "post_admin_report" );
		this.fPostAdminReport.addParameter( "admin_id" , Types.INTEGER );
		this.fPostAdminReport.addParameter( "title" , Types.VARCHAR );
		this.fPostAdminReport.addParameter( "description" , Types.VARCHAR );
		this.fPostAdminReport.addParameter( "public_report" , Types.BOOLEAN );
		this.fPostAdminReport.addOutput( "report_id" , Types.BIGINT );
		this.fPostAdminReport.addOutput( "group_id" , Types.BIGINT );

		this.fPostUserComment = new StoredProc( dataSource , "bugs" , "post_player_comment" );
		this.fPostUserComment.addParameter( "empire_id" , Types.INTEGER );
		this.fPostUserComment.addParameter( "report_id" , Types.BIGINT );
		this.fPostUserComment.addParameter( "comment" , Types.VARCHAR );

		this.fPostAdminComment = new StoredProc( dataSource , "bugs" , "post_admin_comment" );
		this.fPostAdminComment.addParameter( "admin_id" , Types.INTEGER );
		this.fPostAdminComment.addParameter( "report_id" , Types.BIGINT );
		this.fPostAdminComment.addParameter( "comment" , Types.VARCHAR );
		this.fPostAdminComment.addParameter( "public_comment" , Types.BOOLEAN );

		this.fShowComment = new StoredProc( dataSource , "bugs" , "show_comment" );
		this.fShowComment.addParameter( "admin_id" , Types.INTEGER );
		this.fShowComment.addParameter( "comment_id" , Types.BIGINT );

		this.fDeleteComment = new StoredProc( dataSource , "bugs" , "delete_comment" );
		this.fDeleteComment.addParameter( "admin_id" , Types.INTEGER );
		this.fDeleteComment.addParameter( "comment_id" , Types.BIGINT );

		this.fValidateReport = new StoredProc( dataSource , "bugs" , "validate_report" );
		this.fValidateReport.addParameter( "admin_id" , Types.INTEGER );
		this.fValidateReport.addParameter( "report_id" , Types.BIGINT );
		this.fValidateReport.addParameter( "new_status" , "bug_status_type" );
		this.fValidateReport.addParameter( "public_report" , Types.BOOLEAN );
		this.fValidateReport.addParameter( "grant_credits" , Types.INTEGER );
		this.fValidateReport.addParameter( "keep_snapshot" , Types.BOOLEAN );

		this.fSetStatus = new StoredProc( dataSource , "bugs" , "set_report_status" );
		this.fSetStatus.addParameter( "admin_id" , Types.INTEGER );
		this.fSetStatus.addParameter( "report_id" , Types.BIGINT );
		this.fSetStatus.addParameter( "new_status" , "bug_status_type" );

		this.fToggleVisibility = new StoredProc( dataSource , "bugs" , "toggle_report_visibility" );
		this.fToggleVisibility.addParameter( "admin_id" , Types.INTEGER );
		this.fToggleVisibility.addParameter( "report_id" , Types.BIGINT );

		this.fMerge = new StoredProc( dataSource , "bugs" , "merge_reports" );
		this.fMerge.addParameter( "admin_id" , Types.INTEGER );
		this.fMerge.addParameter( "report1_id" , Types.BIGINT );
		this.fMerge.addParameter( "report2_id" , Types.BIGINT );
		this.fMerge.addOutput( "err_code" , Types.INTEGER );
	}


	@Override
	public long countReports( int empireId , BugStatus status , boolean ownOnly )
	{
		StringBuilder builder = new StringBuilder( );
		List< Object > qData = new ArrayList< Object >( );
		qData.add( (Integer) empireId );

		builder.append( "SELECT count(*) FROM bugs.br_user_view WHERE empire_id = ?" );
		this.addQueryParameters( builder , qData , status , ownOnly );

		Object[] args = qData.toArray( );
		return this.dTemplate.queryForLong( builder.toString( ) , args );
	}


	@Override
	public long countReports( Administrator admin , BugStatus status , boolean ownOnly )
	{
		StringBuilder builder = new StringBuilder( );
		List< Object > qData = new ArrayList< Object >( );
		qData.add( (Integer) admin.getId( ) );

		builder.append( "SELECT count(*) FROM bugs.br_admin_view WHERE administrator_id = ?" );
		this.addQueryParameters( builder , qData , status , ownOnly );

		Object[] args = qData.toArray( );
		return this.dTemplate.queryForLong( builder.toString( ) , args );
	}


	@Override
	public long countUpdatedReports( Administrator admin )
	{
		String sql = "SELECT count(*) FROM bugs.br_admin_view WHERE administrator_id = ? AND updated";
		return this.dTemplate.queryForLong( sql , admin.getId( ) );
	}


	@Override
	public List< BugReport > getReports( int empireId , BugStatus status , boolean ownOnly , long first , int count )
	{
		StringBuilder builder = new StringBuilder( ).append( "SELECT * FROM bugs.br_user_view WHERE empire_id = ?" );
		List< Object > qData = new ArrayList< Object >( );
		qData.add( (Integer) empireId );

		this.addQueryParameters( builder , qData , status , ownOnly );
		this.addWindowParameters( builder , qData , first , count );

		Object[] args = qData.toArray( );
		return this.dTemplate.query( builder.toString( ) , this.mBugReport , args );
	}


	@Override
	public List< BugReport > getReports( Administrator admin , BugStatus status , boolean ownOnly , long first ,
			int count )
	{
		StringBuilder builder = new StringBuilder( );
		List< Object > qData = new ArrayList< Object >( );
		qData.add( (Integer) admin.getId( ) );

		builder.append( "SELECT * FROM bugs.br_admin_view WHERE administrator_id = ?" );
		this.addQueryParameters( builder , qData , status , ownOnly );
		this.addWindowParameters( builder , qData , first , count );

		Object[] args = qData.toArray( );
		return this.dTemplate.query( builder.toString( ) , this.mBugReport , args );
	}


	private void addQueryParameters( StringBuilder builder , List< Object > qData , BugStatus status , boolean ownOnly )
	{
		if ( status != null ) {
			builder.append( " AND status = ?" );
			qData.add( status.toString( ) );
		}
		if ( ownOnly ) {
			builder.append( " AND own_report" );
		}
	}


	private void addWindowParameters( StringBuilder builder , List< Object > qData , long first , int count )
	{
		builder.append( " ORDER BY last_update DESC , bug_report_id DESC OFFSET ? LIMIT ?" );
		qData.add( (Long) first );
		qData.add( (Integer) count );
	}


	@Override
	public BugReport getReport( int empireId , long reportId )
	{
		return this.getReport( "player" , empireId , reportId );
	}


	@Override
	public BugReport getReport( Administrator admin , long reportId )
	{
		return this.getReport( "admin" , admin.getId( ) , reportId );
	}


	private BugReport getReport( String viewerType , int viewerId , long reportId )
	{
		String sql = "SELECT * FROM bugs.read_" + viewerType + "_report( ? , ? )";
		try {
			return this.dTemplate.queryForObject( sql , this.mBugReport , viewerId , reportId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public long postReport( int empireId , String title , String contents , String extra )
	{
		return (Long) this.fPostUserReport.execute( empireId , title , contents , extra ).get( "report_id" );
	}


	@Override
	public long postReport( Administrator admin , String title , String contents , boolean publicReport )
	{
		return (Long) this.fPostAdminReport.execute( admin.getId( ) , title , contents , publicReport ).get(
				"report_id" );
	}


	@Override
	public void postComment( int empireId , long reportId , String comment )
	{
		this.fPostUserComment.execute( empireId , reportId , comment );
	}


	@Override
	public void postComment( Administrator admin , long reportId , String comment , boolean publicComment )
	{
		this.fPostAdminComment.execute( admin.getId( ) , reportId , comment , publicComment );
	}


	@Override
	public List< BugEvent > getEvents( long bugId )
	{
		String sql = "SELECT * FROM bugs.br_events WHERE bug_report_id = ?";
		return this.dTemplate.query( sql , this.mBugEvent , bugId );
	}


	@Override
	public void showComment( Administrator admin , long commentId )
	{
		this.fShowComment.execute( admin.getId( ) , commentId );
	}


	@Override
	public void deleteComment( Administrator admin , long commentId )
	{
		this.fDeleteComment.execute( admin.getId( ) , commentId );
	}


	@Override
	public void validateReport( Administrator admin , long bugId , BugStatus status , boolean publicReport ,
			int credits , boolean snapshot )
	{
		this.fValidateReport.execute( admin.getId( ) , bugId , status.toString( ) , publicReport , credits , snapshot );
	}


	@Override
	public void setStatus( Administrator admin , long bugId , BugStatus status )
	{
		this.fSetStatus.execute( admin.getId( ) , bugId , status.toString( ) );
	}


	@Override
	public void toggleVisibility( Administrator admin , long bugId )
	{
		this.fToggleVisibility.execute( admin.getId( ) , bugId );
	}


	@Override
	public int mergeReports( Administrator admin , long id , long mergeId )
	{
		return (Integer) this.fMerge.execute( admin.getId( ) , id , mergeId ).get( "err_code" );
	}


	@Override
	public String getSnapshot( long bugId )
	{
		String sql = "SELECT account_status FROM bugs.account_status_data WHERE event_id = ?";
		return this.dTemplate.queryForObject( sql , this.mString , bugId );
	}
}
