package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.BuildingsInformation;



/**
 * Building information row mapper
 * 
 * <p>
 * Map rows from <code>bugs.dump_buildings_view</code> into {@link BuildingsInformation} instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class BuildingsInformationMapper
		implements RowMapper< BuildingsInformation >
{

	/**
	 * Map a row from <code>bugs.dump_buildings_view</code>
	 * 
	 * <p>
	 * Generate and initialise a {@link BuildingsInformation} instance using the row's fields.
	 */
	@Override
	public BuildingsInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		BuildingsInformation bi = new BuildingsInformation( );
		bi.setPlanetId( rs.getInt( "planet_id" ) );
		bi.setId( rs.getInt( "building_id" ) );
		bi.setName( rs.getString( "building_name" ) );
		bi.setAmount( rs.getInt( "amount" ) );
		bi.setDamage( rs.getDouble( "damage" ) );
		return bi;
	}
}