package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.EmpireResourceInformation;



/**
 * Empire resource row mapper
 * 
 * <p>
 * This class is responsible for transforming rows from <code>bugs.dump_emp_resources_view</code>
 * into {@link EmpireResourceInformation} instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
final class EmpireResourceInformationMapper
		implements RowMapper< EmpireResourceInformation >
{

	/**
	 * Map a row from <code>bugs.dump_emp_resources_view</code>
	 * 
	 * <p>
	 * Create a new {@link EmpireResourceInformation} instance and set its fields using the row's
	 * contents. If a mining priority is present, set it, otherwise leave the field to its default
	 * <code>null</code> value.
	 */
	@Override
	public EmpireResourceInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		EmpireResourceInformation empRes = new EmpireResourceInformation( );

		empRes.setResource( rs.getString( "resource_name" ) );
		empRes.setOwed( rs.getDouble( "empres_owed" ) );
		empRes.setPossessed( rs.getDouble( "empres_possessed" ) );

		int priority = rs.getInt( "mining_priority" );
		if ( !rs.wasNull( ) ) {
			empRes.setMiningPriority( priority );
		}

		return empRes;
	}

}
