package com.deepclone.lw.beans.bt.es;


import com.deepclone.lw.beans.bt.es.data.ResourceDeltaInformation;
import com.deepclone.lw.beans.bt.es.data.ResourceProviderInformation;



/**
 * Planet resource row information
 * 
 * <p>
 * Planet resources and resource providers are both extracted from the same
 * <code>bugs.planet_resources_view</code> rows. Because of that, this class is used to transmit the
 * information (as a {@link ResourceDeltaInformation} instance and an optional
 * {@link ResourceProviderInformation} instance) from the row mapper to the summary creation
 * component.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class PlanetResourceRow
{

	/** Identifier of the planet */
	private final int planetId;

	/** Resource delta information */
	private final ResourceDeltaInformation delta = new ResourceDeltaInformation( );

	/** Resource provider information */
	private ResourceProviderInformation provider;


	/**
	 * Initialise the instance by setting the planet identifier
	 * 
	 * @param planetId
	 *            the planet's identifier
	 */
	public PlanetResourceRow( int planetId )
	{
		this.planetId = planetId;
	}


	/** @return the planet's identifier */
	public int getPlanetId( )
	{
		return this.planetId;
	}


	/** @return the resource delta information record */
	public ResourceDeltaInformation getDelta( )
	{
		return this.delta;
	}


	/** @return the resource provider information record */
	public ResourceProviderInformation getProvider( )
	{
		return this.provider;
	}


	/**
	 * Set the resource provider information record
	 * 
	 * @param provider
	 *            the resource provider information record
	 */
	public void setProvider( ResourceProviderInformation provider )
	{
		this.provider = provider;
	}


	/**
	 * Set the name of the resource
	 * 
	 * <p>
	 * This method should be called once the rest of the record(s) has been initialised. It will set
	 * the resource name on both the delta and the provider (if the latter exists).
	 * 
	 * @param resource
	 *            the name of the resource
	 */
	public void setResource( String resource )
	{
		this.delta.setResource( resource );
		if ( this.provider != null ) {
			this.provider.setResource( resource );
		}
	}
}
