package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.postgresql.util.PGobject;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.ResearchGraphInformation;
import com.deepclone.lw.beans.bt.es.data.ResearchInformation;
import com.deepclone.lw.beans.bt.es.data.TechnologyState;



/**
 * Research information row mapper
 * 
 * <p>
 * This class maps rows from <code>bugs.dump_research_view</code> into
 * {@link ResearchGraphInformation} instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
final class ResearchInformationMapper
		implements RowMapper< ResearchInformation >
{

	/**
	 * Map a <code>bugs.dump_research_view</code> row
	 * 
	 * <p>
	 * Create a {@link ResearchGraphInformation} instance from the row's contents.
	 */
	@Override
	public ResearchInformation mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		ResearchGraphInformation ri = new ResearchGraphInformation( );
		ri.setName( rs.getString( "name" ) );
		ri.setState( TechnologyState.valueOf( ( (PGobject) rs.getObject( "state" ) ).toString( ) ) );
		if ( ri.getState( ) == TechnologyState.RESEARCH ) {
			ri.setPoints( rs.getDouble( "points" ) );
			ri.setPriority( rs.getInt( "priority" ) );
		}
		return ri;
	}
}