package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.beans.bt.es.data.ResourceProviderInformation;



/**
 * Mapper which extract planet resources and resource providers information
 * 
 * <p>
 * This class maps rows from <code>bugs.dump_planet_resources_view</code> into
 * {@link PlanetResourceRow} instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
class ResourceRowMapper
		implements RowMapper< PlanetResourceRow >
{

	/**
	 * Map a row from <code>bugs.dump_planet_resources_view</code>
	 * 
	 * <p>
	 * Generate the {@link PlanetResourceRow} instance with the correct planet identifier, resource
	 * name, income and upkeep. If there is also a resource provider, attach a
	 * {@link ResourceProviderInformation} instance to the result, and set its mining priority if
	 * one exists.
	 */
	@Override
	public PlanetResourceRow mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		PlanetResourceRow row = new PlanetResourceRow( rs.getInt( "planet_id" ) );

		row.getDelta( ).setIncome( rs.getDouble( "pres_income" ) );
		row.getDelta( ).setUpkeep( rs.getDouble( "pres_upkeep" ) );

		double pCapacity = rs.getDouble( "resprov_quantity_max" );
		if ( !rs.wasNull( ) ) {
			ResourceProviderInformation rpi = new ResourceProviderInformation( );
			rpi.setMaximalQuantity( pCapacity );
			rpi.setCurrentQuantity( rs.getDouble( "resprov_quantity" ) );
			rpi.setDifficulty( rs.getDouble( "resprov_difficulty" ) );
			rpi.setRecovery( rs.getDouble( "resprov_recovery" ) );

			int miningPriority = rs.getInt( "mining_priority" );
			if ( !rs.wasNull( ) ) {
				rpi.setMiningPriority( miningPriority );
			}

			row.setProvider( rpi );
		}

		row.setResource( rs.getString( "resource_name" ) );
		return row;
	}

}
