package com.deepclone.lw.beans.bt.es.data;


import java.io.Serializable;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Empire resources XML dump record
 * 
 * <p>
 * This class is used to store information about an empire's resources in debugging XML dumps. Each
 * instance indicates the amount possessed or owed for a given type of resources, as well as the
 * mining priority if it is a natural resource.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@XStreamAlias( "resource" )
@SuppressWarnings( "serial" )
public class EmpireResourceInformation
		implements Serializable
{

	/** The type of resources */
	@XStreamAlias( "id" )
	@XStreamAsAttribute
	private String resource;

	/** The amount of resources possessed */
	@XStreamAsAttribute
	private Double possessed;

	/** The amount of resources owed */
	@XStreamAsAttribute
	private Double owed;

	/**
	 * The mining priority for this type of resource, or <code>null</code> if it is not a natural
	 * resource
	 */
	@XStreamAlias( "mining-priority" )
	@XStreamAsAttribute
	private Integer miningPriority;


	/** @return the type of resources */
	public String getResource( )
	{
		return this.resource;
	}


	/**
	 * Set the type of resources
	 * 
	 * @param resource
	 *            the type of resources
	 * 
	 * @throws InvalidDumpContentsException
	 *             if the specified resource type is <code>null</code>
	 */
	public void setResource( String resource )
			throws InvalidDumpContentsException
	{
		if ( resource == null ) {
			throw new InvalidDumpContentsException( this.getClass( ) , "resource" );
		}
		this.resource = resource;
	}


	/** @return the amount of resources possessed */
	public Double getPossessed( )
	{
		return this.possessed;
	}


	/**
	 * Set the amount of resources possessed
	 * 
	 * @param possessed
	 *            the amount of resources possessed
	 */
	public void setPossessed( Double possessed )
	{
		this.possessed = possessed;
	}


	/** @return the amount of resources owed */
	public Double getOwed( )
	{
		return this.owed;
	}


	/**
	 * Set the amount of resources owed
	 * 
	 * @param owed
	 *            the amount of resources owed
	 */
	public void setOwed( Double owed )
	{
		this.owed = owed;
	}


	/**
	 * @return the mining priority for this type of resource, or <code>null</code> if it is not a
	 *         natural resource
	 */
	public Integer getMiningPriority( )
	{
		return this.miningPriority;
	}


	/**
	 * Set the mining priority for this type of resource
	 * 
	 * @param miningPriority
	 *            the mining priority for this type of resource
	 */
	public void setMiningPriority( int miningPriority )
	{
		this.miningPriority = miningPriority;
	}

}
