package com.deepclone.lw.beans.bt.es.data;


import java.io.Serializable;



/**
 * Invalid dump contents exception
 * 
 * <p>
 * This runtime exception is thrown by the debugging data record classes when one of the fields is
 * set to an incorrect value.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
public final class InvalidDumpContentsException
		extends RuntimeException
{
	/** Serialisation identifier */
	private static final long serialVersionUID = 1L;

	/** Class in which the error was caused */
	private final Class< ? extends Serializable > recordType;

	/** Name of the field to which invalid contents were assigned */
	private final String field;


	/**
	 * Initialise the exception by setting the record and field
	 * 
	 * @param type
	 *            the type of the XML dump record
	 * @param field
	 *            the field to which invalid contents were assigned
	 */
	InvalidDumpContentsException( Class< ? extends Serializable > type , String field )
	{
		super( "Invalid contents for field " + field + " of record " + type.getSimpleName( ) );
		this.recordType = type;
		this.field = field;
	}


	/** @return the type of the XML dump record */
	public Class< ? extends Serializable > getRecordType( )
	{
		return this.recordType;
	}


	/** @return the field to which invalid contents were assigned */
	public String getField( )
	{
		return this.field;
	}

}
