package com.deepclone.lw.beans.bt.es.data;


import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;



/**
 * Technology from the technology graph and associated state information
 * 
 * <p>
 * This class represents the state of a technology from the B6M2+ technology graph for the empire
 * being dumped.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
@SuppressWarnings( "serial" )
@XStreamAlias( "technology" )
public class ResearchGraphInformation
		extends ResearchInformation
{

	/** The technology's name */
	@XStreamAsAttribute
	private String name;

	/** The technology's state */
	@XStreamAsAttribute
	private TechnologyState state;

	/**
	 * The amount of accumulated research points, or <code>null</code> if the technology has been
	 * researched
	 */
	@XStreamAsAttribute
	private Double points;

	/** The research priority, or <code>null</code> if the technology has been researched */
	@XStreamAsAttribute
	private Integer priority;


	/**
	 * Gets the technology's name.
	 * 
	 * @return the technology's name
	 */
	public String getName( )
	{
		return this.name;
	}


	/**
	 * Sets the technology's name.
	 * 
	 * @param name
	 *            the technology's new name
	 */
	public void setName( String name )
	{
		this.name = name;
	}


	/**
	 * Gets the technology's state.
	 * 
	 * @return the technology's state
	 */
	public TechnologyState getState( )
	{
		return this.state;
	}


	/**
	 * Sets the technology's state.
	 * 
	 * @param state
	 *            the technology's new state
	 */
	public void setState( TechnologyState state )
	{
		this.state = state;
	}


	/**
	 * Gets the amount of accumulated research points
	 * 
	 * @return the amount of accumulated research points, or <code>null</code> if the technology has
	 *         been researched
	 */
	public Double getPoints( )
	{
		return this.points;
	}


	/**
	 * Sets the amount of accumulated research points
	 * 
	 * @param points
	 *            the amount of accumulated research points
	 */
	public void setPoints( Double points )
	{
		this.points = points;
	}


	/**
	 * Gets the research priority
	 * 
	 * @return the research priority, or <code>null</code> if the technology has been researched
	 */
	public Integer getPriority( )
	{
		return this.priority;
	}


	/**
	 * Sets the research priority
	 * 
	 * @param priority
	 *            the new research priority
	 */
	public void setPriority( Integer priority )
	{
		this.priority = priority;
	}

}
