package com.deepclone.lw.beans.eventlog;


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.interfaces.eventlog.LogWriter;
import com.deepclone.lw.interfaces.eventlog.Logger;
import com.deepclone.lw.interfaces.eventlog.SystemLogger;
import com.deepclone.lw.sqld.sys.SystemLogEntry;



/**
 * The logger bean can be used to generate the various types of log entry generators. It
 * communicates with a log writer to which entries produced by the generators will be flushed.
 * 
 * @author tseeker
 * 
 */
public class LoggerBean
		implements Logger
{

	/** The log writer bean this logger pushes entries to */
	private LogWriter logWriter;


	/**
	 * Sets the log writer bean (DI)
	 * 
	 * @param logWriter
	 *            the log writer bean to push entries to
	 */
	@Autowired( required = true )
	public void setLogWriter( LogWriter logWriter )
	{
		this.logWriter = logWriter;
	}


	/**
	 * Pushes a set of log entries to the log writer.
	 * 
	 * @param entries
	 *            the list of log entries to push
	 */
	void flush( List< SystemLogEntry > entries )
	{
		this.logWriter.addEntries( entries );
	}


	/* Documentation in Logger interface */
	@Override
	public SystemLogger getSystemLogger( String component )
	{
		return new SystemLoggerImpl( this , component );
	}
}
