package com.deepclone.lw.beans.i18n;


import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;



/**
 * A language store encapsulates all data that defines a language - from the language definition
 * itself to the various translations available in this language.
 * 
 * @author tseeker
 */
class LanguageStore
		implements Serializable
{

	private static final long serialVersionUID = 1L;

	private final String languageId;
	
	private String languageName;
	
	private final Map<String,String> translations = new HashMap< String , String >( );


	public LanguageStore( String languageId , String languageName )
	{
		this.languageId = languageId;
		this.languageName = languageName;
	}


	/** @return the store's language identifier */
	String getLanguageIdentifier( )
	{
		return this.languageId;
	}


	/** @return the name of the language encapsulated by the store */
	String getLanguageName( )
	{
		return this.languageName;
	}


	/** @return the amount of translations available from the store */
	int getTranslationsCount( )
	{
		return this.translations.size( );
	}


	/**
	 * @param string
	 *            a string identifier
	 * @return the translated string
	 */
	String getTranslation( String string )
	{
		return this.translations.get( string );
	}


	void addTranslation( String stringId , String translation )
	{
		this.translations.put( stringId , translation );
	}


	void setLanguageName( String name )
	{
		this.languageName = name;
	}

}
