package com.deepclone.lw.beans.naming;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.admin.naming.Name;
import com.deepclone.lw.cmd.admin.naming.NameType;
import com.deepclone.lw.interfaces.naming.NamingDAO;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.utils.StoredProc;



public class NamingDAOBean
		implements NamingDAO
{
	private static final String sCountNames = "SELECT * FROM naming.names_status_view ORDER BY status";

	private static class CountRow
	{
		String status;
		long count;
	}

	private JdbcTemplate dTemplate;

	private StoredProc fGetEmpire;
	private StoredProc fRenamePlanet;
	private StoredProc fValidateMapName;
	private StoredProc fAllowMapRename;
	private StoredProc fRejectMapName;
	private StoredProc fRejectEmpireName;
	private StoredProc fRejectAllianceName;

	private final RowMapper< Name > mName;
	private final RowMapper< CountRow > mCount;


	public NamingDAOBean( )
	{
		this.mName = new RowMapper< Name >( ) {
			@Override
			public Name mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new Name( rs.getInt( "id" ) , rs.getString( "name" ) , rs.getString( "extra" ) , NameType
						.valueOf( rs.getString( "status" ) ) );
			}
		};
		this.mCount = new RowMapper< CountRow >( ) {
			@Override
			public CountRow mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				CountRow cRow = new CountRow( );
				cRow.count = rs.getLong( "count" );
				cRow.status = rs.getString( "status" );
				return cRow;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fGetEmpire = new StoredProc( dataSource , "emp" , "get_current" );
		this.fGetEmpire.addParameter( "account_id" , Types.INTEGER );
		this.fGetEmpire.addOutput( "empire_id" , Types.INTEGER );

		this.fRenamePlanet = new StoredProc( dataSource , "verse" , "rename_planet" );
		this.fRenamePlanet.addParameter( "planet_id" , Types.INTEGER );
		this.fRenamePlanet.addParameter( "new_name" , Types.VARCHAR );
		this.fRenamePlanet.addOutput( "err_code" , Types.INTEGER );

		this.fValidateMapName = new StoredProc( dataSource , "naming" , "validate_map_name" );
		this.fValidateMapName.addParameter( "admin_id" , Types.INTEGER );
		this.fValidateMapName.addParameter( "name_id" , Types.INTEGER );

		this.fAllowMapRename = new StoredProc( dataSource , "naming" , "allow_map_name_change" );
		this.fAllowMapRename.addParameter( "admin_id" , Types.INTEGER );
		this.fAllowMapRename.addParameter( "name_id" , Types.INTEGER );

		this.fRejectMapName = new StoredProc( dataSource , "naming" , "reject_map_name" );
		this.fRejectMapName.addParameter( "admin_id" , Types.INTEGER );
		this.fRejectMapName.addParameter( "name_id" , Types.INTEGER );
		this.fRejectMapName.addParameter( "ban_name" , Types.BOOLEAN );

		this.fRejectEmpireName = new StoredProc( dataSource , "naming" , "reject_empire_name" );
		this.fRejectEmpireName.addParameter( "admin_id" , Types.INTEGER );
		this.fRejectEmpireName.addParameter( "name_id" , Types.INTEGER );
		this.fRejectEmpireName.addParameter( "ban_name" , Types.BOOLEAN );

		this.fRejectAllianceName = new StoredProc( dataSource , "naming" , "reject_alliance_name" );
		this.fRejectAllianceName.addParameter( "admin_id" , Types.INTEGER );
		this.fRejectAllianceName.addParameter( "alliance_id" , Types.INTEGER );
	}


	@Override
	public List< String > getEmpireNames( int account )
	{
		String sql = "SELECT name FROM naming.empire_names WHERE owner_id = ? ORDER BY name";
		RowMapper< String > mapper = new RowMapper< String >( ) {
			@Override
			public String mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return rs.getString( "name" );
			}
		};

		return this.dTemplate.query( sql , mapper , account );
	}


	@Override
	public Integer getCurrentEmpire( Account account )
	{
		return (Integer) this.fGetEmpire.execute( account.getId( ) ).get( "empire_id" );
	}


	@Override
	public int renamePlanet( int id , String name )
	{
		return (Integer) this.fRenamePlanet.execute( id , name ).get( "err_code" );
	}


	@Override
	public Map< NameType , Long > countNames( )
	{
		Map< NameType , Long > result = new HashMap< NameType , Long >( );
		for ( NameType t : NameType.values( ) ) {
			result.put( t , 0L );
		}

		for ( CountRow qr : this.dTemplate.query( sCountNames , this.mCount ) ) {
			NameType type = NameType.valueOf( qr.status );
			result.put( type , qr.count );
		}

		result.put( NameType.MAP_CHANGED , result.get( NameType.MAP_PENDING ) + result.get( NameType.MAP_VALIDATED ) );

		return result;
	}


	@Override
	public List< Name > getNames( NameType type )
	{
		String sql = "SELECT * FROM naming.names_view WHERE status ";
		if ( type == NameType.MAP_CHANGED ) {
			sql += "IN ('MAP_PENDING','MAP_VALIDATED')";
		} else {
			sql += "= '" + type + "'";
		}
		if ( type == NameType.ALLIANCE || type == NameType.EMPIRE ) {
			sql += " ORDER BY id DESC";
		} else {
			sql += " ORDER BY name";
		}

		return this.dTemplate.query( sql , this.mName );
	}


	@Override
	public void validateMapName( int administrator , int name )
	{
		this.fValidateMapName.execute( administrator , name );
	}


	@Override
	public void allowMapNameChange( int administrator , int name )
	{
		this.fAllowMapRename.execute( administrator , name );
	}


	@Override
	public void rejectMapName( int administrator , int name , boolean ban )
	{
		this.fRejectMapName.execute( administrator , name , ban );
	}


	@Override
	public void rejectEmpireName( int administrator , int empire , boolean ban )
	{
		this.fRejectEmpireName.execute( administrator , empire , ban );
	}


	@Override
	public void rejectAllianceName( int administrator , int alliance )
	{
		this.fRejectAllianceName.execute( administrator , alliance );
	}
}
