package com.deepclone.lw.beans.game.resources;


import com.deepclone.lw.cmd.player.gdata.PlanetListResourceRecord;



/**
 * Intermediary class for planet list resource records extraction
 * 
 * <p>
 * This class is used as an intermediary to extract {@link PlanetListResourceRecord} instances from
 * <code>emp.plist_resources_view</code>. It contains the instance itself, and an additional field
 * which indicates the planet's identifier.
 * 
 * <p>
 * Instances of this class are returned by {@link PlanetListResourceRecordMapper} and then processed
 * to generate the map returned by {@link ResourcesInformationDAOBean#getPlanetListData(int)}.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
class FullPlanetListRecord
{
	/** The planet's identifier */
	private int planetId;

	/** The resource record */
	private final PlanetListResourceRecord record = new PlanetListResourceRecord( );


	/**
	 * Gets the planet's identifier.
	 * 
	 * @return the planet's identifier
	 */
	public int getPlanetId( )
	{
		return this.planetId;
	}


	/**
	 * Sets the planet's identifier.
	 * 
	 * @param planetId
	 *            the new planet's identifier
	 */
	public void setPlanetId( int planetId )
	{
		this.planetId = planetId;
	}


	/**
	 * Gets the resource record.
	 * 
	 * @return the resource record
	 */
	public PlanetListResourceRecord getRecord( )
	{
		return this.record;
	}

}
