package com.deepclone.lw.beans.game.resources;


import java.sql.Types;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.interfaces.game.resources.MiningSettingsDAO;
import com.deepclone.lw.utils.StoredProc;



/**
 * Data access component for mining settings
 * 
 * <p>
 * This component implements the methods which allow mining settings to be updated by calling the
 * appropriate stored procedures.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class MiningSettingsDAOBean
		implements MiningSettingsDAO
{
	/** <code>emp.mset_update_start(INT)</code> stored procedure */
	private StoredProc pStartEmpireUpdate;

	/** <code>emp.mset_update_start(INT,INT)</code> stored procedure */
	private StoredProc pStartPlanetUpdate;

	/** <code>emp.mset_update_set(TEXT,INT)</code> stored procedure */
	private StoredProc pSetMiningPriority;

	/** <code>emp.mset_update_apply()</code> stored procedure */
	private StoredProc pApplyUpdate;

	/** <code>emp.mset_toggle_source(INT,INT)</code> stored procedure */
	private StoredProc pToggleSource;


	/**
	 * Dependency injector that sets the data source
	 * 
	 * @param dataSource
	 *            the data source
	 */
	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.pStartEmpireUpdate = new StoredProc( dataSource , "emp" , "mset_update_start" );
		this.pStartEmpireUpdate.addParameter( "_empire" , Types.INTEGER );
		this.pStartEmpireUpdate.addOutput( "_success" , Types.BOOLEAN );

		this.pStartPlanetUpdate = new StoredProc( dataSource , "emp" , "mset_update_start" );
		this.pStartPlanetUpdate.addParameter( "_empire" , Types.INTEGER );
		this.pStartPlanetUpdate.addParameter( "_planet" , Types.INTEGER );
		this.pStartPlanetUpdate.addOutput( "_success" , Types.BOOLEAN );

		this.pSetMiningPriority = new StoredProc( dataSource , "emp" , "mset_update_set" );
		this.pSetMiningPriority.addParameter( "_resource" , Types.VARCHAR );
		this.pSetMiningPriority.addParameter( "_priority" , Types.INTEGER );
		this.pSetMiningPriority.addOutput( "_success" , Types.BOOLEAN );

		this.pApplyUpdate = new StoredProc( dataSource , "emp" , "mset_update_apply" );
		this.pApplyUpdate.addOutput( "_success" , Types.BOOLEAN );

		this.pToggleSource = new StoredProc( dataSource , "emp" , "mset_toggle_source" );
		this.pToggleSource.addParameter( "_empire" , Types.INTEGER );
		this.pToggleSource.addParameter( "_planet" , Types.INTEGER );
	}


	/* Documented in interface */
	@Override
	public boolean startUpdate( int empireId )
	{
		return (Boolean) this.pStartEmpireUpdate.execute( empireId ).get( "_success" );
	}


	/* Documented in interface */
	@Override
	public boolean startUpdate( int empire , int planet )
	{
		return (Boolean) this.pStartPlanetUpdate.execute( empire , planet ).get( "_success" );
	}


	/* Documented in interface */
	@Override
	public boolean setNewPriority( String resource , int priority )
	{
		return (Boolean) this.pSetMiningPriority.execute( resource , priority ).get( "_success" );
	}


	/* Documented in interface */
	@Override
	public boolean applyUpdate( )
	{
		return (Boolean) this.pApplyUpdate.execute( ).get( "_success" );
	}


	/* Documented in interface */
	@Override
	public void togglePlanet( int empire , int planet )
	{
		this.pToggleSource.execute( empire , planet );
	}

}
