package com.deepclone.lw.beans.game.resources;


import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.player.gdata.PlanetListResourceRecord;



/**
 * Planet list resource records mapper
 * 
 * <p>
 * This class is responsible for mapping rows from <code>emp.plist_resources_view</code> into
 * {@link FullPlanetListRecord} instances.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class PlanetListResourceRecordMapper
		implements RowMapper< FullPlanetListRecord >
{

	/**
	 * Map a row from <code>emp.plist_resources_view</code>
	 * 
	 * <p>
	 * Convert a row from <code>emp.plist_resources_view</code> into a {@link FullPlanetListRecord}
	 * instance. The resulting instance will have both its planet identifier and actual record set
	 * according to the row's contents.
	 */
	@Override
	public FullPlanetListRecord mapRow( ResultSet rs , int rowNum )
			throws SQLException
	{
		FullPlanetListRecord fullRecord = new FullPlanetListRecord( );

		fullRecord.setPlanetId( rs.getInt( "planet_id" ) );
		this.extractRecord( fullRecord.getRecord( ) , rs );

		return fullRecord;
	}


	/**
	 * Extract the contents of the actual record
	 * 
	 * <p>
	 * This method extracts the contents of the record that will be sent to the client from the
	 * result set.
	 * 
	 * @param record
	 *            the planet list resource record to update
	 * @param rs
	 *            the result set to extract information from
	 * 
	 * @throws SQLException
	 *             if a SQLException is encountered getting column values
	 */
	private void extractRecord( PlanetListResourceRecord record , ResultSet rs )
			throws SQLException
	{
		record.setName( rs.getString( "resource_name" ) );
		record.setIncome( rs.getLong( "pres_income" ) );
		record.setUpkeep( rs.getLong( "pres_upkeep" ) );
		record.setCivInvestment( rs.getLong( "civ_investment" ) );
		record.setMilInvestment( rs.getLong( "mil_investment" ) );
	}

}
