package com.deepclone.lw.beans.empire;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.player.gdata.NameIdPair;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceLeaderData;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceMemberData;
import com.deepclone.lw.cmd.player.gdata.alliance.AlliancePlanetData;
import com.deepclone.lw.cmd.player.gdata.alliance.PublicAllianceInformation;
import com.deepclone.lw.interfaces.game.AllianceDAO;
import com.deepclone.lw.sqld.game.AllianceMembership;
import com.deepclone.lw.utils.StoredProc;



public class AllianceDAOBean
		implements AllianceDAO
{

	private JdbcTemplate dTemplate;
	private StoredProc fCreateAlliance;
	private StoredProc fJoinAlliance;
	private StoredProc fCancelJoin;
	private StoredProc fLeaveAlliance;
	private StoredProc fTransferLeadership;
	private StoredProc fAcceptMembers;
	private StoredProc fRejectMembers;
	private StoredProc fKickMembers;


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fCreateAlliance = new StoredProc( dataSource , "emp" , "create_alliance" );
		this.fCreateAlliance.addParameter( "empire_id" , Types.INTEGER );
		this.fCreateAlliance.addParameter( "alliance_tag" , Types.VARCHAR );
		this.fCreateAlliance.addParameter( "alliance_name" , Types.VARCHAR );
		this.fCreateAlliance.addOutput( "alliance_id" , Types.INTEGER );

		this.fJoinAlliance = new StoredProc( dataSource , "emp" , "join_alliance" );
		this.fJoinAlliance.addParameter( "empire_id" , Types.INTEGER );
		this.fJoinAlliance.addParameter( "alliance_id" , Types.INTEGER );
		this.fJoinAlliance.addOutput( "success" , Types.BOOLEAN );

		this.fCancelJoin = new StoredProc( dataSource , "emp" , "cancel_join" );
		this.fCancelJoin.addParameter( "empire_id" , Types.INTEGER );

		this.fLeaveAlliance = new StoredProc( dataSource , "emp" , "leave_alliance" );
		this.fLeaveAlliance.addParameter( "empire_id" , Types.INTEGER );

		this.fTransferLeadership = new StoredProc( dataSource , "emp" , "transfer_leadership" );
		this.fTransferLeadership.addParameter( "empire_id" , Types.INTEGER );
		this.fTransferLeadership.addParameter( "target_id" , Types.INTEGER );

		this.fAcceptMembers = new StoredProc( dataSource , "emp" , "accept_members" );
		this.fAcceptMembers.addParameter( "empire_id" , Types.INTEGER );
		this.fAcceptMembers.addParameter( "member_ids" , "INT[]" );

		this.fRejectMembers = new StoredProc( dataSource , "emp" , "reject_members" );
		this.fRejectMembers.addParameter( "empire_id" , Types.INTEGER );
		this.fRejectMembers.addParameter( "member_ids" , "INT[]" );

		this.fKickMembers = new StoredProc( dataSource , "emp" , "kick_members" );
		this.fKickMembers.addParameter( "empire_id" , Types.INTEGER );
		this.fKickMembers.addParameter( "member_ids" , "INT[]" );
	}


	@Override
	public Integer findAlliance( String tag )
	{
		String sql = "SELECT id FROM emp.alliances_public WHERE lower( tag ) = ?";
		try {
			return this.dTemplate.queryForInt( sql , tag.toLowerCase( ) );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public AllianceMembership getAlliance( int empireId )
	{
		String sql = "SELECT alliance , pending FROM emp.alliance_membership WHERE id = ?";
		RowMapper< AllianceMembership > mapper = new RowMapper< AllianceMembership >( ) {
			@Override
			public AllianceMembership mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AllianceMembership am = new AllianceMembership( );
				am.setAllianceId( rs.getInt( "alliance" ) );
				am.setPending( rs.getBoolean( "pending" ) );
				return am;
			}
		};
		try {
			return this.dTemplate.queryForObject( sql , mapper , empireId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public Integer createAlliance( int empireId , String tag , String name )
	{
		return (Integer) this.fCreateAlliance.execute( empireId , tag , name ).get( "alliance_id" );
	}


	@Override
	public PublicAllianceInformation getPublicInformation( int allianceId )
	{
		String sql = "SELECT * FROM emp.alliances_public WHERE id = ?";
		RowMapper< PublicAllianceInformation > mapper = new RowMapper< PublicAllianceInformation >( ) {
			@Override
			public PublicAllianceInformation mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new PublicAllianceInformation( rs.getInt( "id" ) , rs.getString( "tag" ) ,
						rs.getString( "name" ) , rs.getInt( "leader_id" ) , rs.getString( "leader_name" ) , rs
								.getLong( "planets" ) );
			}
		};

		try {
			return this.dTemplate.queryForObject( sql , mapper , allianceId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	private List< NameIdPair > getMembers( int allianceId , boolean pending )
	{
		String mSql = "SELECT id , name FROM emp.alliance_membership WHERE alliance = ? AND ";
		if ( !pending ) {
			mSql += "NOT ";
		}
		mSql += "pending";
		RowMapper< NameIdPair > mMapper = new RowMapper< NameIdPair >( ) {
			@Override
			public NameIdPair mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new NameIdPair( rs.getInt( 1 ) , rs.getString( 2 ) );
			}
		};
		List< NameIdPair > members = this.dTemplate.query( mSql , mMapper , allianceId );
		return members;
	}


	private List< AlliancePlanetData > getPlanets( int allianceId )
	{
		String pSql = "SELECT * FROM emp.alliance_planets WHERE alliance = ?";
		RowMapper< AlliancePlanetData > pMapper = new RowMapper< AlliancePlanetData >( ) {
			@Override
			public AlliancePlanetData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AlliancePlanetData apd = new AlliancePlanetData( );

				apd.setId( rs.getInt( "planet_id" ) );
				apd.setName( rs.getString( "planet_name" ) );
				apd.setX( rs.getInt( "x" ) );
				apd.setY( rs.getInt( "y" ) );
				apd.setOrbit( rs.getInt( "orbit" ) );
				apd.setOwnerId( rs.getInt( "owner_id" ) );
				apd.setOwner( rs.getString( "owner_name" ) );
				
				apd.setBattle( rs.getBoolean( "battle" ) );
				apd.setAttack( rs.getLong( "attack" ) );
				apd.setDefence( rs.getLong( "defence" ) );

				return apd;
			}
		};
		return this.dTemplate.query( pSql , pMapper , allianceId );
	}


	@Override
	public AllianceMemberData getMemberData( int allianceId )
	{
		List< NameIdPair > members = this.getMembers( allianceId , false );
		List< AlliancePlanetData > planets = this.getPlanets( allianceId );
		return new AllianceMemberData( members , planets );
	}


	@Override
	public AllianceLeaderData getLeaderData( int allianceId )
	{
		return new AllianceLeaderData( this.getMembers( allianceId , true ) );
	}


	@Override
	public boolean requestJoin( int empireId , int allianceId )
	{
		return (Boolean) this.fJoinAlliance.execute( empireId , allianceId ).get( "success" );
	}


	@Override
	public void cancelJoin( int empireId )
	{
		this.fCancelJoin.execute( empireId );
	}


	@Override
	public void leave( int empireId )
	{
		this.fLeaveAlliance.execute( empireId );
	}


	@Override
	public void transferLeadership( int empireId , int toMember )
	{
		this.fTransferLeadership.execute( empireId , toMember );
	}


	private String toSQLArray( int[] members )
	{
		StringBuilder mArray = new StringBuilder( ).append( "{" );
		for ( int i = 0 ; i < members.length ; i++ ) {
			mArray.append( members[ i ] ).append( ( i == members.length - 1 ) ? '}' : ',' );
		}
		String mStr = mArray.toString( );
		return mStr;
	}


	@Override
	public void manageRequests( int empireId , boolean accept , int[] members )
	{
		StoredProc proc = accept ? this.fAcceptMembers : this.fRejectMembers;
		proc.execute( empireId , this.toSQLArray( members ) );
	}


	@Override
	public void kick( int empireId , int[] members )
	{
		this.fKickMembers.execute( empireId , this.toSQLArray( members ) );
	}

}
