package com.deepclone.lw.beans.empire;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.player.gdata.NameIdPair;
import com.deepclone.lw.cmd.player.gdata.PlanetListData;
import com.deepclone.lw.cmd.player.gdata.empire.OverviewData;
import com.deepclone.lw.interfaces.game.EmpireDAO;
import com.deepclone.lw.sqld.game.GeneralInformation;
import com.deepclone.lw.utils.StoredProc;



public class EmpireDAOBean
		implements EmpireDAO
{
	private JdbcTemplate dTemplate;
	private StoredProc fAddEmpEnemy;
	private StoredProc fAddAllEnemy;
	private StoredProc fRemoveEmpEnemy;
	private StoredProc fRemoveAllEnemy;
	private StoredProc fGetNewPlanet;

	private final PlanetListMapper mPlanetList = new PlanetListMapper( );


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );

		this.fAddEmpEnemy = new StoredProc( dataSource , "emp" , "add_enemy_empire" );
		this.fAddEmpEnemy.addParameter( "empire_id" , Types.INTEGER );
		this.fAddEmpEnemy.addParameter( "enemy_name" , Types.VARCHAR );
		this.fAddEmpEnemy.addOutput( "err_code" , Types.INTEGER );

		this.fAddAllEnemy = new StoredProc( dataSource , "emp" , "add_enemy_alliance" );
		this.fAddAllEnemy.addParameter( "empire_id" , Types.INTEGER );
		this.fAddAllEnemy.addParameter( "enemy_tag" , Types.VARCHAR );
		this.fAddAllEnemy.addOutput( "err_code" , Types.INTEGER );

		this.fRemoveEmpEnemy = new StoredProc( dataSource , "emp" , "remove_enemy_empires" );
		this.fRemoveEmpEnemy.addParameter( "empire_id" , Types.INTEGER );
		this.fRemoveEmpEnemy.addParameter( "to_remove" , "INT[]" );

		this.fRemoveAllEnemy = new StoredProc( dataSource , "emp" , "remove_enemy_alliances" );
		this.fRemoveAllEnemy.addParameter( "empire_id" , Types.INTEGER );
		this.fRemoveAllEnemy.addParameter( "to_remove" , "INT[]" );

		this.fGetNewPlanet = new StoredProc( dataSource , "emp" , "get_new_planet" );
		this.fGetNewPlanet.addParameter( "empire_id" , Types.INTEGER );
		this.fGetNewPlanet.addParameter( "planet_name" , Types.VARCHAR );
		this.fGetNewPlanet.addOutput( "err_code" , Types.INTEGER );
	}


	@Override
	public GeneralInformation getInformation( int empireId )
	{
		String sql = "SELECT * FROM emp.general_information WHERE id = ?";
		try {
			return this.dTemplate.queryForObject( sql , new GeneralInformationRowMapper( ) , empireId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
	}


	@Override
	public List< NameIdPair > getPlanets( int empireId )
	{
		String sql = "SELECT id , name FROM emp.planets_view WHERE empire = ?";
		RowMapper< NameIdPair > mapper = new RowMapper< NameIdPair >( ) {
			@Override
			public NameIdPair mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new NameIdPair( rs.getInt( "id" ) , rs.getString( "name" ) );
			}
		};
		return this.dTemplate.query( sql , mapper , empireId );
	}


	@Override
	public OverviewData getOverview( int empireId )
	{
		String sql = "SELECT * FROM emp.overview WHERE empire = ?";
		RowMapper< OverviewData > mapper = new RowMapper< OverviewData >( ) {
			@Override
			public OverviewData mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				OverviewData info = new OverviewData( );
				info.setPlanets( rs.getLong( "planets" ) );

				Double dTemp;
				dTemp = rs.getDouble( "population" );
				info.setPopulation( dTemp == null ? 0 : dTemp.longValue( ) );
				dTemp = rs.getDouble( "avg_happiness" );
				info.setAvgHappiness( dTemp == null ? 0 : dTemp.intValue( ) );
				dTemp = rs.getDouble( "planet_income" );
				info.setPlanetIncome( dTemp == null ? 0 : dTemp.longValue( ) );
				dTemp = rs.getDouble( "planet_upkeep" );
				info.setPlanetUpkeep( dTemp == null ? 0 : dTemp.longValue( ) );

				Long lTemp;
				lTemp = rs.getLong( "fleet_power" );
				info.setFleetPower( lTemp == null ? 0 : lTemp );
				lTemp = rs.getLong( "fleet_upkeep" );
				info.setFleetUpkeep( lTemp == null ? 0 : lTemp );

				Long lTemp2;
				lTemp = rs.getLong( "civ_investment" );
				lTemp2 = rs.getLong( "mil_investment" );
				info.setInvestment( ( lTemp == null ? 0 : lTemp ) + ( lTemp2 == null ? 0 : lTemp2 ) );

				info.setNewMessages( rs.getInt( "new_messages" ) );

				return info;
			}
		};
		return this.dTemplate.queryForObject( sql , mapper , empireId );
	}


	@Override
	public List< PlanetListData > getPlanetList( int empireId )
	{
		String sql = "SELECT * FROM emp.planets_list WHERE empire = ?";
		return this.dTemplate.query( sql , this.mPlanetList , empireId );
	}


	@Override
	public List< NameIdPair > getEnemies( int empireId , boolean alliances )
	{
		String sql = "SELECT id , name FROM emp.enemy_lists WHERE empire = ? AND ";
		if ( !alliances ) {
			sql += "NOT ";
		}
		sql += "alliance";

		RowMapper< NameIdPair > mapper = new RowMapper< NameIdPair >( ) {
			@Override
			public NameIdPair mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				return new NameIdPair( rs.getInt( "id" ) , rs.getString( "name" ) );
			}
		};

		return this.dTemplate.query( sql , mapper , empireId );
	}


	@Override
	public ObjectNameError addEnemy( int empireId , boolean alliance , String name )
	{
		StoredProc addProc = alliance ? this.fAddAllEnemy : this.fAddEmpEnemy;
		int rv = (Integer) addProc.execute( empireId , name ).get( "err_code" );

		switch ( rv ) {
			case 0:
				return null;
			case 1:
				return ObjectNameError.INVALID;
			case 2:
				return ObjectNameError.BANNED;
			case 3:
				return ObjectNameError.UNAVAILABLE;
			default:
				throw new RuntimeException( "unexpected error code " + rv );
		}
	}


	@Override
	public void removeEnemies( int empireId , boolean alliance , int[] ids )
	{
		StoredProc remProc = alliance ? this.fRemoveAllEnemy : this.fRemoveEmpEnemy;
		StringBuilder idArray = new StringBuilder( ).append( "{" );

		for ( int i = 0 ; i < ids.length ; i++ ) {
			idArray.append( ids[ i ] ).append( ( i == ids.length - 1 ) ? "}" : "," );
		}

		remProc.execute( empireId , idArray.toString( ) );
	}


	@Override
	public int getNewPlanet( int empireId , String name )
	{
		return (Integer) this.fGetNewPlanet.execute( empireId , name ).get( "err_code" );
	}

}
