package com.deepclone.lw.beans.empire;


import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.player.EmpireResponse;
import com.deepclone.lw.cmd.player.GetNewPlanetResponse;
import com.deepclone.lw.cmd.player.ListPlanetsResponse;
import com.deepclone.lw.cmd.player.elist.AddEnemyResponse;
import com.deepclone.lw.cmd.player.elist.EnemyListResponse;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.NameIdPair;
import com.deepclone.lw.cmd.player.gdata.PlanetListData;
import com.deepclone.lw.cmd.player.gdata.PlanetListResourceRecord;
import com.deepclone.lw.cmd.player.gdata.battles.BattleListEntry;
import com.deepclone.lw.cmd.player.gdata.empire.OverviewData;
import com.deepclone.lw.interfaces.acm.UsersDAO;
import com.deepclone.lw.interfaces.game.BattlesCache;
import com.deepclone.lw.interfaces.game.BattlesDAO;
import com.deepclone.lw.interfaces.game.EmpireDAO;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.game.resources.ResourcesInformationDAO;
import com.deepclone.lw.interfaces.naming.NamingDAO;
import com.deepclone.lw.interfaces.prefs.AccountPreferences;
import com.deepclone.lw.interfaces.prefs.PreferencesDAO;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.sqld.game.GeneralInformation;
import com.deepclone.lw.sqld.game.battle.BattleListRecord;
import com.deepclone.lw.utils.EmailAddress;



@Transactional
public class EmpireManagementBean
		implements EmpireManagement
{
	private NamingDAO namingDao;
	private UsersDAO usersDao;
	private EmpireDAO empireDao;
	private PreferencesDAO prefsDao;
	private BattlesDAO battlesDao;
	private ResourcesInformationDAO resourcesInformationDao;


	@Autowired( required = true )
	public void setUsersDao( UsersDAO usersDao )
	{
		this.usersDao = usersDao;
	}


	@Autowired( required = true )
	public void setNamingDao( NamingDAO namingDao )
	{
		this.namingDao = namingDao;
	}


	@Autowired( required = true )
	public void setEmpireDao( EmpireDAO empireDao )
	{
		this.empireDao = empireDao;
	}


	@Autowired( required = true )
	public void setPrefsDao( PreferencesDAO prefsDao )
	{
		this.prefsDao = prefsDao;
	}


	@Autowired( required = true )
	public void setBattlesDao( BattlesCache battlesDao )
	{
		this.battlesDao = battlesDao;
	}


	@Autowired( required = true )
	public void setResourcesInformationDao( ResourcesInformationDAO resourcesInformationDao )
	{
		this.resourcesInformationDao = resourcesInformationDao;
	}


	@Override
	public Integer getEmpireId( EmailAddress address )
	{
		Account acnt = usersDao.getAccount( address );
		if ( acnt == null || !acnt.getStatus( ).isActive( ) ) {
			return null;
		}
		return namingDao.getCurrentEmpire( acnt );
	}


	@Override
	public GamePageData getGeneralInformation( int empireId )
	{
		GeneralInformation generalInformation = this.empireDao.getInformation( empireId );
		List< NameIdPair > planets = this.empireDao.getPlanets( empireId );

		AccountPreferences prefs = this.prefsDao.getPreferences( generalInformation.getAccountId( ) );
		boolean rlTime = prefs.getPreference( "useRLTime" , Boolean.class );

		return new GamePageData( generalInformation.getName( ) , generalInformation.getStatus( ) ,
				generalInformation.getTag( ) , generalInformation.getCash( ) , generalInformation.getNextTick( ) ,
				planets , rlTime, generalInformation.getLanguage( ) );
	}


	@Override
	public EmpireResponse getOverview( int empireId )
	{
		OverviewData overview = this.empireDao.getOverview( empireId );

		List< BattleListEntry > battles = new LinkedList< BattleListEntry >( );
		for ( BattleListRecord record : this.battlesDao.getBattles( empireId ) ) {
			if ( record.getLastTick( ) != null ) {
				break;
			}
			BattleListEntry entry = new BattleListEntry( );
			entry.setId( record.getBattle( ) );
			entry.setLocation( new NameIdPair( record.getPlanetId( ) , record.getName( ) ) );
			entry.setX( record.getX( ) );
			entry.setY( record.getY( ) );
			entry.setOrbit( record.getOrbit( ) );
			entry.setFirst( record.getFirstTick( ) );
			if ( record.getLastTick( ) != null ) {
				entry.setLast( record.getLastTick( ) );
			}
			battles.add( entry );
		}

		overview.setEconomy( this.resourcesInformationDao.getEmpireInformation( empireId ) );

		return new EmpireResponse( this.getGeneralInformation( empireId ) , overview , battles );
	}


	@Override
	public ListPlanetsResponse getPlanetList( int empireId )
	{
		List< PlanetListData > planets = this.empireDao.getPlanetList( empireId );

		Map< Integer , List< PlanetListResourceRecord >> resources;
		resources = this.resourcesInformationDao.getPlanetListData( empireId );
		for ( PlanetListData planet : planets ) {
			List< PlanetListResourceRecord > planetResources = resources.get( planet.getId( ) );
			if ( planetResources == null ) {
				planetResources = new ArrayList< PlanetListResourceRecord >( );
			}
			planet.setResources( planetResources );
		}

		return new ListPlanetsResponse( this.getGeneralInformation( empireId ) , planets );
	}


	@Override
	public EnemyListResponse getEnemyLists( int empireId )
	{
		GamePageData page = this.getGeneralInformation( empireId );
		List< NameIdPair > empires = this.empireDao.getEnemies( empireId , false );
		List< NameIdPair > alliances = this.empireDao.getEnemies( empireId , true );
		return new EnemyListResponse( page , empires , alliances );
	}


	@Override
	public EnemyListResponse addEnemy( int empireId , boolean alliance , String name )
	{
		ObjectNameError error = this.empireDao.addEnemy( empireId , alliance , name );
		GamePageData page = this.getGeneralInformation( empireId );
		List< NameIdPair > empires = this.empireDao.getEnemies( empireId , false );
		List< NameIdPair > alliances = this.empireDao.getEnemies( empireId , true );
		if ( error == null ) {
			return new EnemyListResponse( page , empires , alliances );
		}
		return new AddEnemyResponse( page , empires , alliances , error , alliance , name );
	}


	@Override
	public EnemyListResponse removeEnemies( int empireId , boolean alliance , int[] ids )
	{
		this.empireDao.removeEnemies( empireId , alliance , ids );
		return this.getEnemyLists( empireId );
	}


	@Override
	public GetNewPlanetResponse getNewPlanet( int empireId , String name )
	{
		int errCode = this.empireDao.getNewPlanet( empireId , name );
		ObjectNameError error;
		switch ( errCode ) {
			case 0:
				error = null;
				break;
			case 1:
				error = ObjectNameError.BANNED;
				break;
			case 2:
				error = ObjectNameError.UNAVAILABLE;
				break;
			default:
				return new GetNewPlanetResponse( this.getGeneralInformation( empireId ) , null , null );
		}

		Integer planetId;
		if ( error == null ) {
			planetId = this.empireDao.getPlanetList( empireId ).get( 0 ).getId( );
		} else {
			planetId = null;
		}

		if ( planetId == null ) {
			return new GetNewPlanetResponse( this.getGeneralInformation( empireId ) , name , error );
		}

		return new GetNewPlanetResponse( this.getGeneralInformation( empireId ) , planetId );
	}
}
