package com.deepclone.lw.beans.fleets;


import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.cmd.player.battles.GetBattleResponse;
import com.deepclone.lw.cmd.player.battles.ListBattlesResponse;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.NameIdPair;
import com.deepclone.lw.cmd.player.gdata.PlanetRelationType;
import com.deepclone.lw.cmd.player.gdata.TimeCombo;
import com.deepclone.lw.cmd.player.gdata.battles.*;
import com.deepclone.lw.interfaces.game.BattleViewer;
import com.deepclone.lw.interfaces.game.BattlesCache;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.sqld.game.battle.*;



public class BattleViewerBean
		implements BattleViewer
{

	private static final int pageSize = 10;

	private EmpireManagement empireManagement;
	private BattlesCache battlesDao;


	@Autowired( required = true )
	public void setEmpireManagement( EmpireManagement empireManagement )
	{
		this.empireManagement = empireManagement;
	}


	@Autowired( required = true )
	public void setBattlesDao( BattlesCache dao )
	{
		this.battlesDao = dao;
	}


	@Override
	public GetBattleResponse getBattle( int empireId , long battleId )
	{
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		BattleView battle = this.createBattleView( empireId , battleId , null );
		return new GetBattleResponse( page , battle );
	}


	@Override
	public GetBattleResponse getBattle( int empireId , long battleId , long tick )
	{
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		BattleView battle = this.createBattleView( empireId , battleId , tick );
		return new GetBattleResponse( page , battle );
	}


	private BattleView createBattleView( int empireId , long battleId , Long tick )
	{
		EmpireBattleRecord battle = this.battlesDao.getBattleRecord( empireId , battleId );
		if ( battle == null ) {
			return null;
		}

		BattleView view = new BattleView( );
		BattleDescription bDescription = new BattleDescription( );
		BattleDisplay bDisplay = new BattleDisplay( );
		view.setDescription( bDescription );
		view.setDisplay( bDisplay );

		// Generate main battle description
		bDescription.setId( battleId );
		bDescription.setLocation( new NameIdPair( battle.getPlanetId( ) , battle.getPlanetName( ) ) );
		bDescription.setX( battle.getX( ) );
		bDescription.setY( battle.getY( ) );
		bDescription.setOrbit( battle.getOrbit( ) );

		// Generate list of ticks
		List< PresenceRecord > presence = this.battlesDao.getPresence( battle );
		Long previous = null;
		Long next = null;
		Long display = null;
		boolean planetOwner = false;
		boolean wasPresent = false;
		for ( PresenceRecord pRec : presence ) {
			if ( !pRec.isPresent( ) && !wasPresent ) {
				continue;
			}

			long prTick = pRec.getTick( );
			if ( tick == null || display == null || display < tick ) {
				previous = display;
				display = prTick;
				planetOwner = pRec.isPlanetOwner( );
			} else if ( next == null ) {
				next = prTick;
			}
			bDisplay.addTime( prTick );
			wasPresent = pRec.isPresent( );
		}
		if ( display == null && presence.size( ) == 1 ) {
			display = presence.get( 0 ).getTick( );
			bDisplay.addTime( display );
		}
		bDisplay.setCurrent( display );
		if ( previous != null ) {
			bDisplay.setPrevious( previous );
		}
		if ( next != null ) {
			bDisplay.setNext( next );
		}

		// Get the status at the current tick
		Map< Long , Map< Long , ProtagonistRecord >> protagonists = this.battlesDao.getProtagonists( battle );
		view.setShips( this.getBattleStatus( battle , display , protagonists , planetOwner ) );

		// Determine history intervals to display
		List< BattleHistoryInterval > intervals = this.getIntervals( battle , presence , display );

		// Add battle history
		Collections.reverse( intervals );
		this.addEvents( battle , intervals , protagonists , display );
		for ( BattleHistoryInterval i : intervals ) {
			view.addHistoryInterval( i );
		}

		return view;
	}


	private List< BattleHistoryInterval > getIntervals( EmpireBattleRecord battle , List< PresenceRecord > presence ,
			long display )
	{
		List< BattleHistoryInterval > intervals = new LinkedList< BattleHistoryInterval >( );
		for ( PresenceRecord pRec : presence ) {
			long prTick = pRec.getTick( );
			if ( prTick > display ) {
				break;
			}

			if ( intervals.isEmpty( ) || intervals.get( intervals.size( ) - 1 ).getEnd( ) != null ) {
				if ( !pRec.isPresent( ) ) {
					continue;
				}

				BattleHistoryInterval interval = new BattleHistoryInterval( );
				interval.setBegin( prTick , prTick == battle.getFirstTick( ) );
				intervals.add( interval );
			} else if ( !pRec.isPresent( ) ) {
				BattleHistoryInterval interval = intervals.get( intervals.size( ) - 1 );
				interval.setEnd( prTick , false );
			} else if ( battle.getLastTick( ) != null && prTick == battle.getLastTick( ) ) {
				BattleHistoryInterval interval = intervals.get( intervals.size( ) - 1 );
				interval.setEnd( prTick , true );
			}
		}
		if ( intervals.isEmpty( ) && presence.size( ) == 1 ) {
			PresenceRecord pRec = presence.get( 0 );
			BattleHistoryInterval interval = new BattleHistoryInterval( );
			interval.setBegin( pRec.getTick( ) , true );
			interval.setEnd( pRec.getTick( ) , true );
			intervals.add( interval );
		}
		return intervals;
	}


	private void addEvents( EmpireBattleRecord battle , List< BattleHistoryInterval > intervals ,
			Map< Long , Map< Long , ProtagonistRecord >> protagonists , long display )
	{
		Map< Boolean , Map< Long , List< EventItemRecord >>> eItems = this.battlesDao.getEventItems( battle );
		ListIterator< BattleHistoryInterval > bhIterator = intervals.listIterator( );
		BattleHistoryInterval cInterval = bhIterator.next( );
		BattleHistoryTick cTick = null;

		for ( EventRecord event : this.battlesDao.getEvents( battle ) ) {
			long eTick = event.getTick( );
			if ( eTick > display ) {
				continue;
			}

			// Do not add events after the end of the current interval
			TimeCombo intEnd = cInterval.getEnd( );
			if ( intEnd != null && eTick > intEnd.getTicks( ) ) {
				continue;
			}

			// If the event is before the current interval, try fetching the "next" (previous in
			// terms of ticks) interval
			if ( eTick < cInterval.getBegin( ).getTicks( ) ) {
				if ( !bhIterator.hasNext( ) ) {
					break;
				}
				cInterval = bhIterator.next( );
				cTick = null;
				if ( eTick > cInterval.getEnd( ).getTicks( ) ) {
					continue;
				}
			}

			// Create and add the event's tick if necessary
			if ( cTick == null || cTick.getTime( ).getTicks( ) != eTick ) {
				cTick = new BattleHistoryTick( eTick );
				cInterval.addEntry( cTick );
			}

			// Create the event's record
			BattleEvent bEvent = new BattleEvent( );
			bEvent.setType( BattleEventType.valueOf( event.getEventType( ) ) );
			bEvent.setPlanet( event.isPlanet( ) );
			bEvent.setName( event.getName( ) );
			if ( event.getAttack( ) != null ) {
				ProtagonistRecord self = protagonists.get( eTick ).get( battle.getProtagonistId( ) );
				if ( !event.isPlanet( ) && self.getName( ).equals( event.getName( ) ) ) {
					bEvent.setHostile( event.getAttack( ) );
				} else {
					bEvent.setHostile( self.isAttacking( ) != event.getAttack( ) );
				}
			}

			// Add event to history
			cTick.addEvent( bEvent );

			// Add ships/buildings
			if ( event.getEventId( ) == null ) {
				continue;
			}

			List< EventItemRecord > items = eItems.get( event.isPlanet( ) ).get( event.getEventId( ) );
			if ( items == null ) {
				continue;
			}
			for ( EventItemRecord eir : items ) {
				BattleShipType item = new BattleShipType( );
				item.setName( eir.getNature( ) );
				item.setcAmount( eir.getAmount( ) );
				bEvent.addShip( item );
			}
		}
	}


	private BattleShipsView getBattleStatus( EmpireBattleRecord battle , Long display ,
			Map< Long , Map< Long , ProtagonistRecord >> pHistory , boolean planetOwner )
	{
		BattleShipsView view = new BattleShipsView( );
		Map< Long , ProtagonistRecord > protagonists = pHistory.get( display );
		Map< Long , List< ShipHistoryRecord >> ships = this.battlesDao.getShipsHistory( battle ).get( display );
		boolean vMode = protagonists.get( battle.getProtagonistId( ) ).isAttacking( );

		// Handle fleets
		for ( ProtagonistRecord protagonist : protagonists.values( ) ) {
			BattlePlayerShips pShips;
			boolean isSelf = ( protagonist.getProtagonistId( ) == battle.getProtagonistId( ) );
			if ( isSelf ) {
				pShips = view.getOwn( );
			} else {
				pShips = new BattlePlayerShips( );
			}

			// Add ships
			List< ShipHistoryRecord > pShipList = ships.get( protagonist.getProtagonistId( ) );
			if ( pShipList == null ) {
				continue;
			}
			for ( ShipHistoryRecord shr : pShipList ) {
				BattleShipType shipType = new BattleShipType( );
				shipType.setName( shr.getNature( ) );
				shipType.setcAmount( shr.getCurrent( ) );
				shipType.setcPower( shr.getCurrent( ) * shr.getPower( ) );
				shipType.setlAmount( shr.getLost( ) );
				shipType.setlPower( shr.getLost( ) * shr.getPower( ) );
				pShips.addShip( shipType );
			}

			// Set empire name & identifier
			int eId = protagonist.getEmpireId( ) == null ? -1 : protagonist.getEmpireId( ).intValue( );
			pShips.setPlayer( new NameIdPair( eId , protagonist.getName( ) ) );

			// Add to the appropriate list
			if ( isSelf || pShips.getcPower( ) == 0 && pShips.getlPower( ) == 0 ) {
				continue;
			}
			BattleSideShips side = ( protagonist.isAttacking( ) == vMode ) ? view.getFriendly( ) : view.getHostile( );
			side.addPlayer( pShips );
		}

		// Handle planet
		BattlePlanetBuildings planet = view.getPlanet( );
		if ( planetOwner ) {
			planet.setRelation( PlanetRelationType.OWN );
		} else {
			planet.setRelation( vMode ? PlanetRelationType.ENEMY : PlanetRelationType.ALLIED );
		}

		List< BuildingHistoryRecord > toDisplay = this.battlesDao.getBuildingsHistory( battle ).get( display );
		if ( toDisplay != null ) {
			for ( BuildingHistoryRecord bhr : toDisplay ) {
				BattleShipType building = new BattleShipType( );
				building.setName( bhr.getNature( ) );
				building.setcAmount( bhr.getCurrent( ) );
				building.setcPower( bhr.getCurrent( ) * bhr.getPower( ) );
				building.setlAmount( bhr.getLost( ) );
				building.setlPower( bhr.getLost( ) * bhr.getPower( ) );
				planet.addShip( building );
			}
		}

		return view;
	}


	@Override
	public ListBattlesResponse getBattles( int empireId , int page )
	{
		List< BattleListRecord > records = this.battlesDao.getBattles( empireId );
		int rSize = records.size( );
		int mod = rSize % BattleViewerBean.pageSize;
		int pages = ( rSize - mod ) / BattleViewerBean.pageSize + ( mod > 0 ? 1 : 0 );
		if ( page < 0 ) {
			page = 0;
		} else if ( page >= pages ) {
			page = pages - 1;
		}

		List< BattleListEntry > entries = new LinkedList< BattleListEntry >( );
		if ( pages > 0 ) {
			records = records.subList( page * BattleViewerBean.pageSize , Math.min( ( page + 1 )
					* BattleViewerBean.pageSize , rSize ) );

			for ( BattleListRecord record : records ) {
				BattleListEntry entry = new BattleListEntry( );
				entry.setId( record.getBattle( ) );
				entry.setLocation( new NameIdPair( record.getPlanetId( ) , record.getName( ) ) );
				entry.setX( record.getX( ) );
				entry.setY( record.getY( ) );
				entry.setOrbit( record.getOrbit( ) );
				entry.setFirst( record.getFirstTick( ) );
				if ( record.getLastTick( ) != null ) {
					entry.setLast( record.getLastTick( ) );
				}
				entries.add( entry );
			}
		}

		return new ListBattlesResponse( this.empireManagement.getGeneralInformation( empireId ) , entries , page ,
				pages );
	}
}
