package com.deepclone.lw.beans.fleets;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import com.deepclone.lw.interfaces.game.BattlesDAO;
import com.deepclone.lw.sqld.game.battle.*;



public class BattlesDAOBean
		implements BattlesDAO
{

	private static final String sGetBattleRecord = "SELECT * FROM battles.empire_list_view WHERE empire = ? AND battle = ?";
	private static final String sGetPresenceList = "SELECT * FROM battles.protagonist_presence WHERE protagonist = ?";
	private static final String sGetEvents = "SELECT e.* FROM battles.events_history e WHERE e.battle = ? AND e.tick IN ( SELECT tick FROM battles.protagonist_presence WHERE protagonist = ? AND present )";
	private static final String sGetEventItems = "SELECT is_planet , event_id , nature , amount FROM battles.event_items i WHERE empire = ? AND battle = ?";
	private static final String sGetProtagonists = "SELECT * FROM battles.mode_history_view WHERE battle = ?";
	private static final String sGetBuildings = "SELECT * FROM battles.buildings_history WHERE empire = ? AND battle = ?";
	private static final String sGetShips = "SELECT * FROM battles.fleets_history WHERE empire = ? AND battle = ?";
	private static final String sGetAllBattles = "SELECT * FROM battles.battles_list WHERE empire = ? ORDER BY last_tick DESC NULLS FIRST , first_tick DESC , x , y , orbit";
	private static final String sGetNewBattles = "SELECT * FROM battles.battles_list WHERE empire = ? AND ( last_update > ? OR NOT finished ) ORDER BY last_tick DESC NULLS FIRST , first_tick DESC , x , y , orbit";

	private JdbcTemplate dTemplate;

	private final RowMapper< EmpireBattleRecord > mGetBattleRecord;
	private final RowMapper< PresenceRecord > mGetPresenceList;
	private final RowMapper< EventRecord > mGetEvents;
	private final RowMapper< EventItemRecord > mGetEventItems;
	private final RowMapper< ProtagonistRecord > mGetProtagonists;
	private final RowMapper< BuildingHistoryRecord > mGetBuildings;
	private final RowMapper< ShipHistoryRecord > mGetShips;
	private final RowMapper< BattleListRecord > mBattleListEntry;


	public BattlesDAOBean( )
	{
		this.mGetBattleRecord = new RowMapper< EmpireBattleRecord >( ) {
			@Override
			public EmpireBattleRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				EmpireBattleRecord r = new EmpireBattleRecord( );
				r.setFirstTick( rs.getLong( "first_tick" ) );
				r.setLastTick( (Long) rs.getObject( "last_tick" ) );
				r.setPlanetId( rs.getInt( "planet" ) );
				r.setPlanetName( rs.getString( "name" ) );
				r.setX( rs.getInt( "x" ) );
				r.setY( rs.getInt( "y" ) );
				r.setOrbit( rs.getInt( "orbit" ) );
				r.setProtagonistId( rs.getLong( "protagonist" ) );
				return r;
			}
		};
		this.mGetPresenceList = new RowMapper< PresenceRecord >( ) {
			@Override
			public PresenceRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				PresenceRecord pView = new PresenceRecord( );
				pView.setTick( rs.getLong( "tick" ) );
				pView.setPresent( rs.getBoolean( "present" ) );
				pView.setPlanetOwner( rs.getBoolean( "planet_owner" ) );
				return pView;
			}
		};
		this.mGetEvents = new RowMapper< EventRecord >( ) {
			@Override
			public EventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				EventRecord r = new EventRecord( );
				r.setEventType( rs.getString( "event_type" ) );
				r.setTick( rs.getLong( "tick" ) );
				r.setPlanet( rs.getBoolean( "is_planet" ) );
				r.setAttack( (Boolean) rs.getObject( "attack" ) );
				r.setEventId( (Long) rs.getObject( "event_id" ) );
				r.setName( rs.getString( "name" ) );
				return r;
			}
		};
		this.mGetEventItems = new RowMapper< EventItemRecord >( ) {
			@Override
			public EventItemRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				EventItemRecord r = new EventItemRecord( );
				r.setPlanetEvent( rs.getBoolean( "is_planet" ) );
				r.setEventId( rs.getLong( "event_id" ) );
				r.setNature( rs.getString( "nature" ) );
				r.setAmount( rs.getInt( "amount" ) );
				return r;
			}
		};
		this.mGetProtagonists = new RowMapper< ProtagonistRecord >( ) {
			@Override
			public ProtagonistRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				ProtagonistRecord p = new ProtagonistRecord( );
				p.setTick( rs.getLong( "tick" ) );
				p.setProtagonistId( rs.getLong( "protagonist" ) );
				p.setEmpireId( (Long) rs.getObject( "empire_id" ) );
				p.setName( rs.getString( "empire_name" ) );
				p.setAttacking( rs.getBoolean( "attacking" ) );
				return p;
			}
		};
		this.mGetBuildings = new RowMapper< BuildingHistoryRecord >( ) {
			@Override
			public BuildingHistoryRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BuildingHistoryRecord bhr = new BuildingHistoryRecord( );
				bhr.setTick( rs.getLong( "tick" ) );
				bhr.setNature( rs.getString( "building" ) );
				bhr.setCurrent( rs.getLong( "current" ) );
				bhr.setLost( rs.getLong( "lost" ) );
				bhr.setPower( (long) Math.floor( rs.getDouble( "power" ) ) );
				return bhr;
			}
		};
		this.mGetShips = new RowMapper< ShipHistoryRecord >( ) {
			@Override
			public ShipHistoryRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				ShipHistoryRecord shr = new ShipHistoryRecord( );
				shr.setProtagonist( rs.getLong( "protagonist" ) );
				shr.setTick( rs.getLong( "tick" ) );
				shr.setNature( rs.getString( "ship_type" ) );
				shr.setCurrent( rs.getLong( "current" ) );
				shr.setLost( rs.getLong( "lost" ) );
				shr.setPower( rs.getInt( "ship_power" ) );
				return shr;
			}
		};
		this.mBattleListEntry = new RowMapper< BattleListRecord >( ) {
			@Override
			public BattleListRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BattleListRecord entry = new BattleListRecord( );
				entry.setBattle( rs.getLong( "battle" ) );
				entry.setPlanetId( rs.getInt( "planet" ) );
				entry.setX( rs.getInt( "x" ) );
				entry.setY( rs.getInt( "y" ) );
				entry.setOrbit( rs.getInt( "orbit" ) );
				entry.setName( rs.getString( "name" ) );
				entry.setFirstTick( rs.getLong( "first_tick" ) );
				entry.setLastTick( (Long) rs.getObject( "last_tick" ) );
				entry.setLastUpdate( rs.getLong( "last_update" ) );
				entry.setFinished( rs.getBoolean( "finished" ) );
				return entry;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );
	}


	@Override
	public EmpireBattleRecord getBattleRecord( int empireId , long battleId )
	{
		EmpireBattleRecord ebv;
		try {
			ebv = this.dTemplate.queryForObject( sGetBattleRecord , this.mGetBattleRecord , empireId , battleId );
		} catch ( EmptyResultDataAccessException e ) {
			return null;
		}
		ebv.setBattleId( battleId );
		ebv.setEmpireId( empireId );
		return ebv;
	}


	@Override
	public List< PresenceRecord > getPresence( EmpireBattleRecord record )
	{
		return this.dTemplate.query( sGetPresenceList , this.mGetPresenceList , record.getProtagonistId( ) );
	}


	@Override
	public Map< Long , Map< Long , ProtagonistRecord > > getProtagonists( EmpireBattleRecord record )
	{
		List< ProtagonistRecord > all;
		all = this.dTemplate.query( sGetProtagonists , this.mGetProtagonists , record.getBattleId( ) );

		Map< Long , Map< Long , ProtagonistRecord >> result = new HashMap< Long , Map< Long , ProtagonistRecord > >( );
		for ( ProtagonistRecord r : all ) {
			Map< Long , ProtagonistRecord > atTick = result.get( r.getTick( ) );
			if ( atTick == null ) {
				atTick = new HashMap< Long , ProtagonistRecord >( );
				result.put( r.getTick( ) , atTick );
			}
			atTick.put( r.getProtagonistId( ) , r );
		}
		return result;
	}


	@Override
	public List< EventRecord > getEvents( EmpireBattleRecord record )
	{
		return this.dTemplate.query( sGetEvents , this.mGetEvents , record.getBattleId( ) , record.getProtagonistId( ) );
	}


	@Override
	public Map< Boolean , Map< Long , List< EventItemRecord >>> getEventItems( EmpireBattleRecord record )
	{
		List< EventItemRecord > raw;
		raw = this.dTemplate
				.query( sGetEventItems , this.mGetEventItems , record.getEmpireId( ) , record.getBattleId( ) );

		Map< Boolean , Map< Long , List< EventItemRecord >>> result = new HashMap< Boolean , Map< Long , List< EventItemRecord >> >( );
		result.put( true , new HashMap< Long , List< EventItemRecord >>( ) );
		result.put( false , new HashMap< Long , List< EventItemRecord >>( ) );
		for ( EventItemRecord eir : raw ) {
			List< EventItemRecord > event = result.get( eir.isPlanetEvent( ) ).get( eir.getEventId( ) );
			if ( event == null ) {
				event = new LinkedList< EventItemRecord >( );
				result.get( eir.isPlanetEvent( ) ).put( eir.getEventId( ) , event );
			}
			event.add( eir );
		}

		return result;
	}


	@Override
	public Map< Long , List< BuildingHistoryRecord >> getBuildingsHistory( EmpireBattleRecord record )
	{
		List< BuildingHistoryRecord > records;
		records = this.dTemplate.query( sGetBuildings , this.mGetBuildings , record.getEmpireId( ) , record
				.getBattleId( ) );

		Map< Long , List< BuildingHistoryRecord > > result = new HashMap< Long , List< BuildingHistoryRecord > >( );
		for ( BuildingHistoryRecord bhr : records ) {
			List< BuildingHistoryRecord > atTick = result.get( bhr.getTick( ) );
			if ( atTick == null ) {
				atTick = new LinkedList< BuildingHistoryRecord >( );
				result.put( bhr.getTick( ) , atTick );
			}
			atTick.add( bhr );
		}

		return result;
	}


	@Override
	public Map< Long , Map< Long , List< ShipHistoryRecord >>> getShipsHistory( EmpireBattleRecord record )
	{
		List< ShipHistoryRecord > records;
		records = this.dTemplate.query( sGetShips , this.mGetShips , record.getEmpireId( ) , record.getBattleId( ) );

		Map< Long , Map< Long , List< ShipHistoryRecord > >> result = new HashMap< Long , Map< Long , List< ShipHistoryRecord > > >( );
		for ( ShipHistoryRecord shr : records ) {
			Map< Long , List< ShipHistoryRecord > > atTick = result.get( shr.getTick( ) );
			if ( atTick == null ) {
				atTick = new HashMap< Long , List< ShipHistoryRecord > >( );
				result.put( shr.getTick( ) , atTick );
			}

			List< ShipHistoryRecord > pShips = atTick.get( shr.getProtagonist( ) );
			if ( pShips == null ) {
				pShips = new LinkedList< ShipHistoryRecord >( );
				atTick.put( shr.getProtagonist( ) , pShips );
			}
			pShips.add( shr );
		}

		return result;
	}


	@Override
	public List< BattleListRecord > getBattles( int empireId )
	{
		return this.dTemplate.query( sGetAllBattles , this.mBattleListEntry , empireId );
	}


	public List< BattleListRecord > getBattles( int empireId , long after )
	{
		return this.dTemplate.query( sGetNewBattles , this.mBattleListEntry , empireId , after );
	}
}
