package com.deepclone.lw.beans.fleets;


import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.player.fleets.DisbandFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.MergeFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.MoveFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.MultiFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.RenameFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.SetFleetsModeResponse;
import com.deepclone.lw.cmd.player.fleets.SplitFleetResponse;
import com.deepclone.lw.cmd.player.fleets.ViewFleetsResponse;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.fleets.FleetStatus;
import com.deepclone.lw.cmd.player.gdata.fleets.FleetsView;
import com.deepclone.lw.cmd.player.gdata.fleets.ShortFleetView;
import com.deepclone.lw.cmd.player.gdata.fleets.SplitShips;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.game.FleetManagement;
import com.deepclone.lw.interfaces.game.FleetsDAO;



@Transactional
public class FleetManagementBean
		implements FleetManagement
{
	private EmpireManagement empireManagement;
	private FleetsDAO fleetsDao;


	@Autowired( required = true )
	public void setEmpireManagement( EmpireManagement empireManagement )
	{
		this.empireManagement = empireManagement;
	}


	@Autowired( required = true )
	public void setFleetsDao( FleetsDAO fleetsDao )
	{
		this.fleetsDao = fleetsDao;
	}


	@Override
	public ViewFleetsResponse getFleets( int empireId )
	{
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		FleetsView fleets = this.fleetsDao.getFleets( empireId );
		return new ViewFleetsResponse( page , fleets );
	}


	@Override
	public MultiFleetsResponse setMode( int empireId , long[] fleets , boolean attack , boolean doIt )
	{
		SetFleetsModeResponse response;
		response = new SetFleetsModeResponse( this.empireManagement.getGeneralInformation( empireId ) );

		List< ShortFleetView > sFleets = new LinkedList< ShortFleetView >( );
		for ( ShortFleetView sfv : this.fleetsDao.getShortFleets( empireId , fleets , false ) ) {
			if ( sfv.isAttack( ) != attack ) {
				sFleets.add( sfv );
			}
		}

		if ( sFleets.isEmpty( ) ) {
			return response;
		}

		if ( doIt ) {
			this.fleetsDao.setMode( empireId , fleets , attack );
			return response;
		}

		for ( ShortFleetView sfv : sFleets ) {
			response.addFleet( sfv );
		}
		response.setAttack( attack );
		return response;
	}


	@Override
	public MultiFleetsResponse move( int empireId , long[] fleets , String destination , boolean attack )
	{
		MoveFleetsResponse response = new MoveFleetsResponse( this.empireManagement.getGeneralInformation( empireId ) );
		List< ShortFleetView > sFleets = this.fleetsDao.getShortFleets( empireId , fleets , true );
		if ( sFleets.isEmpty( ) ) {
			return response;
		}

		boolean moved;
		if ( destination != null && !sFleets.isEmpty( ) ) {
			moved = this.fleetsDao.move( empireId , fleets , destination );
			if ( !moved ) {
				response.setError( true );
				response.setDestination( destination );
				response.setMode( attack );
			} else {
				this.fleetsDao.setMode( empireId , fleets , attack );
			}
		} else {
			moved = false;
		}

		if ( !moved ) {
			for ( ShortFleetView sfv : sFleets ) {
				response.addFleet( sfv );
			}
		}

		return response;
	}


	@Override
	public MultiFleetsResponse rename( int empireId , long[] fleets , String name )
	{
		RenameFleetsResponse response;
		response = new RenameFleetsResponse( this.empireManagement.getGeneralInformation( empireId ) );

		List< ShortFleetView > sFleets = this.fleetsDao.getShortFleets( empireId , fleets , false );
		if ( sFleets.isEmpty( ) ) {
			return response;
		}

		if ( name != null ) {
			this.fleetsDao.rename( empireId , fleets , name );
		} else {
			for ( ShortFleetView sfv : sFleets ) {
				response.addFleet( sfv );
			}
		}

		return response;
	}


	@Override
	public MultiFleetsResponse merge( int empireId , long[] fleets )
	{
		MergeFleetsResponse response = new MergeFleetsResponse( this.empireManagement.getGeneralInformation( empireId ) );
		List< ShortFleetView > sFleets = this.fleetsDao.getShortFleets( empireId , fleets , true );
		if ( !sFleets.isEmpty( ) ) {
			this.fleetsDao.merge( empireId , fleets );
		}
		return response;
	}


	@Override
	public SplitFleetResponse split( int empireId , long fleetId , Map< Integer , Integer > ships , int nFleets ,
			String name , boolean simulate )
	{
		SplitFleetResponse response = new SplitFleetResponse( this.empireManagement.getGeneralInformation( empireId ) );
		if ( fleetId == -1 ) {
			return response;
		}

		// Try finding the fleet
		ShortFleetView fleet = this.fleetsDao.getShortFleet( empireId , fleetId );
		if ( fleet == null || fleet.getStatus( ) != FleetStatus.AVAILABLE ) {
			return response;
		}

		// If the name is not set, this is the initial request
		if ( name == null ) {
			for ( SplitShips sShips : this.fleetsDao.getShips( empireId , fleetId ) ) {
				sShips.setSelectedAmount( 0 );
				response.addShips( sShips );
			}
			response.setInitialFleet( fleet );
			return response;
		}

		// Try splitting / simulating the split
		boolean splitOk;
		splitOk = this.fleetsDao.initSplit( fleetId , nFleets , name );
		if ( splitOk ) {
			for ( Map.Entry< Integer , Integer > entry : ships.entrySet( ) ) {
				this.fleetsDao.setSplitShips( entry.getKey( ) , entry.getValue( ) );
			}
			splitOk = this.fleetsDao.executeSplit( simulate );
		}

		// If there was an error, or if we were simulating
		if ( !splitOk || simulate ) {
			for ( SplitShips sShips : this.fleetsDao.getShips( empireId , fleetId ) ) {
				Integer amount = ships.get( sShips.getType( ) );
				if ( amount == null ) {
					amount = 0;
				}
				sShips.setSelectedAmount( amount );
				response.addShips( sShips );
			}
			response.setInitialFleet( fleet );
			response.setShipsError( !splitOk );
			response.setnFleets( nFleets );
			response.setName( name );
		}

		return response;
	}


	@Override
	public MultiFleetsResponse disband( int empireId , long[] fleets , boolean doIt )
	{
		DisbandFleetsResponse response;
		response = new DisbandFleetsResponse( this.empireManagement.getGeneralInformation( empireId ) );

		List< ShortFleetView > sFleets = this.fleetsDao.getShortFleets( empireId , fleets , false );
		if ( sFleets.isEmpty( ) ) {
			return response;
		}

		if ( doIt ) {
			this.fleetsDao.disband( empireId , fleets );
		} else {
			for ( ShortFleetView sfv : sFleets ) {
				response.addFleet( sfv );
			}
		}

		return response;
	}

}
