package com.deepclone.lw.beans.map;


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.planets.BuildingData;
import com.deepclone.lw.cmd.player.gdata.planets.OwnPlanetStatusData;
import com.deepclone.lw.cmd.player.gdata.planets.PlanetBasicView;
import com.deepclone.lw.cmd.player.gdata.planets.PlanetOrbitalView;
import com.deepclone.lw.cmd.player.gdata.planets.PlanetOwnView;
import com.deepclone.lw.cmd.player.planets.BuildingActionResponse;
import com.deepclone.lw.cmd.player.planets.RenamePlanetResponse;
import com.deepclone.lw.cmd.player.planets.ViewPlanetResponse;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.game.PlanetDAO;
import com.deepclone.lw.interfaces.game.PlanetsManagement;
import com.deepclone.lw.interfaces.game.resources.ResourcesInformationDAO;
import com.deepclone.lw.interfaces.naming.NamingDAO;
import com.deepclone.lw.sqld.game.PlanetData;
import com.deepclone.lw.sqld.game.PlanetData.AccessType;
import com.deepclone.lw.sqld.game.PlanetData.Basic;
import com.deepclone.lw.sqld.game.PlanetData.Orbital;
import com.deepclone.lw.sqld.game.PlanetData.Owner;



@Transactional
public class PlanetsManagementBean
		implements PlanetsManagement
{

	private EmpireManagement empireManagement;
	private PlanetDAO planetDao;
	private NamingDAO namingDao;
	private ResourcesInformationDAO resourcesInformationDao;


	@Autowired( required = true )
	public void setEmpireManagement( EmpireManagement empireManagement )
	{
		this.empireManagement = empireManagement;
	}


	@Autowired( required = true )
	public void setPlanetDAO( PlanetDAO planetDao )
	{
		this.planetDao = planetDao;
	}


	@Autowired( required = true )
	public void setNamingDao( NamingDAO namingDao )
	{
		this.namingDao = namingDao;
	}


	@Autowired( required = true )
	public void setResourcesInformationDao( ResourcesInformationDAO resourcesInformationDao )
	{
		this.resourcesInformationDao = resourcesInformationDao;
	}


	private PlanetOrbitalView getOrbitalView( int empireId , int planetId , AccessType access )
	{
		if ( access == AccessType.BASIC ) {
			return null;
		}

		Orbital orbital = this.planetDao.getOrbitalInformation( empireId , planetId );
		List< BuildingData > buildings = this.planetDao.getBuildings( empireId , planetId ,
				( access == AccessType.OWNER ) );

		PlanetOrbitalView view = new PlanetOrbitalView( );
		view.setPopulation( orbital.getPopulation( ) );
		view.setDefencePoints( orbital.getDefence( ) );
		view.setOwnFleet( orbital.getOwnPower( ) );
		view.setFriendlyFleet( orbital.getFriendlyPower( ) );
		view.setHostileFleet( orbital.getHostilePower( ) );
		view.setBattle( orbital.getBattle( ) );
		view.setBuildings( buildings );

		return view;
	}


	private PlanetOwnView getOwnerView( int empireId , int planetId , AccessType access )
	{
		if ( access != AccessType.OWNER ) {
			return null;
		}

		Owner data = this.planetDao.getOwnerInformation( empireId , planetId );
		OwnPlanetStatusData status = new OwnPlanetStatusData( );
		status.setRenamePossible( data.isRenamePossible( ) );
		status.setAbandonPossible( data.isAbandonPossible( ) );
		status.setAbandonTime( data.getAbandonTime( ) );

		PlanetOwnView view = new PlanetOwnView( );
		view.setStatus( status );
		view.setHappiness( data.getHappiness( ) );
		view.sethChange( data.gethChange( ) );
		view.setIncome( data.getIncome( ) );
		view.setUpkeep( data.getUpkeep( ) );
		view.setOwnMiningSettings( data.isOwnSettings( ) );

		view.setCivQueue( this.planetDao.getConstructionQueue( planetId ) );
		view.setMilQueue( this.planetDao.getMilitaryQueue( planetId ) );

		view.setbBuildings( this.planetDao.getAvailableBuildings( planetId ) );
		view.setbShips( this.planetDao.getAvailableShips( planetId ) );

		view.setResources( this.resourcesInformationDao.getPlanetInformation( planetId ) );

		return view;
	}


	private PlanetBasicView getBasicView( Basic data )
	{
		return new PlanetBasicView( data.getName( ) , data.getPicture( ) , data.getTag( ) , data.getX( ) ,
				data.getY( ) , data.getOrbit( ) );
	}


	private ViewPlanetResponse planetNotFound( int empireId )
	{
		return new ViewPlanetResponse( this.empireManagement.getGeneralInformation( empireId ) );
	}


	private ViewPlanetResponse ownershipError( int empireId , int planetId , Basic basic )
	{
		AccessType access = basic.getAccess( );
		PlanetOrbitalView orbital = this.getOrbitalView( empireId , planetId , access );
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		return new ViewPlanetResponse( planetId , page , this.getBasicView( basic ) , orbital );
	}


	private ViewPlanetResponse fullView( int empireId , int planetId , Basic basic )
	{
		AccessType access = basic.getAccess( );
		PlanetOrbitalView orbital = this.getOrbitalView( empireId , planetId , access );
		PlanetOwnView owner = this.getOwnerView( empireId , planetId , access );
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		return new ViewPlanetResponse( planetId , page , this.getBasicView( basic ) , orbital , owner );

	}


	@Override
	public ViewPlanetResponse viewPlanet( int empireId , int planetId )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		}

		AccessType access = basic.getAccess( );
		PlanetOrbitalView orbital = this.getOrbitalView( empireId , planetId , access );
		PlanetOwnView owner = this.getOwnerView( empireId , planetId , access );
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );
		return new ViewPlanetResponse( planetId , page , this.getBasicView( basic ) , orbital , owner );
	}


	@Override
	public ViewPlanetResponse renamePlanet( int empireId , int planetId , String name )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		}

		AccessType access = basic.getAccess( );
		if ( access != AccessType.OWNER ) {
			return this.ownershipError( empireId , planetId , basic );
		}

		int rv = this.namingDao.renamePlanet( planetId , name );
		ObjectNameError one = null;
		switch ( rv ) {
			case 0:
				basic.setName( name );
				break;
			case 1:
				one = ObjectNameError.BANNED;
				break;
			case 2:
				one = ObjectNameError.UNAVAILABLE;
				break;
			case 3:
				// "Too early" - someone probably pressed F5
				break;
			default:
				throw new RuntimeException( "unknown error code " + rv );
		}

		PlanetOrbitalView orbital = this.getOrbitalView( empireId , planetId , access );
		PlanetOwnView owner = this.getOwnerView( empireId , planetId , access );
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );

		if ( one == null ) {
			return new ViewPlanetResponse( planetId , page , this.getBasicView( basic ) , orbital , owner );
		}
		return new RenamePlanetResponse( planetId , page , this.getBasicView( basic ) , orbital , owner , name ,
				one.toString( ) );
	}


	@Override
	public ViewPlanetResponse flushQueue( int empireId , int planetId , boolean military )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		} else if ( basic.getAccess( ) != AccessType.OWNER ) {
			return this.ownershipError( empireId , planetId , basic );
		}
		this.planetDao.flushQueue( planetId , military );

		return this.fullView( empireId , planetId , basic );
	}


	@Override
	public ViewPlanetResponse addToMilitaryQueue( int empireId , int planetId , int sType , int amount )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		} else if ( basic.getAccess( ) != AccessType.OWNER ) {
			return this.ownershipError( empireId , planetId , basic );
		}
		this.planetDao.addToMilitaryQueue( planetId , sType , amount );
		return this.fullView( empireId , planetId , basic );
	}


	@Override
	public ViewPlanetResponse addToCivilianQueue( int empireId , int planetId , int bType , boolean destroy , int amount )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		} else if ( basic.getAccess( ) != AccessType.OWNER ) {
			return this.ownershipError( empireId , planetId , basic );
		}

		if ( !destroy ) {
			this.planetDao.constructBuildings( planetId , bType , amount );
		} else if ( !this.planetDao.destroyBuildings( planetId , bType , amount ) ) {
			AccessType access = basic.getAccess( );
			PlanetOrbitalView orbital = this.getOrbitalView( empireId , planetId , access );
			PlanetOwnView owner = this.getOwnerView( empireId , planetId , access );
			GamePageData page = this.empireManagement.getGeneralInformation( empireId );
			return new BuildingActionResponse( planetId , page , this.getBasicView( basic ) , orbital , owner );
		}
		return this.fullView( empireId , planetId , basic );
	}


	@Override
	public ViewPlanetResponse abandon( int empireId , int planetId )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		} else if ( basic.getAccess( ) != AccessType.OWNER ) {
			return this.ownershipError( empireId , planetId , basic );
		}

		AccessType access = basic.getAccess( );
		PlanetOrbitalView orbital = this.getOrbitalView( empireId , planetId , access );
		PlanetOwnView owner = this.getOwnerView( empireId , planetId , access );
		GamePageData page = this.empireManagement.getGeneralInformation( empireId );

		Integer x;
		if ( owner.getStatus( ).isAbandonPossible( ) ) {
			x = this.planetDao.abandon( planetId );
		} else {
			x = null;
		}

		if ( x != null ) {
			owner.getStatus( ).setAbandonPossible( false );
			owner.getStatus( ).setAbandonTime( x );
		}

		return new ViewPlanetResponse( planetId , page , this.getBasicView( basic ) , orbital , owner );
	}


	@Override
	public ViewPlanetResponse cancelAbandon( int empireId , int planetId )
	{
		PlanetData.Basic basic = this.planetDao.getBasicInformation( empireId , planetId );
		if ( basic == null ) {
			return this.planetNotFound( empireId );
		} else if ( basic.getAccess( ) != AccessType.OWNER ) {
			return this.ownershipError( empireId , planetId , basic );
		}
		this.planetDao.cancelAbandon( planetId );
		return this.fullView( empireId , planetId , basic );
	}
}
