package com.deepclone.lw.beans.msgs;


import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.msgdata.*;
import com.deepclone.lw.cmd.admin.msg.*;
import com.deepclone.lw.interfaces.admin.AdminDAO;
import com.deepclone.lw.interfaces.game.EmpireDAO;
import com.deepclone.lw.interfaces.msg.AdminMessages;
import com.deepclone.lw.interfaces.msg.MessageBoxDAO;
import com.deepclone.lw.interfaces.msg.MessageContentCache;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatRegistry;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



@Transactional
public class AdminMessagesBean
		implements AdminMessages
{

	private MessageBoxDAO mboxDao;
	private MessageContentCache contentCache;
	private MessageFormatRegistry formats;
	private EmpireDAO empireDao;
	private AdminDAO adminDao;


	@Autowired( required = true )
	public void setMboxDao( MessageBoxDAO mboxDao )
	{
		this.mboxDao = mboxDao;
	}


	@Autowired( required = true )
	public void setContentCache( MessageContentCache contentCache )
	{
		this.contentCache = contentCache;
	}


	@Autowired( required = true )
	public void setFormats( MessageFormatRegistry formats )
	{
		this.formats = formats;
	}


	@Autowired( required = true )
	public void setEmpireDao( EmpireDAO empireDao )
	{
		this.empireDao = empireDao;
	}


	@Autowired( required = true )
	public void setAdminDao( AdminDAO adminDao )
	{
		this.adminDao = adminDao;
	}


	private Message readMessage( Administrator admin , boolean inbox , long id )
	{
		// Get list of messages
		List< InboxRecord > mbox = this.mboxDao.getList( true , admin.getId( ) , inbox );
		Map< Long , MessageDataRecord > contents = this.contentCache.getContent( mbox );
		if ( !contents.containsKey( id ) ) {
			return null;
		}

		// Find next and previous messages
		Long previous = null;
		Long next = null;
		InboxRecord record = null;
		boolean found = false;
		for ( InboxRecord message : mbox ) {
			if ( !found ) {
				if ( message.getId( ) == id ) {
					found = true;
					record = message;
				} else {
					previous = message.getId( );
				}
			} else if ( next == null ) {
				next = message.getId( );
				break;
			}
		}

		MessageDataRecord content = contents.get( record.getId( ) );
		MessageExtractor formatter = this.formats.getFormatter( content ).getExtractor( record , content , "en" );

		// Extract basic information
		Message message = new Message( );
		message.setId( id );
		message.setNext( next );
		message.setPrevious( previous );
		message.setTime( record.getReceived( ) );
		message.setSender( inbox ? formatter.getSender( ) : record.getSenderName( ) );
		message.setReceiver( record.getReceiverName( ) );
		message.setTitle( formatter.getSubject( ) );
		message.setContents( formatter.getContents( ) );
		message.setUnread( !record.isRead( ) );
		message.setGameTime( content.getTick( ) );

		// Handle message type
		String type = inbox ? record.getSenderType( ) : record.getReceiverType( );
		if ( "EMP".equals( type ) ) {
			message.setType( MessageType.EMPIRE );
		} else if ( "ADM".equals( type ) ) {
			message.setType( MessageType.ADMINISTRATOR );
		} else {
			throw new RuntimeException( "unknown message type " + type );
		}

		return message;
	}


	@Override
	public GetMessagesResponse getMessages( Administrator admin , boolean inbox )
	{
		// Get list of messages
		List< InboxRecord > mbox = this.mboxDao.getList( true , admin.getId( ) , inbox );
		Map< Long , MessageDataRecord > contents = this.contentCache.getContent( mbox );

		// Format messages and create list of entries
		List< MessageListEntry > results = new LinkedList< MessageListEntry >( );
		for ( InboxRecord mRecord : mbox ) {
			MessageDataRecord content = contents.get( mRecord.getId( ) );
			MessageExtractor formatter = this.formats.getFormatter( content ).getExtractor( mRecord , content , "en" );

			MessageListEntry result = new MessageListEntry( );
			result.setId( mRecord.getId( ) );
			result.setTime( mRecord.getReceived( ) );
			result.setRead( mRecord.isRead( ) );
			result.setTitle( formatter.getSubject( ) );
			result.setSender( inbox ? formatter.getSender( ) : mRecord.getReceiverName( ) );

			String type = inbox ? mRecord.getSenderType( ) : mRecord.getReceiverType( );
			if ( "ADM".equals( type ) ) {
				result.setType( MessageType.ADMINISTRATOR );
			} else if ( "EMP".equals( type ) ) {
				result.setType( MessageType.EMPIRE );
			} else {
				throw new RuntimeException( "unknown message type " + type );
			}
			results.add( result );
		}

		return new GetMessagesResponse( admin , results );
	}


	@Override
	public ReadMessageResponse getMessage( Administrator admin , boolean inbox , long id )
	{
		// Get list of messages
		Message message = this.readMessage( admin , inbox , id );
		if ( message == null ) {
			return new ReadMessageResponse( admin , inbox );
		}

		// Mark message as read
		if ( inbox && message.isUnread( ) ) {
			this.mboxDao.markRead( true , admin.getId( ) , new long[] {
				id
			} );
		}

		return new ReadMessageResponse( admin , inbox , message );
	}


	@Override
	public void deleteMessages( Administrator admin , boolean inbox , long[] selection )
	{
		if ( selection == null ) {
			this.mboxDao.delete( true , admin.getId( ) , inbox );
		} else {
			this.mboxDao.delete( true , admin.getId( ) , inbox , selection );
		}
	}


	@Override
	public void markRead( Administrator admin , long[] selection )
	{
		if ( selection == null ) {
			this.mboxDao.markRead( true , admin.getId( ) );
		} else {
			this.mboxDao.markRead( true , admin.getId( ) , selection );
		}
	}


	@Override
	public void markUnread( Administrator admin , long[] selection )
	{
		if ( selection == null ) {
			this.mboxDao.markUnread( true , admin.getId( ) );
		} else {
			this.mboxDao.markUnread( true , admin.getId( ) , selection );
		}
	}


	@Override
	public ComposeMessageResponse prepareBlankMessage( Administrator admin )
	{
		ComposeMessageResponse response = new ComposeMessageResponse( admin );
		response.setMessageType( MessageType.ADMINISTRATOR );
		response.setTarget( "" );
		response.setTitle( "" );
		response.setContents( "" );
		return response;
	}


	@Override
	public ComposeMessageResponse prepareMessageTo( Administrator admin , MessageType type , int id )
	{
		ComposeMessageResponse response = this.prepareBlankMessage( admin );
		response.setMessageType( type );
		try {
			switch ( type ) {
				case EMPIRE:
					response.setTarget( this.empireDao.getInformation( id ).getName( ) );
					break;
				case ADMINISTRATOR:
					response.setTarget( this.adminDao.getAdmin( id ).getName( ) );
					break;
				default:
					throw new RuntimeException( "unsupported target type " + type );
			}
		} catch ( NullPointerException e ) {
			response.setTargetError( true );
		}
		return response;
	}


	@Override
	public ComposeMessageResponse sendMessage( Administrator admin , MessageType type , String target , String title ,
			String contents , boolean simulate )
	{
		int errCode = this.mboxDao.sendMessage( true , admin.getId( ) , type , target , title , contents , simulate );
		if ( errCode == 0 && !simulate ) {
			return this.prepareBlankMessage( admin );
		}

		ComposeMessageResponse response = new ComposeMessageResponse( admin );
		return this.sendError( type , target , title , contents , errCode , response );
	}


	@Override
	public ComposeMessageResponse sendReply( Administrator admin , boolean inbox , long replyTo , MessageType type ,
			String target , String title , String contents , boolean simulate )
	{
		int errCode = this.mboxDao.sendMessage( true , admin.getId( ) , type , target , title , contents , simulate );
		if ( errCode == 0 && !simulate ) {
			return this.prepareBlankMessage( admin );
		}

		Message message = this.readMessage( admin , inbox , replyTo );
		ComposeMessageResponse response;
		if ( message == null || message.getType( ) == MessageType.INTERNAL ) {
			response = new ComposeMessageResponse( admin );
		} else {
			response = new ComposeMessageResponse( admin , inbox , message );
		}

		return this.sendError( type , target , title , contents , errCode , response );
	}


	private ComposeMessageResponse sendError( MessageType type , String target , String title , String contents ,
			int errCode , ComposeMessageResponse response )
	{
		switch ( errCode ) {
			case 1:
				response.setTargetError( true );
				break;
			case 2:
				response.setTimingError( true );
				break;
		}

		response.setMessageType( type );
		response.setTarget( target );
		response.setTitle( title );
		response.setContents( contents );

		return response;
	}


	@Override
	public ComposeMessageResponse prepareReply( Administrator admin , boolean inbox , long id )
	{
		Message message = this.readMessage( admin , inbox , id );
		if ( message == null || message.getType( ) == MessageType.INTERNAL ) {
			return this.prepareBlankMessage( admin );
		}

		ComposeMessageResponse response;
		response = new ComposeMessageResponse( admin , inbox , message );

		response.setMessageType( message.getType( ) );
		response.setTarget( message.getSender( ) );

		String title = message.getTitle( );
		if ( !title.startsWith( "Re: " ) ) {
			title = "Re: " + title;
			if ( title.length( ) > 64 ) {
				title = title.substring( 0 , 64 );
			}
		}
		response.setTitle( title );

		response.setContents( "" );
		return response;
	}


	@Override
	public void sendSpam( Administrator admin , String title , String body )
	{
		this.mboxDao.sendSpam( admin.getId( ) , title , body );
	}

}
