package com.deepclone.lw.beans.msgs;


import java.util.Collection;

import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import com.deepclone.lw.interfaces.msg.MessageFormatRegistry;
import com.deepclone.lw.interfaces.msg.MessageFormatter;



public class MessageFormatWiringBean
		implements BeanPostProcessor , ApplicationContextAware
{
	private final Logger logger = Logger.getLogger( MessageFormatWiringBean.class );
	private ApplicationContext context;


	@Override
	public void setApplicationContext( ApplicationContext applicationContext )
			throws BeansException
	{
		this.context = applicationContext;
	}


	@Override
	public Object postProcessAfterInitialization( Object bean , String beanName )
			throws BeansException
	{
		if ( bean instanceof MessageFormatRegistry ) {
			this.logger.debug( "Wiring message format registry" );
			this.autowire( (MessageFormatRegistry) bean );
		}
		return bean;
	}


	@Override
	public Object postProcessBeforeInitialization( Object bean , String beanName )
			throws BeansException
	{
		return bean;
	}


	private void autowire( MessageFormatRegistry bean )
	{
		Collection< MessageFormatter > formatters;
		formatters = this.context.getBeansOfType( MessageFormatter.class ).values( );

		for ( MessageFormatter formatter : formatters ) {
			this.logger.debug( "Adding formats " + formatter.getFormats( ) );
			bean.addFormatter( formatter );
		}
	}
}
