package com.deepclone.lw.beans.msgs;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import com.deepclone.lw.interfaces.msg.MessageRecordsDAO;
import com.deepclone.lw.sqld.msgs.*;



public class MessageRecordsDAOBean
		implements MessageRecordsDAO
{
	private static abstract class EventMapperBase< T extends EventRecord >
			implements RowMapper< T >
	{
		protected void mapEvent( ResultSet rs , T event )
				throws SQLException
		{
			event.setId( rs.getLong( "id" ) );
			event.setType( EventType.valueOf( rs.getString( "evt_type" ) ) );
			event.setSubType( rs.getInt( "evt_subtype" ) );
			event.setTick( rs.getLong( "tick" ) );
			event.setTimestamp( rs.getTimestamp( "real_time" ) );
		}
	}

	private static final String sGetTextMessages = "SELECT * FROM msgs.text_messages WHERE id = ANY( ? ::BIGINT[] )";
	private static final String sGetEventTypes = "SELECT event_id , evt_type FROM events.events WHERE event_id = ANY( ? ::BIGINT[] )";
	private static final String sGetEventParts[] = {
			"SELECT * FROM events." , "_events_view WHERE id = ANY( ? ::BIGINT[] )"
	};
	private static final String sGetQueueLocations = "SELECT * FROM events.queue_locations_view WHERE event_id = ANY( ? ::BIGINT[] )";
	private static final String sGetEventFleets = "SELECT * FROM events.fleet_lists WHERE event_id = ANY( ? ::BIGINT[] )";

	private JdbcTemplate dTemplate;

	private final RowMapper< TextMessageRecord > mTextMessage;
	private final RowMapper< EventTypeRecord > mEventTypes;
	private final EventMapperBase< AdminEventRecord > mAdminEvent;
	private final EventMapperBase< AllianceEventRecord > mAllianceEvent;
	private final EventMapperBase< EmpireEventRecord > mEmpireEvent;
	private final EventMapperBase< FleetEventRecord > mFleetEvent;
	private final EventMapperBase< PlanetEventRecord > mPlanetEvent;
	private final EventMapperBase< QueueEventRecord > mQueueEvent;
	private final EventMapperBase< BugEventRecord > mBugEvent;
	private final RowMapper< QueueEventLocation > mQueueLocation;
	private final RowMapper< FleetEventFleet > mFleetList;


	public MessageRecordsDAOBean( )
	{
		this.mTextMessage = new RowMapper< TextMessageRecord >( ) {
			@Override
			public TextMessageRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				TextMessageRecord tmr = new TextMessageRecord( );
				tmr.setId( rs.getLong( "id" ) );
				tmr.setTimestamp( rs.getTimestamp( "t" ) );
				tmr.setTick( rs.getLong( "tick" ) );
				tmr.setSubject( rs.getString( "title" ) );
				tmr.setText( rs.getString( "contents" ) );
				return tmr;
			}
		};
		this.mEventTypes = new RowMapper< EventTypeRecord >( ) {
			@Override
			public EventTypeRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				EventTypeRecord etr = new EventTypeRecord( );
				etr.setId( rs.getLong( "event_id" ) );
				etr.setType( EventType.valueOf( rs.getString( "evt_type" ) ) );
				return etr;
			}
		};
		this.mAdminEvent = new EventMapperBase< AdminEventRecord >( ) {
			@Override
			public AdminEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AdminEventRecord event = new AdminEventRecord( );
				this.mapEvent( rs , event );
				event.setWarnings( (Integer) rs.getObject( "n_warnings" ) );
				event.setLocationId( (Integer) rs.getObject( "location_id" ) );
				event.setOldName( rs.getString( "old_name" ) );
				event.setNewName( rs.getString( "new_name" ) );
				return event;
			}
		};
		this.mAllianceEvent = new EventMapperBase< AllianceEventRecord >( ) {
			@Override
			public AllianceEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				AllianceEventRecord event = new AllianceEventRecord( );
				this.mapEvent( rs , event );
				event.setAllianceId( (Integer) rs.getObject( "alliance_id" ) );
				event.setAllianceTag( rs.getString( "alliance_tag" ) );
				event.setEmpireId( (Integer) rs.getObject( "empire_id" ) );
				event.setEmpireName( rs.getString( "empire_name" ) );
				event.setReqResult( (Boolean) rs.getObject( "req_result" ) );
				return event;
			}
		};
		this.mEmpireEvent = new EventMapperBase< EmpireEventRecord >( ) {
			@Override
			public EmpireEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				EmpireEventRecord event = new EmpireEventRecord( );
				this.mapEvent( rs , event );
				event.setTech( rs.getString( "technology" ) );
				return event;
			}
		};
		this.mFleetEvent = new EventMapperBase< FleetEventRecord >( ) {
			@Override
			public FleetEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				FleetEventRecord event = new FleetEventRecord( );
				this.mapEvent( rs , event );
				event.setLocationId( rs.getInt( "location_id" ) );
				event.setLocationName( rs.getString( "location_name" ) );
				event.setX( rs.getInt( "x" ) );
				event.setY( rs.getInt( "y" ) );
				event.setOrbit( rs.getInt( "orbit" ) );
				return event;
			}
		};
		this.mPlanetEvent = new EventMapperBase< PlanetEventRecord >( ) {
			@Override
			public PlanetEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				PlanetEventRecord event = new PlanetEventRecord( );
				this.mapEvent( rs , event );
				event.setLocationId( rs.getInt( "location_id" ) );
				event.setLocationName( rs.getString( "location_name" ) );
				event.setX( rs.getInt( "x" ) );
				event.setY( rs.getInt( "y" ) );
				event.setOrbit( rs.getInt( "orbit" ) );
				event.setEmpireId( (Integer) rs.getObject( "empire_id" ) );
				event.setEmpireName( rs.getString( "empire_name" ) );
				event.setBattleId( (Long) rs.getObject( "battle_id" ) );
				return event;
			}
		};
		this.mQueueEvent = new EventMapperBase< QueueEventRecord >( ) {
			@Override
			public QueueEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				QueueEventRecord event = new QueueEventRecord( );
				this.mapEvent( rs , event );
				return event;
			}
		};
		this.mBugEvent = new EventMapperBase< BugEventRecord >( ) {
			@Override
			public BugEventRecord mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				BugEventRecord event = new BugEventRecord( );
				this.mapEvent( rs , event );
				event.setReportId( rs.getLong( "bug_id" ) );
				event.setAdmin( rs.getBoolean( "submitter_admin" ) );
				event.setSubmitter( rs.getString( "submitter_name" ) );
				return event;
			}
		};
		this.mQueueLocation = new RowMapper< QueueEventLocation >( ) {
			@Override
			public QueueEventLocation mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				QueueEventLocation loc = new QueueEventLocation( );
				loc.setEventId( rs.getLong( "event_id" ) );
				loc.setLocationId( rs.getInt( "location_id" ) );
				loc.setLocationName( rs.getString( "location_name" ) );
				loc.setX( rs.getInt( "x" ) );
				loc.setY( rs.getInt( "y" ) );
				loc.setOrbit( rs.getInt( "orbit" ) );
				return loc;
			}
		};
		this.mFleetList = new RowMapper< FleetEventFleet >( ) {
			@Override
			public FleetEventFleet mapRow( ResultSet rs , int rowNum )
					throws SQLException
			{
				FleetEventFleet fleet = new FleetEventFleet( );
				fleet.setEventId( rs.getLong( "event_id" ) );
				fleet.setOwnerId( (Integer) rs.getObject( "owner_id" ) );
				fleet.setOwnerName( rs.getString( "owner_name" ) );
				fleet.setFleetName( rs.getString( "fleet_name" ) );
				fleet.setFleetPower( rs.getLong( "fleet_power" ) );
				fleet.setStatus( (Boolean) rs.getObject( "status" ) );
				fleet.setSourceId( (Integer) rs.getObject( "source_id" ) );
				fleet.setSourceName( rs.getString( "source_name" ) );
				return fleet;
			}
		};
	}


	@Autowired( required = true )
	public void setDataSource( DataSource dataSource )
	{
		this.dTemplate = new JdbcTemplate( dataSource );
	}


	private String getIdArray( List< Long > ids )
	{
		StringBuilder iArray = new StringBuilder( ).append( "{" );
		int i = 0;
		int s = ids.size( );
		for ( Long id : ids ) {
			iArray.append( id );
			if ( i != s - 1 ) {
				iArray.append( "," );
			}
			i++;
		}
		iArray.append( "}" );
		return iArray.toString( );
	}


	@Override
	public List< TextMessageRecord > getTextMessages( List< Long > ids )
	{
		return this.dTemplate.query( sGetTextMessages , mTextMessage , this.getIdArray( ids ) );
	}


	@Override
	public List< EventTypeRecord > getEventTypes( List< Long > ids )
	{
		return this.dTemplate.query( sGetEventTypes , mEventTypes , this.getIdArray( ids ) );
	}


	@Override
	@SuppressWarnings( "unchecked" )
	public List< EventRecord > getEvents( EventType type , List< Long > ids )
	{
		String query = sGetEventParts[ 0 ] + type.toString( ).toLowerCase( ) + sGetEventParts[ 1 ];
		EventMapperBase< ? > mapper;
		switch ( type ) {
			case ADMIN:
				mapper = this.mAdminEvent;
				break;
			case ALLIANCE:
				mapper = this.mAllianceEvent;
				break;
			case EMPIRE:
				mapper = this.mEmpireEvent;
				break;
			case FLEETS:
				mapper = this.mFleetEvent;
				break;
			case PLANET:
				mapper = this.mPlanetEvent;
				break;
			case QUEUE:
				mapper = this.mQueueEvent;
				break;
			case BUGS:
				mapper = this.mBugEvent;
				break;
			default:
				throw new RuntimeException( "unsupported event type " + type );
		}

		String idArray = this.getIdArray( ids );
		List< EventRecord > result = (List< EventRecord >) this.dTemplate.query( query , mapper , idArray );

		if ( type == EventType.FLEETS ) {
			this.getFleetDetails( idArray , result );
		} else if ( type == EventType.QUEUE ) {
			this.getQueueDetails( idArray , result );
		}

		return result;
	}


	private void getFleetDetails( String idArray , List< EventRecord > result )
	{
		Map< Long , FleetEventRecord > records = new HashMap< Long , FleetEventRecord >( );
		for ( EventRecord e : result ) {
			records.put( e.getId( ) , (FleetEventRecord) e );
		}

		List< FleetEventFleet > fleets = this.dTemplate.query( sGetEventFleets , mFleetList , idArray );
		for ( FleetEventFleet fleet : fleets ) {
			records.get( fleet.getEventId( ) ).addFleet( fleet );
		}
	}


	private void getQueueDetails( String idArray , List< EventRecord > result )
	{
		Map< Long , QueueEventRecord > records = new HashMap< Long , QueueEventRecord >( );
		for ( EventRecord e : result ) {
			records.put( e.getId( ) , (QueueEventRecord) e );
		}

		List< QueueEventLocation > locations = this.dTemplate.query( sGetQueueLocations , mQueueLocation , idArray );
		for ( QueueEventLocation l : locations ) {
			records.get( l.getEventId( ) ).addLocation( l );
		}
	}
}
