package com.deepclone.lw.beans.msgs.fmt;


import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.sqld.msgs.AdminEventRecord;



public class AdminMessageExtractor
		implements MessageExtractor
{

	private Translator translator;
	private String language;
	private AdminEventRecord event;
	private String eType;


	private String translate( String what )
	{
		try {
			return this.translator.translate( this.language , what );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
	}


	public AdminMessageExtractor( AdminEventRecord event , Translator translator , String language )
	{
		this.event = event;
		this.translator = translator;
		this.language = language;
		switch ( this.event.getSubType( ) ) {
			case 0:
				this.eType = "Planet";
				break;
			case 1:
				this.eType = "Empire";
				break;
			case 2:
				this.eType = "Alliance";
				break;
			default:
				throw new RuntimeException( "unsupported event sub-type " + this.event.getSubType( ) );
		}
	}


	@Override
	public String getSender( )
	{
		return this.translate( "imSenderAdmin" );
	}


	@Override
	public String getSubject( )
	{
		String s = this.translate( "imtAdmin" + this.eType ).replace( "${oldName}" , this.event.getOldName( ) );
		if ( this.event.getNewName( ) != null ) {
			s = s.replace( "${newName}" , this.event.getNewName( ) );
		}
		return s;
	}


	@Override
	public String getContents( )
	{
		String s = this.translate( "imAdmin" + this.eType ).replace( "${oldName}" , this.event.getOldName( ) );
		if ( this.event.getNewName( ) != null ) {
			s = s.replace( "${newName}" , this.event.getNewName( ) );
		}
		if ( this.event.getLocationId( ) != null ) {
			s = s.replace( "${locationId}" , this.event.getLocationId( ).toString( ) );
		}
		if ( this.event.getWarnings( ) != null ) {
			s += "\n\n";
			s += this.translate( "imAdminWarning" ).replace( "${warnings}" , this.event.getWarnings( ).toString( ) );
		}
		return s;
	}

}
