package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.BugEventRecord;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



@Component
public class BugMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static class BugMessageExtractor
			implements MessageExtractor
	{
		private BugEventRecord event;
		private String mTitle;
		private String mBody;
		private String mSource;


		public BugMessageExtractor( BugEventRecord event , String mTitle , String mBody , String mSource )
		{
			this.event = event;
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
		}


		@Override
		public String getContents( )
		{
			String bug = "{{bug:" + this.event.getReportId( ) + "}}";
			return this.mBody.replace( "${bug}" , bug ).replace( "${submitter}" , this.event.getSubmitter( ) );
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle.replace( "${id}" , ( (Long) this.event.getReportId( ) ).toString( ) );
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.BUGS , 0 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		BugEventRecord event = (BugEventRecord) contents;
		String suffix = event.isAdmin( ) ? "Admin" : "Empire";
		String mTitle , mBody , mSource;
		try {
			mTitle = this.translator.translate( language , "imtBugReportUpdate" );
			mBody = this.translator.translate( language , "imBugReportUpdate" + suffix );
			mSource = this.translator.translate( language , "imSenderBugtracker" );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return new BugMessageExtractor( event , mTitle , mBody , mSource );
	}

}
