package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.EmpireEventRecord;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



@Component
public class DebtMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static class DebtMessageExtractor
			implements MessageExtractor
	{
		private String mTitle;
		private String mBody;
		private String mSource;


		public DebtMessageExtractor( String mTitle , String mBody , String mSource )
		{
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
		}


		@Override
		public String getContents( )
		{
			return this.mBody;
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle;
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.EMPIRE , 1 ) );
		fmts.add( new FormatType( EventType.EMPIRE , 2 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		EmpireEventRecord event = (EmpireEventRecord) contents;
		String type = ( event.getSubType( ) == 1) ? "Start" : "End";
		String mTitle , mBody , mSource;
		try {
			mTitle = this.translator.translate( language , "imtDebt" + type );
			mBody = this.translator.translate( language , "imDebt" + type );
			mSource = this.translator.translate( language , "imSenderEco" );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return new DebtMessageExtractor( mTitle , mBody , mSource );
	}

}
