package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.AllianceEventRecord;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;



@Component
public class LeftAllianceMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static class LeftAllianceMessageExtractor
			extends AllianceMessageExtractor
	{

		public LeftAllianceMessageExtractor( AllianceEventRecord event , String mTitle , String mBody )
		{
			super( event , mTitle , mBody );
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle.replace( "${empire}" , this.event.getEmpireName( ) );
		}


		@Override
		public String getContents( )
		{
			String empLink;
			if ( this.event.getEmpireId( ) == null ) {
				empLink = this.event.getEmpireName( );
			} else {
				empLink = "{{empire:" + this.event.getEmpireId( ) + " " + this.event.getEmpireName( ) + "}}";
			}
			return this.mBody.replace( "${empire}" , empLink );
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.ALLIANCE , 4 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		AllianceEventRecord event = (AllianceEventRecord) contents;
		String mTitle , mBody;
		try {
			mTitle = this.translator.translate( language , "imtLeftAlliance" );
			mBody = this.translator.translate( language , "imLeftAlliance" );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return new LeftAllianceMessageExtractor( event , mTitle , mBody );
	}

}
