package com.deepclone.lw.beans.msgs.fmt;


import java.util.HashSet;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.msg.MessageExtractor;
import com.deepclone.lw.interfaces.msg.MessageFormatter;
import com.deepclone.lw.sqld.msgs.EventType;
import com.deepclone.lw.sqld.msgs.FormatType;
import com.deepclone.lw.sqld.msgs.InboxRecord;
import com.deepclone.lw.sqld.msgs.MessageDataRecord;
import com.deepclone.lw.sqld.msgs.PlanetEventRecord;



@Component
public class TakenPlanetMessageFormatterBean
		implements MessageFormatter
{

	private Translator translator;

	private static class TakenPlanetMessageExtractor
			extends PlanetMessageExtractor
	{
		private String mTitle;
		private String mBody;
		private String mSource;


		public TakenPlanetMessageExtractor( PlanetEventRecord event , String mTitle , String mBody , String mSource )
		{
			super( event );
			this.mTitle = mTitle;
			this.mBody = mBody;
			this.mSource = mSource;
		}


		@Override
		public String getContents( )
		{
			String locLink = this.getLocation( );
			String empLink;
			if ( this.event.getEmpireName( ) == null ) {
				empLink = "";
			} else if ( this.event.getEmpireId( ) == null ) {
				empLink = this.event.getEmpireName( );
			} else {
				empLink = "{{empire:" + this.event.getEmpireId( ) + " " + this.event.getEmpireName( ) + "}}";
			}
			return this.mBody.replace( "${location}" , locLink ).replace( "${owner}" , empLink );
		}


		@Override
		public String getSender( )
		{
			return this.mSource;
		}


		@Override
		public String getSubject( )
		{
			return this.mTitle.replace( "${location}" , this.event.getLocationName( ) );
		}

	}


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Override
	public Set< FormatType > getFormats( )
	{
		Set< FormatType > fmts = new HashSet< FormatType >( );
		fmts.add( new FormatType( EventType.PLANET , 6 ) );
		return fmts;
	}


	@Override
	public MessageExtractor getExtractor( InboxRecord envelope , MessageDataRecord contents , String language )
	{
		PlanetEventRecord event = (PlanetEventRecord) contents;
		boolean neutral = ( event.getEmpireName( ) == null );
		String mTitle , mBody , mSource;
		try {
			mTitle = this.translator.translate( language , "imtTakePlanet" );
			mBody = this.translator.translate( language , "imTakePlanet" + ( neutral ? "Neutral" : "" ) );
			mSource = this.translator.translate( language , "imSenderWar" );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}
		return new TakenPlanetMessageExtractor( event , mTitle , mBody , mSource );
	}

}
