package com.deepclone.lw.beans.sys;


import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.deepclone.lw.interfaces.sys.ConstantDefinition;
import com.deepclone.lw.interfaces.sys.ConstantsAdministration;
import com.deepclone.lw.interfaces.sys.InvalidConstantValue;
import com.deepclone.lw.interfaces.sys.UnknownConstantError;
import com.deepclone.lw.sqld.sys.Constant;



/**
 * The constants administration implementation mostly serves as a session-specific proxy for the
 * share {@link ConstantsData} instance.
 * 
 * @author tseeker
 */
class ConstantsAdministrationImpl
		implements ConstantsAdministration
{
	/** Shared data storage instance */
	private ConstantsData data;

	private int admin;


	/**
	 * Copies the required references.
	 * 
	 * @param data
	 *            shared data storage instance
	 * @param logger
	 *            session-specific logger
	 */
	ConstantsAdministrationImpl( ConstantsData data , int admin )
	{
		this.data = data;
		this.admin = admin;
	}


	/* Documented in ConstantsAdministration interface */
	@Override
	public Set< String > getCategories( )
	{
		return this.data.getCategories( );
	}


	/* Documented in ConstantsAdministration interface */
	@Override
	public Collection< ConstantDefinition > getConstants( String category )
	{
		List< Constant > constants = this.data.getCategory( category );
		List< ConstantDefinition > defs = new LinkedList< ConstantDefinition >( );
		if ( constants == null ) {
			return defs;
		}

		// Convert data to immutable records
		for ( Constant constant : constants ) {
			ConstantDefinition def;
			if ( constant.getMaxValue( ) == null && constant.getMinValue( ) == null ) {
				def = new ConstantDefinition( constant.getName( ) , category , constant.getDescription( ) , constant
						.getValue( ) );
			} else if ( constant.getMaxValue( ) == null || constant.getMinValue( ) == null ) {
				boolean isMin = ( constant.getMaxValue( ) == null );
				Double val = isMin ? constant.getMinValue( ) : constant.getMaxValue( );
				def = new ConstantDefinition( constant.getName( ) , category , constant.getDescription( ) , constant
						.getValue( ) , val , isMin );
			} else {
				if ( constant.getMaxValue( ) == null && constant.getMinValue( ) == null ) {
					def = new ConstantDefinition( constant.getName( ) , category , constant.getDescription( ) ,
							constant.getValue( ) );
				} else if ( constant.getMaxValue( ) == null ) {
					def = new ConstantDefinition( constant.getName( ) , category , constant.getDescription( ) ,
							constant.getValue( ) , constant.getMinValue( ) , true );
				} else if ( constant.getMinValue( ) == null ) {
					def = new ConstantDefinition( constant.getName( ) , category , constant.getDescription( ) ,
							constant.getValue( ) , constant.getMaxValue( ) , false );
				} else {
					def = new ConstantDefinition( constant.getName( ) , category , constant.getDescription( ) , constant
							.getValue( ) , constant.getMinValue( ) , constant.getMaxValue( ) );
				}
			}
			defs.add( def );
		}

		return defs;
	}


	/* Documented in ConstantsAdministration interface */
	@Override
	public void setConstant( String name , Double value )
			throws UnknownConstantError , InvalidConstantValue
	{
		this.data.setValue( name , value , this.admin );
	}

}
