package com.deepclone.lw.beans.user.abst;


import java.util.HashMap;
import java.util.Map;

import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.interfaces.session.SessionTypeDefiner;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.SessionCommandException;
import com.deepclone.lw.session.SessionStateException;



public abstract class StatefulSessionTypeDefiner
		implements SessionTypeDefiner
{

	private SessionSubTypeDelegate commonDelegate = null;

	private Map< String , SessionSubTypeDelegate > subTypes = new HashMap< String , SessionSubTypeDelegate >( );


	protected abstract String initAuthToken( ServerSession session , String identifier , String sha1Hash ,
			String md5Hash );


	protected abstract String getSessionType( ServerSession session );


	protected final String getAuthToken( ServerSession session )
	{
		return session.get( "authenticationToken" , String.class );
	}


	public final void registerSubType( SessionSubTypeDelegate delegate )
	{
		String name = delegate.getName( );
		synchronized ( this.subTypes ) {
			if ( name == null ) {
				this.commonDelegate = delegate;
			} else {
				this.subTypes.put( delegate.getName( ) , delegate );
			}
		}
	}


	private String updateSessionType( ServerSession session )
	{
		String type = this.getSessionType( session );
		session.put( "sessionState" , type );
		return type;
	}


	@Override
	public final String getState( ServerSession session )
	{
		return session.get( "sessionState" , String.class );
	}


	@Override
	public final void authenticate( ServerSession session , String identifier , String sha1Hash , String md5Hash )
			throws SessionStateException
	{
		if ( this.isAuthenticated( session ) ) {
			throw new SessionStateException( );
		}

		String token = this.initAuthToken( session , identifier , sha1Hash , md5Hash );
		if ( token == null ) {
			return;
		}
		session.put( "authenticationToken" , token );
		this.updateSessionType( session );
	}


	@Override
	public final boolean isAuthenticated( ServerSession session )
	{
		return ( this.getAuthToken( session ) != null );
	}


	@Override
	public final CommandResponse execute( ServerSession session , Command command )
			throws SessionStateException , SessionCommandException
	{
		if ( !this.isAuthenticated( session ) ) {
			throw new SessionStateException( );
		}

		String type = this.updateSessionType( session );
		SessionSubTypeDelegate delegate;
		SessionSubTypeDelegate fallback;
		synchronized ( this.subTypes ) {
			delegate = this.subTypes.get( type );
			fallback = this.commonDelegate;
		}
		if ( delegate == null ) {
			if ( fallback == null ) {
				throw new SessionStateException( );
			} else {
				delegate = fallback;
				fallback = null;
			}
		}

		CommandResponse response;
		try {
			response = delegate.execute( session , command );
		} catch ( SessionCommandException e ) {
			if ( fallback != null ) {
				response = fallback.execute( session , command );
			} else {
				throw e;
			}
		}

		this.updateSessionType( session );
		return response;
	}

}
