package com.deepclone.lw.beans.user.admin;


import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.StatefulSessionTypeDefiner;
import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.interfaces.admin.Administration;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.interfaces.session.SessionManager;
import com.deepclone.lw.sqld.admin.AdminRecord;
import com.deepclone.lw.utils.EmailAddress;



public class AdminSessionDefinerBean
		extends StatefulSessionTypeDefiner
{
	private Administration administration;


	@Autowired( required = true )
	public void setSessionManager( SessionManager manager )
	{
		manager.registerSessionType( this );
	}


	@Autowired( required = true )
	public void setAdministration( Administration administration )
	{
		this.administration = administration;
	}


	@Override
	public String getName( )
	{
		return "admin";
	}


	@Override
	public void initialise( ServerSession session )
	{
		// EMPTY
	}


	@Override
	protected String initAuthToken( ServerSession session , String identifier , String sha1Hash , String md5Hash )
	{
		EmailAddress address = new EmailAddress( identifier );
		if ( !address.isValid( ) ) {
			return null;
		}

		AdminRecord admin = this.administration.login( address , session.getChallenge( ) , sha1Hash , md5Hash , session
				.getAddress( ) );
		if ( admin == null ) {
			return null;
		}

		session.setExpirationDate( new Date( new Date( ).getTime( ) + 600000L ) );
		return ( (Integer) admin.getId( ) ).toString( );
	}


	@Override
	protected String getSessionType( ServerSession session )
	{
		int adminId = Integer.parseInt( session.get( "authenticationToken" , String.class ) );
		AdminRecord admin = this.administration.getAdmin( adminId );
		session.put( "admin" , admin );

		// Administrator has been disabled
		if ( !admin.isActive( ) ) {
			session.terminate( SessionTerminationType.GONE );
			return null;
		}

		return ( admin.getPassChangeRequired( ) ? "pass" : "main" );
	}


	@Override
	public void terminate( ServerSession session , SessionTerminationType reason )
	{
		String authToken = session.get( "authenticationToken" , String.class );
		if ( authToken != null ) {
			int adminId = Integer.parseInt( session.get( "authenticationToken" , String.class ) );
			this.administration.logout( adminId );
		}
	}

}
