package com.deepclone.lw.beans.user.admin.common;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.cmd.admin.SetPasswordCommand;
import com.deepclone.lw.cmd.admin.SetPasswordResponse;
import com.deepclone.lw.cmd.admin.SetPasswordResponse.PasswordChangeStatus;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.interfaces.acm.PasswordProhibitedException;
import com.deepclone.lw.interfaces.admin.Administration;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.Password;



public class SetPasswordCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{
	private Administration administration;
	private int minPasswordStrength = 20;


	@Autowired( required = true )
	public void setAdministration( Administration administration )
	{
		this.administration = administration;
	}


	public void setMinPasswordStrength( int strength )
	{
		this.minPasswordStrength = strength;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return CommonCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return SetPasswordCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		Administrator admin = this.getAdministrator( session );
		SetPasswordCommand command = (SetPasswordCommand) cParam;

		String challenge = session.get( "tempChallenge" , String.class );
		session.put( "tempChallenge" , null );

		PasswordChangeStatus status = PasswordChangeStatus.OK;
		Password pwd = null;
		if ( "".equals( command.getPassword( ) ) ) {
			status = PasswordChangeStatus.EMPTY;
		} else if ( !command.getPassword( ).equals( command.getPasswordConfirm( ) ) ) {
			status = PasswordChangeStatus.MISMATCH;
		} else {
			pwd = new Password( command.getPassword( ) );
			if ( pwd.getStrength( ) < this.minPasswordStrength ) {
				status = PasswordChangeStatus.TOO_WEAK;
			}
		}

		boolean authError;
		if ( status == PasswordChangeStatus.OK ) {
			try {
				authError = !this.administration.setPassword( admin.getId( ) , challenge , command.getSha1Auth( ) ,
						command.getMd5Auth( ) , pwd );
			} catch ( PasswordProhibitedException e ) {
				authError = false;
				status = PasswordChangeStatus.PROHIBITED;
			}
		} else {
			authError = false;
		}

		if ( !authError && status == PasswordChangeStatus.OK ) {
			return new SetPasswordResponse( );
		}
		return new SetPasswordResponse( admin , authError , status );
	}
}
