package com.deepclone.lw.beans.user.admin.main.bt;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.bt.ReportBugResponse;
import com.deepclone.lw.cmd.bt.ReportBugCommand;
import com.deepclone.lw.interfaces.bt.AdminBugs;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class ReportBugCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{
	private AdminBugs bugs;


	@Autowired( required = true )
	public void setBugs( AdminBugs bugs )
	{
		this.bugs = bugs;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return ReportBugCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.BUGT ) ) {
			return new ReportBugResponse( admin );
		}

		ReportBugCommand command = (ReportBugCommand) cParam;
		String title = command.getTitle( ).trim( );
		String desc = command.getDescription( ).trim( );
		boolean isPublic = command.isPublicReport( );

		ObjectNameError titleError;
		if ( "".equals( title ) ) {
			titleError = ObjectNameError.EMPTY;
		} else if ( title.length( ) < 10 || title.length( ) > 127 ) {
			titleError = ObjectNameError.INVALID;
		} else {
			titleError = null;
		}

		ObjectNameError descError;
		if ( "".equals( desc ) ) {
			descError = ObjectNameError.EMPTY;
		} else if ( desc.length( ) < 30 ) {
			descError = ObjectNameError.INVALID;
		} else {
			descError = null;
		}

		if ( descError != null || titleError != null ) {
			return new ReportBugResponse( admin , titleError , title , descError , desc , isPublic );
		}

		return this.bugs.postReport( admin , title , desc , isPublic );
	}
}
