package com.deepclone.lw.beans.user.admin.main.cnst;


import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.constants.Category;
import com.deepclone.lw.cmd.admin.constants.Definition;
import com.deepclone.lw.cmd.admin.constants.SetConstantCommand;
import com.deepclone.lw.cmd.admin.constants.SetConstantResponse;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.interfaces.sys.ConstantDefinition;
import com.deepclone.lw.interfaces.sys.ConstantsAdministration;
import com.deepclone.lw.interfaces.sys.ConstantsManager;
import com.deepclone.lw.interfaces.sys.InvalidConstantValue;
import com.deepclone.lw.interfaces.sys.UnknownConstantError;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class SetConstantCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{

	private ConstantsManager constants;


	@Autowired( required = true )
	public void setConstants( ConstantsManager constants )
	{
		this.constants = constants;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return SetConstantCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.CNST ) ) {
			return new SetConstantResponse( admin , true );
		}

		ConstantsAdministration cAdmin = this.constants.getAdminSession( admin.getId( ) );
		SetConstantCommand command = (SetConstantCommand) cParam;
		try {
			cAdmin.setConstant( command.getName( ) , command.getValue( ) );
			return new SetConstantResponse( admin , false );
		} catch ( UnknownConstantError e ) {
			return new SetConstantResponse( admin , false );
		} catch ( InvalidConstantValue e ) {
			// EMPTY
		}

		List< Category > cats = new LinkedList< Category >( );
		for ( String cName : cAdmin.getCategories( ) ) {
			List< Definition > defs = new LinkedList< Definition >( );
			for ( ConstantDefinition cnst : cAdmin.getConstants( cName ) ) {
				Definition def = new Definition( );
				def.setName( cnst.name );
				def.setDescription( cnst.description );
				def.setValue( cnst.defaultValue );
				def.setMinValue( cnst.minValue );
				def.setMaxValue( cnst.maxValue );
				defs.add( def );
			}

			cats.add( new Category( cName , defs ) );
		}

		return new SetConstantResponse( admin , cats , command.getName( ) , command.getValue( ) );
	}
}
