package com.deepclone.lw.beans.user.admin.main.i18n;


import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.admin.common.AdminOperation;
import com.deepclone.lw.beans.user.admin.main.AdminCommandsBean;
import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.i18n.ChangeLanguageResponse;
import com.deepclone.lw.cmd.admin.i18n.I18NString;
import com.deepclone.lw.cmd.admin.i18n.Language;
import com.deepclone.lw.cmd.admin.i18n.SetStringCommand;
import com.deepclone.lw.cmd.admin.i18n.SetStringResponse;
import com.deepclone.lw.interfaces.i18n.I18NAdministration;
import com.deepclone.lw.interfaces.i18n.I18NManager;
import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.UnknownLanguageException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class SetStringCommandDelegateBean
		extends AdminOperation
		implements AutowiredCommandDelegate
{

	private I18NManager i18n;


	@Autowired( required = true )
	public void setI18n( I18NManager i18n )
	{
		this.i18n = i18n;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return AdminCommandsBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return SetStringCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		Administrator admin = this.getAdministrator( session );
		if ( !admin.hasPrivilege( Privileges.I18N ) ) {
			return new ChangeLanguageResponse( admin );
		}

		SetStringCommand command = (SetStringCommand) cParam;
		String lId = command.getLanguage( );
		String sId = command.getId( );
		String text = command.getText( ).trim( );

		I18NAdministration i18n = this.i18n.getAdminSession( admin.getId( ) );
		if ( !i18n.getLanguages( ).contains( lId ) ) {
			return new SetStringResponse( admin , null );
		}

		Language language = new Language( );
		language.setId( lId );

		if ( !i18n.getStrings( ).contains( sId ) ) {
			try {
				language.setName( i18n.getLanguageName( lId ) );
				language.setCompletion( (int) Math.floor( i18n.getLanguageSupport( lId ) * 100 ) );
			} catch ( UnknownLanguageException e ) {
				throw new RuntimeException( e );
			}
			return new SetStringResponse( admin , language );
		}

		boolean error = "".equals( text );
		List< I18NString > strings = new LinkedList< I18NString >( );
		try {
			if ( error ) {
				for ( String s : i18n.getStrings( ) ) {
					strings.add( new I18NString( s , i18n.getTranslation( lId , s ) ) );
				}
			} else {
				i18n.updateTranslation( lId , sId , text );
			}
			language.setName( i18n.getLanguageName( lId ) );
			language.setCompletion( (int) Math.floor( i18n.getLanguageSupport( lId ) * 100 ) );
		} catch ( TranslationException e ) {
			throw new RuntimeException( e );
		}

		return error ? new SetStringResponse( admin , language , strings , sId ) : new SetStringResponse( admin ,
				language );
	}

}
