package com.deepclone.lw.beans.user.ext;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.cmd.MailError;
import com.deepclone.lw.cmd.PasswordError;
import com.deepclone.lw.cmd.ext.CreateAccountCommand;
import com.deepclone.lw.cmd.ext.CreateAccountResponse;
import com.deepclone.lw.interfaces.acm.AccountMailException;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.i18n.TranslationException;
import com.deepclone.lw.interfaces.i18n.Translator;
import com.deepclone.lw.interfaces.mailer.MailerException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.Password;



public class AccountCreationCommandDelegateBean
		extends LanguageListRequired
		implements AutowiredCommandDelegate
{
	private int minPasswordStrength = 10;
	private AccountManagement accountManagement;


	@Autowired( required = true )
	public void setTranslator( Translator translator )
	{
		this.translator = translator;
	}


	@Autowired( required = true )
	public void setAccountManagement( AccountManagement accountManagement )
	{
		this.accountManagement = accountManagement;
	}


	public void setMinPasswordStrength( int strength )
	{
		this.minPasswordStrength = strength;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return ExternalSessionDefinerBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return CreateAccountCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		CreateAccountCommand command = (CreateAccountCommand) cParam;
		String temp , tempConf;

		// Check mail addresses
		EmailAddress address = null;
		MailError mailError = null;
		temp = command.getMail( );
		tempConf = command.getMailConfirm( );
		if ( temp == null || temp.trim( ).equals( "" ) ) {
			mailError = MailError.EMPTY;
		} else if ( !temp.equals( tempConf ) ) {
			mailError = MailError.MISMATCH;
		} else {
			address = new EmailAddress( temp );
			if ( !address.isValid( ) ) {
				mailError = MailError.INVALID;
			}
		}

		// Check passwords
		Password password = null;
		PasswordError passError = null;
		temp = command.getPassword( );
		tempConf = command.getPasswordConfirm( );
		if ( temp == null || temp.trim( ).equals( "" ) ) {
			passError = PasswordError.EMPTY;
		} else if ( !temp.equals( tempConf ) ) {
			passError = PasswordError.MISMATCH;
		} else {
			password = new Password( temp );
			if ( password.getStrength( ) < this.minPasswordStrength ) {
				passError = PasswordError.TOO_WEAK;
			}
		}

		// Return error response if necessary at this point
		if ( passError != null || mailError != null ) {
			return this.accountValidationError( command , passError , mailError );
		}

		return this.createAccount( address , password , command.getLanguage( ) );
	}


	private CreateAccountResponse createAccount( EmailAddress address , Password password , String language )
	{
		try {
			this.accountManagement.createAccount( address , password , language );
		} catch ( TranslationException e ) {
			return this.accountCreationError( address.getAddress( ) , null , null );
		} catch ( AccountMailException e ) {
			return this.accountCreationError( address.getAddress( ) , MailError.IN_USE , language );
		} catch ( MailerException e ) {
			return this.accountCreationError( address.getAddress( ) , MailError.SEND_FAIL , language );
		}
		return new CreateAccountResponse( address.getAddress( ) , language );
	}


	private CreateAccountResponse accountValidationError( CreateAccountCommand command , PasswordError pError ,
			MailError mError )
	{
		return new CreateAccountResponse( command.getMail( ) , command.getMailConfirm( ) , pError , mError , command
				.getLanguage( ) , this.getSupportedLanguages( ) );
	}


	private CreateAccountResponse accountCreationError( String address , MailError mError , String language )
	{
		return new CreateAccountResponse( address , address , null , mError , language , this.getSupportedLanguages( ) );
	}

}
