package com.deepclone.lw.beans.user.ext;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.cmd.ext.RequestPasswordRecoveryCommand;
import com.deepclone.lw.cmd.ext.RequestPasswordRecoveryResponse;
import com.deepclone.lw.cmd.ext.RequestPasswordRecoveryResponse.PasswordRecoveryRequestStatus;
import com.deepclone.lw.interfaces.acm.AccountMailException;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.acm.PasswordRecoveryException;
import com.deepclone.lw.interfaces.mailer.MailerException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;



public class ReqPwdRecoveryCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AccountManagement accountManagement;


	@Autowired( required = true )
	public void setAccountManagement( AccountManagement accountManagement )
	{
		this.accountManagement = accountManagement;
	}

	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return ExternalSessionDefinerBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return RequestPasswordRecoveryCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		RequestPasswordRecoveryCommand command = (RequestPasswordRecoveryCommand) cParam;
		EmailAddress address = new EmailAddress( command.getMailAddress( ) );
		if ( !address.isValid( ) ) {
			return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.INVALID_INPUT );
		}

		try {
			this.accountManagement.requestPasswordRecovery( address );
		} catch ( AccountMailException e ) {
			return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.ACCOUNT_NOT_FOUND );
		} catch ( MailerException e ) {
			return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.MAIL_ERROR );
		} catch ( PasswordRecoveryException e ) {
			if ( e.statusProblem ) {
				return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.ACCOUNT_STATUS );
			}
			return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.RECOVERY_IN_PROGRESS );
		}

		return new RequestPasswordRecoveryResponse( PasswordRecoveryRequestStatus.OK );
	}

}
