package com.deepclone.lw.beans.user.player;


import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.StatefulSessionTypeDefiner;
import com.deepclone.lw.cmd.admin.users.AccountStatus;
import com.deepclone.lw.cmd.admin.users.SessionTerminationType;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.acm.AccountSession;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.interfaces.session.SessionManager;
import com.deepclone.lw.interfaces.sys.MaintenanceData;
import com.deepclone.lw.interfaces.sys.SystemStatus;
import com.deepclone.lw.sqld.accounts.Account;
import com.deepclone.lw.utils.EmailAddress;



public class PlayerSessionDefinerBean
		extends StatefulSessionTypeDefiner
{
	private SystemStatus systemStatus;
	private AccountManagement accountManagement;
	private EmpireManagement empireManagement;


	@Autowired( required = true )
	public void setSystemStatus( SystemStatus systemStatus )
	{
		this.systemStatus = systemStatus;
	}


	@Autowired( required = true )
	public void setAccountManagement( AccountManagement accountManagement )
	{
		this.accountManagement = accountManagement;
	}


	@Autowired( required = true )
	public void setEmpireManagement( EmpireManagement empireManagement )
	{
		this.empireManagement = empireManagement;
	}


	@Autowired( required = true )
	public void setSessionManager( SessionManager manager )
	{
		manager.registerSessionType( this );
	}


	@Override
	public String getName( )
	{
		return "player";
	}


	@Override
	public void initialise( ServerSession session )
	{
		// EMPTY
	}


	@Override
	protected String getSessionType( ServerSession session )
	{
		// If the system is under maintenance, do not check
		MaintenanceData mData = this.systemStatus.checkMaintenance( );
		if ( mData != null ) {
			return this.getState( session );
		}

		// Get account status
		Account account = this.accountManagement.restoreSession( session.get( "sessionId" , Long.class ) );
		if ( account == null ) {
			session.terminate( SessionTerminationType.GONE );
			return null;
		}
		AccountStatus status = account.getStatus( );

		switch ( status ) {
			case ACTIVE:
			case QUITTING:
			case START_VACATION:
			case VACATION:
				session.put( "vacation" , (Boolean) ( status == AccountStatus.VACATION ) );
				this.updateEmpireId( session );
				return "game";
		}

		session.put( "empireId" , null );
		switch ( status ) {
			case UNCONFIRMED:
			case REACTIVATING:
				return "validation";
			case BANNED:
				return "banned";
			case DISABLED:
				return "disabled";
		}

		return null;
	}


	private void updateEmpireId( ServerSession session )
	{
		if ( session.get( "empireId" , Integer.class ) != null ) {
			return;
		}
		session.put( "empireId" , this.empireManagement.getEmpireId( this.getPlayer( session ) ) );
	}


	@Override
	protected String initAuthToken( ServerSession session , String identifier , String sha1Hash , String md5Hash )
	{
		// If the system is under maintenance, do not check
		MaintenanceData mData = this.systemStatus.checkMaintenance( );
		if ( mData != null ) {
			return null;
		}

		EmailAddress address = new EmailAddress( identifier );
		if ( !address.isValid( ) ) {
			return null;
		}

		AccountSession lResult;
		lResult = this.accountManagement.login( address , session.getChallenge( ) , sha1Hash , md5Hash , session
				.getAddress( ) , session.getClient( ) , session.getIdentifier( ) );
		if ( lResult == null ) {
			return null;
		}

		// FIXME: that should not be hardcoded
		session.setExpirationDate( new Date( new Date( ).getTime( ) + 1800000L ) );
		session.put( "sessionId" , (Long) lResult.session );
		return address.getAddress( );
	}


	@Override
	public void terminate( ServerSession session , SessionTerminationType reason )
	{
		if ( !this.isAuthenticated( session ) ) {
			return;
		}

		this.accountManagement.logout( session.get( "sessionId" , Long.class ) , reason );
	}


	private EmailAddress getPlayer( ServerSession session )
	{
		return new EmailAddress( this.getAuthToken( session ) );
	}
}
