package com.deepclone.lw.beans.user.player.account;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.account.SetAddressCommand;
import com.deepclone.lw.cmd.player.account.SetAddressResponse;
import com.deepclone.lw.cmd.player.account.SetAddressResponse.AddressChangeStatus;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.account.AccountData;
import com.deepclone.lw.interfaces.acm.AccountMailException;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.mailer.MailerException;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;



public class SetAddressCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AccountManagement accountManager;
	private EmpireManagement empireManager;


	@Autowired( required = true )
	public void setAccountManager( AccountManagement accountManager )
	{
		this.accountManager = accountManager;
	}


	@Autowired( required = true )
	public void setEmpireManager( EmpireManagement empireManager )
	{
		this.empireManager = empireManager;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return SetAddressCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		EmailAddress address = new EmailAddress( session.get( "authenticationToken" , String.class ) );
		SetAddressCommand command = (SetAddressCommand) cParam;
		String challenge = session.get( "tempChallenge" , String.class );
		session.put( "tempChallenge" , null );

		EmailAddress nAddress = null;
		AddressChangeStatus mailStatus = AddressChangeStatus.OK;
		String temp = command.getMail( );
		String tempConf = command.getMailConfirm( );
		if ( temp == null || temp.trim( ).equals( "" ) ) {
			mailStatus = AddressChangeStatus.EMPTY;
		} else if ( !temp.equals( tempConf ) ) {
			mailStatus = AddressChangeStatus.MISMATCH;
		} else {
			nAddress = new EmailAddress( temp );
			if ( !address.isValid( ) ) {
				mailStatus = AddressChangeStatus.INVALID;
			}
		}

		boolean authError = false;
		if ( mailStatus == AddressChangeStatus.OK ) {
			try {
				authError = !this.accountManager.setAddress( address , challenge , command.getSha1Auth( ) , command
						.getMd5Auth( ) , nAddress );
			} catch ( AccountMailException e ) {
				mailStatus = AddressChangeStatus.IN_USE;
			} catch ( MailerException e ) {
				mailStatus = AddressChangeStatus.SEND_FAIL;
			}
		}

		if ( !authError && mailStatus == AddressChangeStatus.OK ) {
			return new SetAddressResponse( );
		}

		AccountData accnt = this.accountManager.getAccountPage( address );
		GamePageData gPage = this.empireManager.getGeneralInformation( session.get( "empireId" , Integer.class ) );
		return new SetAddressResponse( gPage , accnt , authError , mailStatus , command.getMail( ) );
	}
}
