package com.deepclone.lw.beans.user.player.account;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.account.SetPasswordCommand;
import com.deepclone.lw.cmd.player.account.SetPasswordResponse;
import com.deepclone.lw.cmd.player.account.SetPasswordResponse.PasswordChangeStatus;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.account.AccountData;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.acm.PasswordProhibitedException;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;
import com.deepclone.lw.utils.Password;



public class SetPasswordCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AccountManagement accountManager;
	private EmpireManagement empireManager;
	private int minPasswordStrength = 10;


	@Autowired( required = true )
	public void setAccountManager( AccountManagement accountManager )
	{
		this.accountManager = accountManager;
	}


	@Autowired( required = true )
	public void setEmpireManager( EmpireManagement empireManager )
	{
		this.empireManager = empireManager;
	}


	public void setMinPasswordStrength( int strength )
	{
		this.minPasswordStrength = strength;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return SetPasswordCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		EmailAddress address = new EmailAddress( session.get( "authenticationToken" , String.class ) );
		SetPasswordCommand command = (SetPasswordCommand) cParam;
		String challenge = session.get( "tempChallenge" , String.class );
		session.put( "tempChallenge" , null );

		PasswordChangeStatus status = PasswordChangeStatus.OK;
		Password pwd = null;
		if ( "".equals( command.getPassword( ) ) ) {
			status = PasswordChangeStatus.EMPTY;
		} else if ( !command.getPassword( ).equals( command.getPasswordConfirm( ) ) ) {
			status = PasswordChangeStatus.MISMATCH;
		} else {
			pwd = new Password( command.getPassword( ) );
			if ( pwd.getStrength( ) < this.minPasswordStrength ) {
				status = PasswordChangeStatus.TOO_WEAK;
			}
		}

		boolean authError;
		if ( status == PasswordChangeStatus.OK ) {
			try {
				authError = !this.accountManager.setPassword( address , challenge , command.getSha1Auth( ) , command
						.getMd5Auth( ) , pwd );
			} catch ( PasswordProhibitedException e ) {
				authError = false;
				status = PasswordChangeStatus.PROHIBITED;
			}
		} else {
			authError = false;
		}

		if ( !authError && status == PasswordChangeStatus.OK ) {
			return new SetPasswordResponse( );
		}

		AccountData accnt = this.accountManager.getAccountPage( address );
		GamePageData gPage = this.empireManager.getGeneralInformation( session.get( "empireId" , Integer.class ) );
		return new SetPasswordResponse( gPage , accnt , authError , status );
	}
}
