package com.deepclone.lw.beans.user.player.account;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.account.ValidateSetAddressCommand;
import com.deepclone.lw.cmd.player.account.ValidateSetAddressResponse;
import com.deepclone.lw.cmd.player.gdata.GamePageData;
import com.deepclone.lw.cmd.player.gdata.account.AccountData;
import com.deepclone.lw.interfaces.acm.AccountManagement;
import com.deepclone.lw.interfaces.game.EmpireManagement;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.utils.EmailAddress;



public class ValidateSetAddressCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AccountManagement accountManager;
	private EmpireManagement empireManager;


	@Autowired( required = true )
	public void setAccountManager( AccountManagement accountManager )
	{
		this.accountManager = accountManager;
	}


	@Autowired( required = true )
	public void setEmpireManager( EmpireManagement empireManager )
	{
		this.empireManager = empireManager;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return ValidateSetAddressCommand.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		ValidateSetAddressCommand command = (ValidateSetAddressCommand) cParam;
		EmailAddress cAddress = new EmailAddress( session.get( "authenticationToken" , String.class ) );

		if ( command.isCancel( ) ) {
			this.accountManager.cancelAddressChange( cAddress );
		} else {
			String code = command.getCode( );
			AccountData data = this.accountManager.confirmAddressChange( cAddress , code );
			if ( data.getAddress( ).equals( cAddress.getAddress( ) ) ) {
				GamePageData gPage = this.empireManager
						.getGeneralInformation( session.get( "empireId" , Integer.class ) );
				return new ValidateSetAddressResponse( gPage , data , code );
			}
			session.put( "authenticationToken" , data.getAddress( ) );
		}

		return new ValidateSetAddressResponse( );
	}
}
