package com.deepclone.lw.beans.user.player.game.alliances;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.ObjectNameError;
import com.deepclone.lw.cmd.player.alliances.AllianceStatusResponse;
import com.deepclone.lw.cmd.player.alliances.CreateAllianceCommand;
import com.deepclone.lw.cmd.player.alliances.CreateAllianceResponse;
import com.deepclone.lw.cmd.player.gdata.alliance.AllianceCreationStatus;
import com.deepclone.lw.interfaces.game.AllianceManagement;
import com.deepclone.lw.interfaces.naming.ObjectNameValidator;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;



public class CreateAllianceCommandDelegateBean
		implements AutowiredCommandDelegate
{

	private AllianceManagement allianceManagement;
	private ObjectNameValidator validator;


	@Autowired( required = true )
	public void setAllianceManagement( AllianceManagement allianceManagement )
	{
		this.allianceManagement = allianceManagement;
	}


	@Autowired( required = true )
	public void setObjectNameValidator( ObjectNameValidator validator )
	{
		this.validator = validator;
	}


	@Override
	public Class< ? extends Command > getType( )
	{
		return CreateAllianceCommand.class;
	}


	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	@Override
	public CommandResponse execute( ServerSession session , Command cParam )
	{
		CreateAllianceCommand command = (CreateAllianceCommand) cParam;
		int empireId = session.get( "empireId" , Integer.class );
		if ( session.get( "vacation" , Boolean.class ) ) {
			return this.allianceManagement.getView( empireId );
		}

		String tag = command.getTag( );
		ObjectNameError tagError = this.validator.validate( tag , 2 , 5 );

		String name = command.getName( );
		ObjectNameError nameError = this.validator.validate( name , 5 , 64 );

		if ( tagError != null || nameError != null ) {
			AllianceStatusResponse r = this.allianceManagement.getView( empireId );
			AllianceCreationStatus status = new AllianceCreationStatus( tag , tagError , name , nameError );
			return new CreateAllianceResponse( r.getPage( ) , r.getAlliance( ) , status );
		}

		return this.allianceManagement.create( empireId , tag , name );
	}

}
