package com.deepclone.lw.beans.user.player.game.planets;


import org.springframework.beans.factory.annotation.Autowired;

import com.deepclone.lw.beans.user.abst.AutowiredCommandDelegate;
import com.deepclone.lw.beans.user.abst.SessionCommandHandler;
import com.deepclone.lw.beans.user.player.GameSubTypeBean;
import com.deepclone.lw.cmd.player.planets.UpdatePlanetMiningSettingsCommand;
import com.deepclone.lw.interfaces.game.resources.ResourcesController;
import com.deepclone.lw.interfaces.session.ServerSession;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.NullResponse;



/**
 * Command handler for {@link UpdatePlanetMiningSettingsCommand}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
class UpdatePlanetMiningSettingsCommandDelegateBean
		implements AutowiredCommandDelegate
{

	/** The resources controller */
	private ResourcesController resourcesController;


	/**
	 * Dependency injector that sets the resources controller
	 * 
	 * @param resourcesController
	 *            the resources controller
	 */
	@Autowired( required = true )
	public void setResourcesController( ResourcesController resourcesController )
	{
		this.resourcesController = resourcesController;
	}


	/** This class handles {@link UpdatePlanetMiningSettingsCommand} instances */
	@Override
	public Class< ? extends Command > getType( )
	{
		return UpdatePlanetMiningSettingsCommand.class;
	}


	/** This class is enabled for the {@link GameSubTypeBean} session type */
	@Override
	public Class< ? extends SessionCommandHandler > getCommandHandler( )
	{
		return GameSubTypeBean.class;
	}


	/** If the empire is not in vacation mode, try updating the planet's mining priorities */
	@Override
	public CommandResponse execute( ServerSession session , Command command )
	{
		if ( !session.get( "vacation" , Boolean.class ) ) {
			UpdatePlanetMiningSettingsCommand settings = (UpdatePlanetMiningSettingsCommand) command;
			int empireId = session.get( "empireId" , Integer.class );
			this.resourcesController.updatePlanetSettings( empireId , settings.getId( ) , settings.getSettings( ) );
		}
		return new NullResponse( );
	}

}
