package com.deepclone.lw.interfaces.game.technologies;


import java.util.List;

import com.deepclone.lw.cmd.player.gdata.empire.ResearchData;



/**
 * Player-oriented data access interface for technologies
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public interface PlayerTechnologiesDAO
{

	/**
	 * Obtain the list of technologies for an empire
	 * 
	 * <p>
	 * Query the database for technology entries as seen from an empire's perspective. The
	 * technologies will be returned with I18N string identifiers for categories, names and
	 * descriptions (unless the details are not available).
	 * 
	 * @param empire
	 *            the empire's identifier
	 * 
	 * @return the list of technology entries
	 */
	public List< ResearchData > getResearchData( int empire );


	/**
	 * Start a research priority update
	 * 
	 * <p>
	 * This method initialises a research priorities update by calling the appropriate stored
	 * procedure.
	 * 
	 * @param empire
	 *            the empire for which an update is being performed
	 * 
	 * @return <code>true</code> on success, <code>false</code> on failure
	 */
	public boolean startPriorityUpdate( int empire );


	/**
	 * Upload a single research priority
	 * 
	 * <p>
	 * This method uploads the priority for one of the in-progress research items. It must be called
	 * only after a research priority update has been initiated.
	 * 
	 * @param identifier
	 *            the technology's identifier
	 * @param priority
	 *            the research priority
	 * 
	 * @return <code>true</code> on success, <code>false</code> on failure
	 */
	public boolean uploadResearchPriority( String identifier , int priority );


	/**
	 * Apply an in-progress research priority update
	 * 
	 * <p>
	 * This method applies a previously uploaded set of research priority updates.
	 * 
	 * @return <code>true</code> on success, <code>false</code> on failure
	 */
	public boolean applyPriorityUpdate( );


	/**
	 * Implement a technology
	 * 
	 * <p>
	 * This method calls the stored procedure which causes an empire to implement a technology.
	 * 
	 * @param empire
	 *            the empire's identifier
	 * @param technology
	 *            the technology's identifier
	 * 
	 * @return <code>true</code> on success, <code>false</code> on failure
	 */
	public boolean implement( int empire , String technology );

}
