package com.deepclone.lw.interfaces.game.updates;


/**
 * Update batch processor interface
 * 
 * <p>
 * This interface must be implemented by all components which provide support for server-side game
 * updates processing. The components will be gathered automatically when the server initialises.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
public interface UpdateBatchProcessor
{

	/**
	 * Obtain the component's supported update type
	 * 
	 * <p>
	 * This method must be implemented to return a string which corresponds to the name of the
	 * update type in the database's <code>sys.update_types</code> registry table.
	 * 
	 * @return the update type supported by the component
	 */
	public String getUpdateType( );


	/**
	 * Process a batch of updates
	 * 
	 * <p>
	 * This method is called by the main update processor in the transaction during which items from
	 * the batch will have been marked for processing. It should process the update, but it must not
	 * modify the update rows in the <code>sys.updates</code> table.
	 * 
	 * @param tickId
	 *            the identifier of the current game update cycle
	 */
	public void processBatch( long tickId );

}
