package com.deepclone.lw.interfaces.mailer;


/**
 * A MailData instance allows an email to be prepared by substituting fields in the message template
 * then sent either synchronously or asynchronously.
 * 
 * @author tseeker
 */
public interface MailData
{

	/**
	 * Sets the value of a template field
	 * 
	 * @param identifier
	 *            the field's identifier
	 * @param value
	 *            the value with which the field is to be replaced
	 * @throws IllegalArgumentException
	 *             if the identifier does not correspond to a field, if the value is null or if the
	 *             field had already been set.
	 */
	public void setData( String identifier , Object value )
			throws IllegalArgumentException;


	/**
	 * Sends the message immediately, returning only after it has been sent.
	 * 
	 * @throws AlreadySentException
	 *             if the message had already been sent
	 * @throws MissingDataException
	 *             if some of the template fields have not been set
	 * @throws NotSentException
	 *             if the mail could not be sent
	 */
	public void sendNow( )
			throws AlreadySentException , MissingDataException , NotSentException;


	/**
	 * Queue the message so that it may be sent asynchronously.
	 * 
	 * @throws AlreadySentException
	 *             if the message had already been sent
	 * @throws MissingDataException
	 *             if some of the template fields have not been set
	 */
	public void queue( )
			throws AlreadySentException , MissingDataException;

}
