package com.deepclone.lw.interfaces.naming;


/**
 * Map object name manager exception
 * 
 * This exception is thrown by {@link MapNamesDAO} to indicate that something went wrong. It
 * carries an error code indicating the nature of the problem.
 * 
 * @author tseeker
 */
public class MapNameException
		extends Exception
{

	private static final long serialVersionUID = 1L;

	/** Possible causes for this exception */
	public static enum Error {
		/** Attempting to use a banned name */
		BANNED ,
		/** Attempting to use a name which already exists in the DB */
		UNAVAILABLE ,
		/** Attempting a rename too early after the previous one */
		TOO_EARLY ,
		/** Attempting to validate a name that's already been validated */
		VALIDATED ,
		/** Attempting to validate a name that's never been modified by a player */
		NOT_RENAMED
	};

	/** Actual error code */
	private Error error;;


	public MapNameException( Error error )
	{
		super( error.toString( ) );
		this.error = error;
	}


	/** @return the error code for the exception */
	public Error getError( )
	{
		return this.error;
	}
}
