package com.deepclone.lw.interfaces.prefs;


import java.util.List;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.player.gdata.account.PrefCategory;



/**
 * Interface for the preference definitions management bean.
 * 
 * @author tseeker
 */
public interface PreferenceDefinitions
{

	/**
	 * Registers a group of preferences.
	 * 
	 * @param name
	 *            programmatic name of the group
	 * @param displayName
	 *            internationalised string identifier
	 * @throws PreferenceDefinitionException
	 *             when the display name string doesn't exist (with the
	 *             {@link PreferenceDefinitionException.Error#MISSING_STRING} error code)
	 * @throws IllegalArgumentException
	 *             if one of the parameters is <code>null</code>
	 */
	public void registerGroup( String name , String displayName )
			throws PreferenceDefinitionException , IllegalArgumentException;


	/**
	 * Registers a new type of user preference.
	 * 
	 * @param name
	 *            programmatic name of the preference
	 * @param group
	 *            programmatic name of the group the preference is a part of
	 * @param displayName
	 *            internationalised string identifier that allows the preference's name to be
	 *            displayed
	 * @param displayDescription
	 *            internationalised string identifier that allows the preference's description to be
	 *            displayed (may be <code>null</code>)
	 * @param defaultValue
	 *            default value of the preference
	 * @throws PreferenceDefinitionException
	 *             when the group doesn't exist (with the
	 *             {@link PreferenceDefinitionException.Error#MISSING_GROUP} error code), when the
	 *             display name or description string doesn't exist (with the
	 *             {@link PreferenceDefinitionException.Error#MISSING_STRING} error code) or if the
	 *             preference already exists but is defined as having a different type (with the
	 *             {@link PreferenceDefinitionException.Error#INVALID_TYPE} error code).
	 * @throws IllegalArgumentException
	 *             if one of the mandatory arguments is <code>null</code>
	 */
	public void registerPreference( String name , String group , String displayName , String displayDescription ,
			Object defaultValue )
			throws PreferenceDefinitionException , IllegalArgumentException;


	public List< PrefCategory > getDefaults( );


	public void setDefault( Administrator admin , String preference , String value );

}
