package com.deepclone.lw.interfaces.sys;


import java.util.Collection;
import java.util.Set;



/**
 * This is the constants administrative interface, which allows constants to be listed and modified
 * by the game's administrators.
 * 
 * @author tseeker
 */
public interface ConstantsAdministration
{

	/** @return the set of constant categories */
	public Set< String > getCategories( );


	/**
	 * List all constants in a category. The constant definition objects that are returned have the
	 * constant's current value set as their default value field.
	 * 
	 * @param category
	 *            the name of the category to list
	 * @return the list of constants in a given category or an empty list if the category does not
	 *         exist
	 */
	public Collection< ConstantDefinition > getConstants( String category );


	/**
	 * Modifies a constant and notifies all constant user components of the change.
	 * 
	 * @param name
	 *            the constant's name
	 * @param value
	 *            the constant's new value
	 * @throws UnknownConstantError
	 *             if the name doesn't match any actual constant
	 * @throws InvalidConstantValue
	 *             if the specified value is out of bounds
	 */
	public void setConstant( String name , Double value )
			throws UnknownConstantError , InvalidConstantValue;
}
