package com.deepclone.lw.cli;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;

import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.remoting.rmi.RmiProxyFactoryBean;



public abstract class CLITool
		extends ToolBase
{

	private String makeRmiUrl( String service )
	{
		return "rmi://localhost:" + this.getRmiPort( ) + "/" + service;
	}


	private void addRMIClient( GenericApplicationContext context , String service , String iface )
	{
		BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition( RmiProxyFactoryBean.class );
		builder.setScope( BeanDefinition.SCOPE_PROTOTYPE );
		builder.addPropertyValue( "serviceInterface" , iface );
		builder.addPropertyValue( "serviceUrl" , this.makeRmiUrl( service ) );
		context.registerBeanDefinition( service , builder.getBeanDefinition( ) );
	}


	protected ApplicationContext createClientContext( String... configurationFiles )
	{
		return this.createClientContext( null , configurationFiles );
	}


	protected ApplicationContext createClientContext( Map< String , String > rmiServices )
	{
		return this.createClientContext( rmiServices , (String[]) null );
	}


	protected ApplicationContext createClientContext( Map< String , String > rmiServices , String... configurationFiles )
	{
		if ( rmiServices == null ) {
			return new ClassPathXmlApplicationContext( configurationFiles , true );
		}

		GenericApplicationContext context = new GenericApplicationContext( );
		for ( Map.Entry< String , String > rmiService : rmiServices.entrySet( ) ) {
			this.addRMIClient( context , rmiService.getKey( ) , rmiService.getValue( ) );
		}
		context.refresh( );
		if ( configurationFiles == null ) {
			return context;
		}
		return new ClassPathXmlApplicationContext( configurationFiles , true , context );
	}


	public static ToolBase create( String tool )
	{
		String cName = "com.deepclone.lw.cli." + tool;
		Class< ? > toolClass;
		Object o;

		try {
			toolClass = Class.forName( cName );
		} catch ( ClassNotFoundException e ) {
			return null;
		}

		try {
			o = toolClass.newInstance( );
		} catch ( Exception e ) {
			return null;
		}

		if ( ! ( o instanceof CLITool ) ) {
			return null;
		}

		return (ToolBase) o;
	}


	protected static String getLine( )
	{
		BufferedReader stdin = new BufferedReader( new InputStreamReader( System.in ) );
		do {
			try {
				return stdin.readLine( );
			} catch ( IOException e ) {
				e.printStackTrace( );
			}
		} while ( true );
	}

}
