package com.deepclone.lw.cli;


import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.context.support.GenericApplicationContext;

import com.deepclone.lw.interfaces.sys.Ticker;
import com.deepclone.lw.interfaces.sys.Ticker.Frequency;



public class Tick
		extends CLITool
{

	public static class FakeTickerBean
			implements Ticker
	{
		Map< Frequency , List< Runnable > > tasks = new HashMap< Frequency , List< Runnable > >( );


		@Override
		public boolean isActive( )
		{
			return true;
		}


		@Override
		public void pause( )
				throws IllegalStateException
		{
			// EMPTY
		}


		@Override
		synchronized public void registerTask( Frequency frequency , String name , Runnable task )
		{
			List< Runnable > tasks = this.tasks.get( frequency );
			if ( tasks == null ) {
				tasks = new LinkedList< Runnable >( );
				this.tasks.put( frequency , tasks );
			}
			tasks.add( task );
		}


		@Override
		public void unpause( )
				throws IllegalStateException
		{
			// EMPTY
		}


		public void runFrequency( Frequency frequency )
		{
			List< Runnable > tasks = this.tasks.get( frequency );
			if ( tasks == null ) {
				return;
			}
			for ( Runnable r : tasks ) {
				r.run( );
			}
		}

	}


	private AbstractApplicationContext getContext( )
	{
		// Load data source and Hibernate properties
		String[] dataConfig = {
				this.getDataSource( ) ,
		};
		FileSystemXmlApplicationContext ctx = new FileSystemXmlApplicationContext( dataConfig );
		ctx.refresh( );

		// Load fake ticker
		GenericApplicationContext gac = new GenericApplicationContext( ctx );
		gac.registerBeanDefinition( "fakeTicker" , BeanDefinitionBuilder.rootBeanDefinition( FakeTickerBean.class )
				.getBeanDefinition( ) );
		gac.refresh( );

		String[] cfg = {
				"configuration/context-configuration.xml" , "configuration/transaction-bean.xml" ,
				"configuration/accounts-beans.xml" , "configuration/bt-beans.xml" , "configuration/eventlog-beans.xml" ,
				"configuration/i18n-beans.xml" , "configuration/mailer-beans.xml" , "configuration/naming-beans.xml" ,
				"configuration/simple-beans.xml" ,
				"configuration/system/constants-manager-bean.xml" , "configuration/system/system-status-bean.xml"
		};
		return new ClassPathXmlApplicationContext( cfg , true , gac );
	}


	@Override
	public void run( )
	{
		AbstractApplicationContext ctx = this.getContext( );
		FakeTickerBean ftb = ctx.getBean( FakeTickerBean.class );
		ftb.runFrequency( Frequency.MINUTE );
		ToolBase.destroyContext( ctx );
	}

}
