package com.deepclone.lw.beans.bt.es;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.beans.bt.es.data.BuildingsInformation;
import com.deepclone.lw.testing.MockResultSet;



/**
 * Tests for the {@link BuildingsInformationMapper} class.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestBuildingsInformationMapper
{

	/** Planet identifier found in the "result" */
	private static final int PLANET_ID = 1;

	/** Building identifier found in the "result" */
	private static final int BUILDING_ID = 2;

	/** Name found in the "result" */
	private static final String BUILDING_NAME = "Test";

	/** Quantity of buildings found in the "result" */
	private static final int QUANTITY = 3;

	/** Damage value found in the "result" */
	private static final double DAMAGE = 4.0;

	/** The fake result set used in the tests */
	private ResultSet resultSet;

	/** The mapper used in the tests */
	private BuildingsInformationMapper mapper;


	/**
	 * Create the contents of the fake result set
	 */
	@Before
	@SuppressWarnings( "unchecked" )
	public void setUp( )
	{
		HashMap< String , Object > row = new HashMap< String , Object >( );
		row.put( "planet_id" , PLANET_ID );
		row.put( "building_id" , BUILDING_ID );
		row.put( "building_name" , BUILDING_NAME );
		row.put( "amount" , QUANTITY );
		row.put( "damage" , DAMAGE );
		this.resultSet = MockResultSet.create( new HashMap[] {
			row
		} );
		this.mapper = new BuildingsInformationMapper( );
	}


	/**
	 * The row mapper returns an instance that contains the necessary data.
	 */
	@Test
	public void testRowMapper( )
			throws SQLException
	{
		this.resultSet.absolute( 1 );
		BuildingsInformation info = this.mapper.mapRow( this.resultSet , 0 );
		assertNotNull( info );
		assertEquals( PLANET_ID , info.getPlanetId( ) );
		assertEquals( BUILDING_ID , info.getId( ) );
		assertEquals( BUILDING_NAME , info.getName( ) );
		assertEquals( QUANTITY , info.getAmount( ) );
		assertEquals( DAMAGE , info.getDamage( ) , 0 );
	}
}
