package com.deepclone.lw.beans.bt.es;


import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.beans.bt.es.data.ResourceProviderInformation;



/**
 * Tests for the {@link PlanetResourceRow} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestPlanetResourceRow
{

	/** Identifier of the planet in the test object */
	private static final int TEST_IDENTIFIER = 42;

	/** Resource name used in the tests */
	private static final String TEST_NAME = "Test";

	/** The planet resource row instance used in tests */
	private PlanetResourceRow prr;


	/**
	 * Create a planet resource row instance with the planet identifier specified by
	 * {@link #TEST_IDENTIFIER}.
	 */
	@Before
	public void setUp( )
	{
		this.prr = new PlanetResourceRow( TEST_IDENTIFIER );
	}


	/**
	 * Planet identifier has been initialised correctly.
	 */
	@Test
	public void testPlanetIdentifier( )
	{
		assertEquals( TEST_IDENTIFIER , this.prr.getPlanetId( ) );
	}


	/**
	 * By default, the resource delta exists, the provider doesn't.
	 */
	@Test
	public void testDefaultValues( )
	{
		assertNotNull( this.prr.getDelta( ) );
		assertNull( this.prr.getProvider( ) );
	}


	/**
	 * Setting and reading the resource provider record.
	 */
	@Test
	public void testSetProvider( )
	{
		ResourceProviderInformation rpi = new ResourceProviderInformation( );
		rpi.setResource( TEST_NAME );
		this.prr.setProvider( rpi );
		assertNotNull( this.prr.getProvider( ) );
		assertEquals( TEST_NAME , this.prr.getProvider( ).getResource( ) );
	}


	/**
	 * Setting the resource name when there is no resource provider record.
	 */
	@Test
	public void testSetNameDeltaOnly( )
	{
		this.prr.setResource( TEST_NAME );
		assertEquals( TEST_NAME , this.prr.getDelta( ).getResource( ) );
		assertNull( this.prr.getProvider( ) );
	}


	/**
	 * Setting the resource name when there are both a resource delta record and a resource provider
	 * record.
	 */
	@Test
	public void testSetNameFull( )
	{
		this.prr.setProvider( new ResourceProviderInformation( ) );
		this.prr.setResource( TEST_NAME );
		assertEquals( TEST_NAME , this.prr.getDelta( ).getResource( ) );
		assertNotNull( this.prr.getProvider( ) );
		assertEquals( TEST_NAME , this.prr.getProvider( ).getResource( ) );
	}

}
