package com.deepclone.lw.beans.bt.es.data;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.Serializable;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.beans.bt.es.data.InvalidDumpContentsException;



/**
 * Tests for the {@link InvalidDumpContentsException} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestInvalidDumpContentsException
{

	/**
	 * A dummy class used to test the {@link InvalidDumpContentsException} class.
	 * 
	 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
	 */
	@SuppressWarnings( "serial" )
	private static final class TestRecord
			implements Serializable
	{
		// EMPTY
	}

	/** Test string used as the "field" parameter */
	private static final String TEST_STRING = "Test";

	/** Exception instance used to run tests */
	private InvalidDumpContentsException exception;


	/**
	 * Set up the test by creating an exception instance using {@link TestRecord} as the target
	 * class and {@link #TEST_STRING} as the field name.
	 */
	@Before
	public void setUp( )
	{
		this.exception = new InvalidDumpContentsException( TestRecord.class , TEST_STRING );
	}


	/**
	 * The constructor must set the record's class and field name.
	 */
	@Test
	public void testExceptionData( )
	{
		assertEquals( TestRecord.class , this.exception.getRecordType( ) );
		assertEquals( TEST_STRING , this.exception.getField( ) );
		assertNotNull( this.exception.getMessage( ) );
	}


	/**
	 * The exception's message must end with the name of the class.
	 */
	@Test
	public void testClassNameInMessage( )
	{
		assertTrue( this.exception.getMessage( ).endsWith( " " + TestRecord.class.getSimpleName( ) ) );
	}


	/**
	 * The exception's message must contain the name of the field.
	 */
	@Test
	public void testFieldNameInMessage( )
	{
		assertTrue( this.exception.getMessage( ).contains( " " + TEST_STRING + " " ) );
	}
}
