package com.deepclone.lw.beans.game.resources;


import static org.junit.Assert.*;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.cmd.player.gdata.AbstractResourceRecord;
import com.deepclone.lw.testing.MockResultSet;



/**
 * Tests of the {@link AbstractResourceMapper} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestAbstractResourceMapper
{
	/** Strings used for the various fields */
	private static final String TEST_STRINGS[] = {
			"1" , "2" , "3" , "4"
	};

	/**
	 * An empty resource record used to test the {@link AbstractResourceMapper}
	 * 
	 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
	 */
	@SuppressWarnings( "serial" )
	private static class EmptyResourceRecord
			extends AbstractResourceRecord
	{
		// EMPTY
	}

	/**
	 * Resource row mapper that calls
	 * {@link AbstractResourceMapper#getResourceDescription(AbstractResourceRecord, ResultSet)}
	 * 
	 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
	 * 
	 */
	private static class EmptyResourceMapper
			extends AbstractResourceMapper< EmptyResourceRecord >
	{

		@Override
		public EmptyResourceRecord mapRow( ResultSet rs , int rowNum )
				throws SQLException
		{
			EmptyResourceRecord record = new EmptyResourceRecord( );
			this.getResourceDescription( record , rs );
			return record;
		}

	}

	/** The result set fed to the resource row mapper */
	private ResultSet resultSet;

	/** The mapper used in the tests */
	private EmptyResourceMapper mapper;


	/**
	 * Create the test mapper and a fake result set with two results: one that includes a category,
	 * and another without category.
	 */
	@Before
	public void setUp( )
	{
		this.mapper = new EmptyResourceMapper( );

		@SuppressWarnings( "unchecked" )
		HashMap< String , Object > rows[] = new HashMap[ 2 ];
		for ( int i = 0 ; i < 2 ; i++ ) {
			HashMap< String , Object > row = new HashMap< String , Object >( );
			row.put( "resource_identifier" , TEST_STRINGS[ 0 ] );
			row.put( "resource_name" , TEST_STRINGS[ 1 ] );
			row.put( "resource_description" , TEST_STRINGS[ 2 ] );
			if ( i == 1 ) {
				row.put( "resource_category" , TEST_STRINGS[ 3 ] );
			}
			rows[ i ] = row;
		}

		this.resultSet = MockResultSet.create( rows );
	}


	/** Test mapping a row with a NULL category */
	@Test
	public void testMapRowWithoutCategory( )
			throws SQLException
	{
		this.resultSet.absolute( 1 );
		EmptyResourceRecord record = this.mapper.mapRow( this.resultSet , 1 );

		assertEquals( TEST_STRINGS[ 0 ] , record.getIdentifier( ) );
		assertEquals( TEST_STRINGS[ 1 ] , record.getTitle( ) );
		assertEquals( TEST_STRINGS[ 2 ] , record.getDescription( ) );
		assertNull( record.getCategory( ) );
	}


	/** Test mapping a row with a non-NULL category */
	@Test
	public void testMapRowWithCategory( )
			throws SQLException
	{
		this.resultSet.absolute( 2 );
		EmptyResourceRecord record = this.mapper.mapRow( this.resultSet , 1 );

		assertEquals( TEST_STRINGS[ 0 ] , record.getIdentifier( ) );
		assertEquals( TEST_STRINGS[ 1 ] , record.getTitle( ) );
		assertEquals( TEST_STRINGS[ 2 ] , record.getDescription( ) );
		assertEquals( TEST_STRINGS[ 3 ] , record.getCategory( ) );
	}

}
