package com.deepclone.lw.beans.game.resources;


import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.cmd.player.gdata.planets.PlanetResourceRecord;
import com.deepclone.lw.cmd.player.gdata.planets.ResourceProviderRecord;
import com.deepclone.lw.testing.MockResultSet;



/**
 * Tests for {@link PlanetResourceMapper}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestPlanetResourceMapper
{

	/** Value used as the resource identifier */
	private static final Object RID_VALUE = "resource id";

	/** Value used as the income */
	private static final long P_INCOME_VALUE = 1;

	/** Value used as the upkeep */
	private static final long P_UPKEEP_VALUE = 2;

	/** Value used as the investment */
	private static final long P_INVEST_VALUE = 3;

	/** Value used as the resource provider's capacity */
	private static final long RP_CAPACITY_VALUE = 5L;

	/** Value used as the resource provider's current quantity */
	private static final long RP_QUANTITY_VALUE = 6L;

	/** Value used as the resource provider's difficulty */
	private static final int RP_DIFF_VALUE = 7;

	/** Value used as the mining setting */
	private static final int MS_WEIGHT_VALUE = 8;

	/** The mapper under test */
	private PlanetResourceMapper mapper;

	/** The fake result set fed to the mapper */
	private ResultSet resultSet;


	/**
	 * Create the mapper and a result set to test it with.
	 */
	@Before
	public void setUp( )
	{
		this.mapper = new PlanetResourceMapper( );

		@SuppressWarnings( "unchecked" )
		HashMap< String , Object > rows[] = new HashMap[ 2 ];
		for ( int i = 0 ; i < 2 ; i++ ) {
			HashMap< String , Object > row = new HashMap< String , Object >( );

			row.put( "resource_identifier" , RID_VALUE );

			row.put( "pres_income" , P_INCOME_VALUE + i );
			row.put( "pres_upkeep" , P_UPKEEP_VALUE + i );
			row.put( "pres_invested" , P_INVEST_VALUE + i );
			if ( i == 1 ) {
				row.put( "resprov_capacity" , RP_CAPACITY_VALUE );
				row.put( "resprov_quantity" , RP_QUANTITY_VALUE );
				row.put( "resprov_difficulty" , RP_DIFF_VALUE );
				row.put( "mset_weight" , MS_WEIGHT_VALUE );
			}

			rows[ i ] = row;
		}
		this.resultSet = MockResultSet.create( rows );
	}


	/**
	 * Planet resource row with no resource provider
	 */
	@Test
	public void testRowWithoutResourceProvider( )
			throws SQLException
	{
		this.resultSet.absolute( 1 );
		PlanetResourceRecord row = this.mapper.mapRow( this.resultSet , 1 );

		assertEquals( RID_VALUE , row.getIdentifier( ) );

		assertEquals( P_INCOME_VALUE , row.getIncome( ) );
		assertEquals( P_UPKEEP_VALUE , row.getUpkeep( ) );
		assertEquals( P_INVEST_VALUE , row.getInvested( ) );

		assertNull( row.getResourceProvider( ) );
	}


	/**
	 * Planet resource row with a resource provider
	 */
	@Test
	public void testRowWithResourceProvider( )
			throws SQLException
	{
		this.resultSet.absolute( 2 );
		PlanetResourceRecord row = this.mapper.mapRow( this.resultSet , 2 );

		assertEquals( RID_VALUE , row.getIdentifier( ) );

		assertEquals( P_INCOME_VALUE + 1 , row.getIncome( ) );
		assertEquals( P_UPKEEP_VALUE + 1 , row.getUpkeep( ) );
		assertEquals( P_INVEST_VALUE + 1 , row.getInvested( ) );

		ResourceProviderRecord rp = row.getResourceProvider( );
		assertNotNull( rp );
		assertEquals( RP_CAPACITY_VALUE , rp.getCapacity( ) );
		assertEquals( RP_QUANTITY_VALUE , rp.getQuantity( ) );
		assertEquals( RP_DIFF_VALUE , rp.getDifficulty( ) );
		assertEquals( MS_WEIGHT_VALUE , rp.getPriority( ) );
	}
}
