package com.deepclone.lw.beans.updates;


import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.interfaces.sys.Ticker.Frequency;
import com.deepclone.lw.testing.MockTicker;



/**
 * Tests for {@link GameUpdateTaskBean}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestGameUpdateTaskBean
{
	/** The mock ticker */
	private MockTicker ticker;

	/** The mock game update processor */
	private MockGameUpdateProcessor processor;

	/** The game update task */
	private GameUpdateTaskBean gut;


	/**
	 * Initialise the mock components and the game update task instance
	 */
	@Before
	public void setUp( )
	{
		this.ticker = new MockTicker( Frequency.MINUTE , "Game update" , GameUpdateTaskBean.class );
		this.processor = new MockGameUpdateProcessor( );

		this.gut = new GameUpdateTaskBean( );
		this.gut.setTicker( this.ticker );
		this.gut.setGameUpdateProcessor( this.processor );
	}


	/**
	 * Check the component's initialisation
	 */
	@Test
	public void testInitialisation( )
	{
		this.gut.afterPropertiesSet( );
		assertTrue( this.processor.wasTryLockCalled( ) );
		assertTrue( this.processor.wasUnlockCalled( ) );
		assertTrue( this.processor.wasEndCycleCalled( ) );
		assertFalse( this.processor.wasExecuteCalled( ) );
	}


	/**
	 * Check the component's run() method
	 */
	@Test
	public void testNormalRun( )
	{
		this.gut.run( );
		assertTrue( this.processor.wasTryLockCalled( ) );
		assertTrue( this.processor.wasUnlockCalled( ) );
		assertFalse( this.processor.wasEndCycleCalled( ) );
		assertTrue( this.processor.wasExecuteCalled( ) );
	}


	/**
	 * Make sure the component is unlocked even if the execution fails
	 */
	@Test
	public void testFailedRun( )
	{
		this.processor.setFailExecute( true );
		try {
			this.gut.run( );
			fail( "Mock processor failed to fail" );
		} catch ( RuntimeException e ) {
			// EMPTY
		}
		assertTrue( this.processor.wasTryLockCalled( ) );
		assertTrue( this.processor.wasUnlockCalled( ) );
		assertFalse( this.processor.wasEndCycleCalled( ) );
		assertTrue( this.processor.wasExecuteCalled( ) );
	}

}
