package com.deepclone.lw.beans.updates;


import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;



/**
 * Tests for {@link GameUpdateTransaction}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestGameUpdateTransaction
{

	/** The mock updates DAO */
	private MockUpdatesDAO updatesDAO;

	/** The dummy batch processor used in the tests */
	private DummyBatchProcessor processor;

	/** The mock processor registry */
	private MockRegistry registry;

	/** The game update transaction used as a "guinea pig" */
	private GameUpdateTransaction transaction;


	/** Set up the mock objects and the guinea pig transaction instance */
	@Before
	public void setUp( )
	{
		this.updatesDAO = new MockUpdatesDAO( );
		this.registry = new MockRegistry( );
		this.registry.put( this.processor = new DummyBatchProcessor( "test" ) );
		this.transaction = new GameUpdateTransaction( this.updatesDAO , this.registry , 1 );
	}


	/** Test return value when the stored procedure says that no more updates are to be processed */
	@Test
	public void testWhenFinished( )
	{
		assertTrue( this.transaction.doInTransaction( null ) );
	}


	/** Test return value when the stored procedure says that more updates are to be processed */
	@Test
	public void testWhenHasMore( )
	{
		this.updatesDAO.values = new String[] {
			null
		};
		assertFalse( this.transaction.doInTransaction( null ) );
	}


	/** Test what happens when the stored procedure indicates the need for local processing */
	@Test
	public void testLocalProcessing( )
	{
		this.updatesDAO.values = new String[] {
			"test"
		};
		assertFalse( this.transaction.doInTransaction( null ) );
		assertEquals( 1 , this.processor.getTicks( ).size( ) );
		assertTrue( this.processor.getTicks( ).contains( 1L ) );
	}


	/**
	 * Test what happens when the stored procedure indicates the need for local processing but the
	 * specified processor does not exist
	 */
	@Test( expected = UnsupportedUpdateException.class )
	public void testMissingProcessor( )
	{
		this.updatesDAO.values = new String[] {
			"missing processor"
		};
		this.transaction.doInTransaction( null );
	}
}
