package com.deepclone.lw.beans.updates;


import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.BeanInitializationException;



/**
 * Tests for {@link ServerProcessorRegistryBean}
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestServerProcessorRegistryBean
{
	/** The "guinea pig" instance */
	private ServerProcessorRegistryBean registry;


	@Before
	public void setUp( )
	{
		this.registry = new ServerProcessorRegistryBean( );
		this.registry.postProcessAfterInitialization( new DummyBatchProcessor( "test" ) , "test" );
	}


	/** The registry returns <code>null</code> when the type has no processor */
	@Test
	public void testMissingReturnsNull( )
	{
		assertNull( this.registry.getProcessorFor( "does not exist" ) );
	}


	/** The registry returns the processor when there is one */
	@Test
	public void testExistingReturnsProcessor( )
	{
		assertNotNull( this.registry.getProcessorFor( "test" ) );
	}


	/** The registry ignores objects which are not batch processors */
	@Test
	public void testIgnoresOtherObjects( )
	{
		this.registry.postProcessAfterInitialization( "test" , "test" );
	}


	/** The registry crashes when two batch processors are defined for the same type */
	@Test( expected = BeanInitializationException.class )
	public void testFailsIfDuplicate( )
	{
		this.registry.postProcessAfterInitialization( new DummyBatchProcessor( "test" ) , "test" );
	}

}
