package com.deepclone.lw.cli.xmlimport.data.resources;


import static org.junit.Assert.assertEquals;

import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.resources.BasicResource;
import com.thoughtworks.xstream.converters.ConversionException;



/**
 * Unit tests for {@link BasicResource}.
 * 
 * @author <a href="mailto:tim@mitheren.com>T. Rosser</a>
 * 
 */
public class TestBasicResource
		extends BaseTest
{
	/** Test loading a valid basic resource */
	@Test
	public void testLoadBasicResourceOK( )
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , "0" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		assertEquals( "test" , instance.getName( ) );
		assertEquals( "test" , instance.getDescription( ) );
		assertEquals( new Integer( 0 ) , instance.getWeight( ) );
		assertEquals( "test" , instance.getCategory( ) );
	}


	/** Test loading a basic resource without name */
	@Test
	public void testLoadResourceNoName( )
	{
		String resource = this.createBasicResourceDefinition( null , "test" , "0" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		assertEquals( null , instance.getName( ) );
		assertEquals( "test" , instance.getDescription( ) );
		assertEquals( new Integer( 0 ) , instance.getWeight( ) );
		assertEquals( "test" , instance.getCategory( ) );
	}


	/** Test loading a basic resource without description */
	@Test
	public void testLoadResourceNoDescription( )
	{
		String resource = this.createBasicResourceDefinition( "test" , null , "0" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		assertEquals( "test" , instance.getName( ) );
		assertEquals( null , instance.getDescription( ) );
		assertEquals( new Integer( 0 ) , instance.getWeight( ) );
		assertEquals( "test" , instance.getCategory( ) );
	}


	/** Test loading a basic resource without weight */
	@Test
	public void testLoadBasicResourceNoWeight( )
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , null , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		assertEquals( "test" , instance.getName( ) );
		assertEquals( "test" , instance.getDescription( ) );
		assertEquals( null , instance.getWeight( ) );
		assertEquals( "test" , instance.getCategory( ) );
	}


	/** Test loading a basic resource without category */
	@Test
	public void testLoadBasicResourceNoCategory( )
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , "0" , null );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		assertEquals( "test" , instance.getName( ) );
		assertEquals( "test" , instance.getDescription( ) );
		assertEquals( new Integer( 0 ) , instance.getWeight( ) );
		assertEquals( null , instance.getCategory( ) );
	}


	/**
	 * Test validating a valid basic resource
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateBasicResourceOK( )
			throws DataImportException
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , "0" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a basic resource without name
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateResourceNoName( )
			throws DataImportException
	{
		String resource = this.createBasicResourceDefinition( null , "test" , "0" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a basic resource without description
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateResourceNoDescription( )
			throws DataImportException
	{
		String resource = this.createBasicResourceDefinition( "test" , null , "0" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a basic resource without weight
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateBasicResourceNoWeight( )
			throws DataImportException
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , null , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a basic resource without category
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateBasicResourceNoCategory( )
			throws DataImportException
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , "0" , null );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a basic resource with a non-numeric string weight
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 * @throws ConversionException
	 *             if XStream cannot convert the text to an object
	 */
	@Test( expected = ConversionException.class )
	public void testValidateBasicResourceStringWeight( )
			throws DataImportException , ConversionException
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , "test" , "test" );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a basic resource with a non-integer weight
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 * @throws ConversionException
	 *             if XStream cannot convert the text to an object
	 */
	@Test( expected = ConversionException.class )
	public void testValidateBasicResourceStringFloat( )
			throws DataImportException , ConversionException
	{
		String resource = this.createBasicResourceDefinition( "test" , "test" , "1.2" , null );
		BasicResource instance = this.createObject( resource , BasicResource.class );
		instance.verifyData( );
	}

}
