package com.deepclone.lw.cli.xmlimport.data.resources;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.resources.NaturalResource;
import com.deepclone.lw.cli.xmlimport.data.resources.ResourceParameter;
import com.thoughtworks.xstream.converters.ConversionException;



/**
 * Unit tests for {@link NaturalResource}.
 * 
 * @author <a href="mailto:tim@mitheren.com>T. Rosser</a>
 * 
 */
public class TestNaturalResource
		extends BaseTest
{
	/** The XML definition for a {@link ResourceParameter} representing a resources quantity */
	private String quantity;

	/** The XML definition for a {@link ResourceParameter} representing a resources quantity */
	private String difficulty;

	/** The XML definition for a {@link ResourceParameter} representing a resources quantity */
	private String recovery;


	/**
	 * Create valid definitions for the ResourceParameters to be used in the NaturalResource tests
	 */
	@Before
	public void setup( )
	{
		this.quantity = this.createResourceParameterDefinition( "quantity" , "0" , "0" );
		this.difficulty = this.createResourceParameterDefinition( "difficulty" , "0" , "0" );
		this.recovery = this.createResourceParameterDefinition( "recovery" , "0" , "0" );
	}


	/** Test loading a valid natural resource */
	@Test
	public void testLoadNaturalResourceOK( )
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , this.quantity ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		assertEquals( new Double( 0 ) , instance.getPresenceProbability( ) );
		if ( instance.getQuantity( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getDeviation( ) );
		} else {
			fail( "Quantity is null" );
		}
		if ( instance.getDifficulty( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getDeviation( ) );
		} else {
			fail( "Difficulty is null" );
		}
		if ( instance.getRecovery( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getDeviation( ) );
		} else {
			fail( "Recovery is null" );
		}

	}


	/** Test loading a valid natural resource with presence-probability giving a Double value */
	@Test
	public void testLoadNaturalResourceWithDoubleOK( )
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "1.2" , this.quantity ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		assertEquals( new Double( 1.2 ) , instance.getPresenceProbability( ) );
		if ( instance.getQuantity( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getDeviation( ) );
		} else {
			fail( "Quantity is null" );
		}
		if ( instance.getDifficulty( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getDeviation( ) );
		} else {
			fail( "Difficulty is null" );
		}
		if ( instance.getRecovery( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getDeviation( ) );
		} else {
			fail( "Recovery is null" );
		}
	}


	/** Test loading a natural resource with no presence-probability */
	@Test
	public void testLoadNaturalResourceNoProbability( )
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , null , this.quantity ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		assertEquals( null , instance.getPresenceProbability( ) );
		if ( instance.getQuantity( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getDeviation( ) );
		} else {
			fail( "Quantity is null" );
		}
		if ( instance.getDifficulty( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getDeviation( ) );
		} else {
			fail( "Difficulty is null" );
		}
		if ( instance.getRecovery( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getDeviation( ) );
		} else {
			fail( "Recovery is null" );
		}
	}


	/** Test loading a natural resource with no quantity */
	@Test
	public void testLoadNaturalResourceNoQuantity( )
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , null ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		assertEquals( new Double( 0 ) , instance.getPresenceProbability( ) );
		assertEquals( null , instance.getQuantity( ) );
		if ( instance.getDifficulty( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getDeviation( ) );
		} else {
			fail( "Difficulty is null" );
		}
		if ( instance.getRecovery( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getDeviation( ) );
		} else {
			fail( "Recovery is null" );
		}
	}


	/** Test loading a natural resource with no difficulty */
	@Test
	public void testLoadNaturalResourceNoDifficulty( )
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , this.quantity ,
				null , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		assertEquals( new Double( 0 ) , instance.getPresenceProbability( ) );

		if ( instance.getQuantity( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getDeviation( ) );
		} else {
			fail( "Quantity is null" );
		}
		assertEquals( null , instance.getDifficulty( ) );
		if ( instance.getRecovery( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getRecovery( ).getDeviation( ) );
		} else {
			fail( "Recovery is null" );
		}
	}


	/** Test loading a natural resource with no recovery */
	@Test
	public void testLoadNaturalResourceNoRecovery( )
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , this.quantity ,
				this.difficulty , null );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		assertEquals( new Double( 0 ) , instance.getPresenceProbability( ) );
		if ( instance.getQuantity( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getQuantity( ).getDeviation( ) );
		} else {
			fail( "Quantity is null" );
		}
		if ( instance.getDifficulty( ) != null ) {
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getAverage( ) );
			assertEquals( new Double( 0 ) , instance.getDifficulty( ).getDeviation( ) );
		} else {
			fail( "Difficulty is null" );
		}
		assertEquals( null , instance.getRecovery( ) );
	}


	/**
	 * Test validating a valid natural resource
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateNaturalResourceOK( )
			throws DataImportException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , this.quantity ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a valid natural resource with presence-probability containing a Double value
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateNaturalResourceWithDoubleOK( )
			throws DataImportException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "1.2" , this.quantity ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a natural resource with no presence-probability
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNaturalResourceNoProbability( )
			throws DataImportException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , null , this.quantity ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a natural resource with no quantity
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNaturalResourceNoQuantity( )
			throws DataImportException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , null ,
				this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a natural resource with no difficulty
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNaturalResourceNoDifficulty( )
			throws DataImportException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , this.quantity ,
				null , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a natural resource with no recovery
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateNaturalResourceNoRecovery( )
			throws DataImportException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "0" , this.quantity ,
				this.difficulty , null );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}


	/**
	 * Test validating a natural resource with a non-numerical string as presence-probability
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 * @throws ConversionException
	 *             if XStream cannot convert the text into an object
	 */
	@Test( expected = ConversionException.class )
	public void testValidateNaturalResourceStringProbability( )
			throws DataImportException , ConversionException
	{
		String resource = this.createNaturalResourceDefinition( "test" , "test" , "0" , "test" , "test" ,
				this.quantity , this.difficulty , this.recovery );
		NaturalResource instance = this.createObject( resource , NaturalResource.class );
		instance.verifyData( );
	}

}
