package com.deepclone.lw.cli.xmlimport.data.resources;


import static org.junit.Assert.assertEquals;

import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.resources.ResourceParameter;
import com.thoughtworks.xstream.converters.ConversionException;



/**
 * Unit tests for {@link ResourceParameter}.
 * 
 * @author <a href="mailto:tim@mitheren.com>T. Rosser</a>
 * 
 */
public class TestResourceParameter
		extends BaseTest
{
	/** Test loading a valid resource parameter */
	@Test
	public void testLoadResourceParameterOK( )
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		assertEquals( new Double( 0 ) , instance.getAverage( ) );
		assertEquals( new Double( 0 ) , instance.getDeviation( ) );
	}


	/** Test loading a valid resource parameter with average containing a Double value */
	@Test
	public void testLoadResourceParameterDoubleAverageOK( )
	{
		String rp = this.createResourceParameterDefinition( "test" , "1.2" , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		assertEquals( new Double( 1.2 ) , instance.getAverage( ) );
		assertEquals( new Double( 0 ) , instance.getDeviation( ) );
	}


	/** Test loading a valid resource parameter with deviation containing a Double value */
	@Test
	public void testLoadResourceParameterDoubleDeviationOK( )
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , "1.2" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		assertEquals( new Double( 0 ) , instance.getAverage( ) );
		assertEquals( new Double( 1.2 ) , instance.getDeviation( ) );
	}


	/** Test loading a valid resource parameter with no average */
	@Test
	public void testLoadResourceParameterNoAverage( )
	{
		String rp = this.createResourceParameterDefinition( "test" , null , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		assertEquals( null , instance.getAverage( ) );
		assertEquals( new Double( 0 ) , instance.getDeviation( ) );
	}


	/** Test loading a valid resource parameter with no deviation */
	@Test
	public void testLoadResourceParameterNoDeviation( )
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , null );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		assertEquals( new Double( 0 ) , instance.getAverage( ) );
		assertEquals( null , instance.getDeviation( ) );
	}


	/**
	 * Test validating a valid resource parameter
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateResourceParameterOK( )
			throws DataImportException
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}


	/**
	 * Test validating a valid resource parameter with average containing a Double value
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateResourceParameterDoubleAverageOK( )
			throws DataImportException
	{
		String rp = this.createResourceParameterDefinition( "test" , "1.2" , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}


	/**
	 * Test validating a valid resource parameter with deviation containing a Double value
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test
	public void testValidateResourceParameterDoubleDeviationOK( )
			throws DataImportException
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , "1.2" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}


	/**
	 * Test validating a resource parameter with no average
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateResourceParameterNoAverage( )
			throws DataImportException
	{
		String rp = this.createResourceParameterDefinition( "test" , null , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}


	/**
	 * Test validating a resource parameter with no deviation
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 */
	@Test( expected = DataImportException.class )
	public void testValidateResourceParameterNoDeviation( )
			throws DataImportException
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , null );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}


	/**
	 * Test validating a resource parameter with a non-numerical string average
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 * @throws ConversionException
	 *             if XStream cannot convert the text into an object
	 */
	@Test( expected = ConversionException.class )
	public void testValidateResourceParameterStringAverage( )
			throws DataImportException , ConversionException
	{
		String rp = this.createResourceParameterDefinition( "test" , "test" , "0" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}


	/**
	 * Test validating a resource parameter with a non-numerical string deviation
	 * 
	 * @throws DataImportException
	 *             if the data is not valid
	 * @throws ConversionException
	 *             if XStream cannot convert the text into an object
	 */
	@Test( expected = ConversionException.class )
	public void testValidateResourceParameterStringDeviation( )
			throws DataImportException , ConversionException
	{
		String rp = this.createResourceParameterDefinition( "test" , "0" , "test" );
		ResourceParameter instance = this.createObject( rp , ResourceParameter.class );
		instance.verifyData( "test" );
	}
}
