package com.deepclone.lw.cli.xmlimport.data.resources;


import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;
import com.deepclone.lw.cli.xmlimport.data.resources.BasicResource;
import com.deepclone.lw.cli.xmlimport.data.resources.Resources;



/**
 * Unit tests for {@link Resources}.
 * 
 * @author <a href="mailto:tim@mitheren.com>T. Rosser</a>
 * 
 */
public class TestResources
		extends BaseTest
{
	/** The basic resource definition used in most tests */
	private String basicResourceDefinition;


	/** Initialise the basic resource definition used in tests */
	@Before
	public void setUp( )
	{
		this.basicResourceDefinition = this.createBasicResourceDefinition( "test" , "test" , "0" , "test" );
	}


	/**
	 * Try loading a valid top-level resource definition and make sure it's contents are correct
	 * 
	 * @throws DataImportException
	 *             if the resource definition check fails
	 */
	@Test
	public void testLoadValidResources( )
			throws DataImportException
	{
		String xml = this.createTopLevel( this.basicResourceDefinition );
		Resources resources = this.createObject( xml , Resources.class );

		int count = 0;
		for ( BasicResource resource : resources ) {
			resource.verifyData( );
			count++;
		}
		assertEquals( 1 , count );
	}


	/**
	 * Try loading an empty top-level element
	 * 
	 * @throws NullPointerException
	 *             if the resource definition check fails
	 */
	@Test( expected = NullPointerException.class )
	public void testLoadEmpty( )
			throws NullPointerException
	{
		String xml = this.createTopLevel( );
		Resources resources = this.createObject( xml , Resources.class );
		resources.iterator( );
	}


	/**
	 * Test validating a valid top-level resource definition
	 * 
	 * @throws DataImportException
	 *             if the resource definition check fails
	 */
	@Test
	public void testValidateOK( )
			throws DataImportException
	{
		String xml = this.createTopLevel( this.basicResourceDefinition );
		Resources resources = this.createObject( xml , Resources.class );
		resources.verifyData( );
	}


	/**
	 * Try rejecting a top-level element that does not contain any resource definitions
	 * 
	 * @throws DataImportException
	 *             if the resource definition check fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateEmpty( )
			throws DataImportException
	{
		String xml = this.createTopLevel( );
		Resources resources = this.createObject( xml , Resources.class );
		resources.verifyData( );
	}


	/**
	 * Try rejecting a top-level element that contains an invalid resource definition
	 * 
	 * @throws DataImportException
	 *             if the resource definition check fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateBadDefinition( )
			throws DataImportException
	{
		String xml = this.createTopLevel( this.createBasicResourceDefinition( null , null , null , null ) );
		Resources resources = this.createObject( xml , Resources.class );
		resources.verifyData( );
	}


	/**
	 * Try rejecting a top-level element that contains two valid but duplicate resource definitions
	 * 
	 * @throws DataImportException
	 *             if the resource definition check fails
	 */
	@Test( expected = DataImportException.class )
	public void testValidateDuplicateDefinition( )
			throws DataImportException
	{
		String xml = this.createTopLevel( this.basicResourceDefinition , this.basicResourceDefinition );
		Resources resources = this.createObject( xml , Resources.class );
		resources.verifyData( );
	}

}