package com.deepclone.lw.cli.xmlimport.data.techs;


import static org.junit.Assert.*;

import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;



/**
 * Unit tests for the {@link Technologies} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestTechnologies
		extends BaseTest
{

	/**
	 * Test loading an empty list of technologies
	 * 
	 * <p>
	 * {@link Technologies#iterator()} will throw {@link NullPointerException} when there are no
	 * definitions. Use that to determine if it is empty.
	 */
	@Test( expected = NullPointerException.class )
	public void testEmpty( )
	{
		String defs = this.createTopLevel( );
		Technologies techs = this.createObject( defs , Technologies.class );
		techs.iterator( );
	}


	/**
	 * Test loading a list of technologies
	 */
	@Test
	public void testTechs( )
	{
		String tDef = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L );
		String defs = this.createTopLevel( tDef );
		Technologies techs = this.createObject( defs , Technologies.class );

		int i = 0;
		for ( Technology technology : techs ) {
			assertNotNull( technology );
			assertEquals( 0 , i );
			i++;
		}
	}


	/**
	 * Test verifying an empty list of technologies
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyEmpty( )
			throws DataImportException
	{
		String defs = this.createTopLevel( );
		Technologies techs = this.createObject( defs , Technologies.class );
		techs.verifyData( );
	}


	/**
	 * Test verifying a list of technologies with an invalid technology in it
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyInvalid( )
			throws DataImportException
	{
		String tDef = this.createTechnology( null , "category" , "discovery" , "description" , 12L , 34L );
		String defs = this.createTopLevel( tDef );
		Technologies techs = this.createObject( defs , Technologies.class );
		techs.verifyData( );
	}


	/**
	 * Test verifying a valid list of technologies
	 */
	@Test
	public void testVerifyOk( )
			throws DataImportException
	{
		String tDef = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L );
		String defs = this.createTopLevel( tDef );
		Technologies techs = this.createObject( defs , Technologies.class );
		techs.verifyData( );
	}

}
