package com.deepclone.lw.cli.xmlimport.data.techs;


import static org.junit.Assert.*;

import org.junit.Test;

import com.deepclone.lw.cli.xmlimport.data.DataImportException;



/**
 * Test the {@link Technology} class
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class TestTechnology
		extends BaseTest
{

	/**
	 * Test loading a definition with no dependencies
	 */
	@Test
	public void testLoadNoDeps( )
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );

		assertNotNull( technology );
		assertEquals( "name" , technology.getName( ) );
		assertEquals( "category" , technology.getCategory( ) );
		assertEquals( "discovery" , technology.getDiscovery( ) );
		assertEquals( "description" , technology.getDescription( ) );
		assertEquals( new Long( 12 ) , technology.getCost( ) );
		assertEquals( new Long( 34 ) , technology.getPoints( ) );

		assertNotNull( technology.getDependencies( ) );
		assertTrue( technology.getDependencies( ).isEmpty( ) );
	}


	/**
	 * Test loading a definition with dependencies
	 */
	@Test
	public void testLoadDeps( )
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L ,
				"dep1" , "dep2" );
		Technology technology = this.createObject( definition , Technology.class );

		assertNotNull( technology );
		assertEquals( "name" , technology.getName( ) );
		assertEquals( "category" , technology.getCategory( ) );
		assertEquals( "discovery" , technology.getDiscovery( ) );
		assertEquals( "description" , technology.getDescription( ) );
		assertEquals( new Long( 12 ) , technology.getCost( ) );
		assertEquals( new Long( 34 ) , technology.getPoints( ) );

		assertNotNull( technology.getDependencies( ) );
		assertEquals( 2 , technology.getDependencies( ).size( ) );
		assertEquals( "dep1" , technology.getDependencies( ).get( 0 ) );
		assertEquals( "dep2" , technology.getDependencies( ).get( 1 ) );
	}


	/**
	 * Test {@link Technology#verifyData()} when the name is null
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyNameNull( )
			throws DataImportException
	{
		String definition = this.createTechnology( null , "category" , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the name is empty
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyNameEmpty( )
			throws DataImportException
	{
		String definition = this.createTechnology( "" , "category" , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the name consists of spaces only
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyNameSpaces( )
			throws DataImportException
	{
		String definition = this.createTechnology( "    " , "category" , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the category is null
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyCategoryNull( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , null , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the category is empty
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyCategoryEmpty( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "" , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the category consists of spaces only
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyCategorySpaces( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "    " , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the discovery string is null
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDiscoveryNull( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , null , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the discovery string is empty
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDiscoveryEmpty( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the discovery string consists of spaces only
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDiscoverySpaces( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "    " , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the description is null
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDescriptionNull( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , null , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the description is empty
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDescriptionEmpty( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the description consists of spaces only
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDescriptionSpaces( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "  " , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the cost is null
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyCostNull( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , null , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the cost is 0
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyCostZero( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 0L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the amount of points is null
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyPointsNull( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , null );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when the amount of points is 0
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyPointsZero( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 0L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} on a valid definition with no dependencies
	 */
	@Test
	public void testVerifyValidNoDeps( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when a dependency is empty
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDepsEmpty( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L , "" );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} when a dependency consists of spaces only
	 */
	@Test( expected = DataImportException.class )
	public void testVerifyDepsSpaces( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L ,
				"   " );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}


	/**
	 * Test {@link Technology#verifyData()} on a valid definition with dependencies
	 */
	@Test
	public void testVerifyValidDeps( )
			throws DataImportException
	{
		String definition = this.createTechnology( "name" , "category" , "discovery" , "description" , 12L , 34L ,
				"dep1" , "dep2" );
		Technology technology = this.createObject( definition , Technology.class );
		technology.verifyData( );
	}

}
