package com.deepclone.lw.testing;


import com.deepclone.lw.interfaces.sys.MaintenanceData;
import com.deepclone.lw.interfaces.sys.MaintenanceStatusException;
import com.deepclone.lw.interfaces.sys.SystemStatus;
import com.deepclone.lw.interfaces.sys.TickStatusException;



/**
 * A mock system status component
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class MockSystemStatus
		implements SystemStatus
{
	/** Maintenance data to return when {@link #checkMaintenance()} is called */
	private MaintenanceData maintenance = null;

	/** Value that determines the behaviour of {@link #startTick()} */
	private long stValue = 0;

	/** Value that determines the behaviour of {@link #checkStuckTick()} */
	private long cstValue = -1;

	/** Was {@link #startTick()} called? */
	private boolean startTickCalled = false;


	/**
	 * Set the maintenance data to return when {@link #checkMaintenance()} is called
	 * 
	 * @param maintenance
	 *            the data to return from {@link #checkMaintenance()}
	 */
	public void setMaintenance( MaintenanceData maintenance )
	{
		this.maintenance = maintenance;
	}


	/**
	 * Set the value that determines the behaviour of {@link #checkStuckTick()}
	 * 
	 * <ul>
	 * <li>-2 will cause a {@link MaintenanceStatusException},
	 * <li>-1 will cause it to return <code>null</code>,
	 * <li>any other value will be returned.
	 * </ul>
	 * 
	 * @param cstValue
	 *            the value
	 */
	public void setCSTBehaviour( long cstValue )
	{
		this.cstValue = cstValue;
	}


	/**
	 * Set the value that determines the behaviour of {@link #startTick()}
	 * 
	 * <ul>
	 * <li>-2 will cause a {@link MaintenanceStatusException},
	 * <li>-1 will cause a {@link TickStatusException},
	 * <li>any other value will be returned.
	 * </ul>
	 * 
	 * @param cstValue
	 *            the value
	 */
	public void setSTBehaviour( long stValue )
	{
		this.stValue = stValue;
	}


	/**
	 * @return <code>true</code> if {@link #startTick()} was called.
	 */
	public boolean wasStartTickCalled( )
	{
		return startTickCalled;
	}


	@Override
	public MaintenanceData checkMaintenance( )
	{
		return this.maintenance;
	}


	@Override
	public void startMaintenance( int adminId , String reason , int duration )
			throws MaintenanceStatusException
	{
		// EMPTY - ignored
	}


	@Override
	public void updateMaintenance( int adminId , int durationFromNow )
			throws MaintenanceStatusException
	{
		// EMPTY - ignored
	}


	@Override
	public void endMaintenance( int adminId )
			throws MaintenanceStatusException
	{
		// EMPTY - ignored
	}


	@Override
	public long startTick( )
			throws TickStatusException , MaintenanceStatusException
	{
		this.startTickCalled = true;
		if ( this.stValue == -1 ) {
			throw new TickStatusException( );
		}
		if ( this.stValue == -2 ) {
			throw new MaintenanceStatusException( );
		}
		return this.stValue;
	}


	@Override
	public Long checkStuckTick( )
			throws MaintenanceStatusException
	{
		if ( this.cstValue == -1 ) {
			return null;
		}
		if ( this.cstValue == -2 ) {
			throw new MaintenanceStatusException( );
		}
		return this.cstValue;
	}

}
