package com.deepclone.lw.testing;


import com.deepclone.lw.interfaces.sys.Ticker;

import static org.junit.Assert.*;



/**
 * A mock ticker component
 * 
 * <p>
 * This mock component can be used to make sure that another component registers some task as
 * expected.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
public class MockTicker
		implements Ticker
{
	/** The task's expected frequency */
	private final Frequency expectedFrequency;

	/** The task's expected name */
	private final String expectedName;

	/** The task's expected type */
	private final Class< ? > expectedType;


	/**
	 * Initialise the mock ticker's expectations
	 * 
	 * @param expectedFrequency
	 *            the task's expected frequency
	 * @param expectedName
	 *            the task's expected name
	 * @param expectedType
	 *            the task's expected type
	 */
	public MockTicker( Frequency expectedFrequency , String expectedName , Class< ? > expectedType )
	{
		this.expectedFrequency = expectedFrequency;
		this.expectedName = expectedName;
		this.expectedType = expectedType;
	}


	@Override
	public void registerTask( Frequency frequency , String name , Runnable task )
	{
		assertEquals( this.expectedFrequency , frequency );
		assertEquals( this.expectedName , name );
		assertTrue( this.expectedType.isInstance( task ) );
	}


	@Override
	public void pause( )
			throws IllegalStateException
	{
		// EMPTY - ignored
	}


	@Override
	public void unpause( )
			throws IllegalStateException
	{
		// EMPTY - ignored
	}


	@Override
	public boolean isActive( )
	{
		return true;
	}

}
