package com.deepclone.lw.utils;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import org.apache.commons.codec.binary.Base64;



/**
 * Helper class for serialisation to Base-64 and de-serialisation from Base-64.
 * 
 * @author tseeker
 */
public final class Base64Serializer
{

	/** Empty, private constructor that prevents instantiation */
	private Base64Serializer( )
	{
		// EMPTY
	}


	/**
	 * Serialises an object and encodes it as Base-64.
	 * 
	 * @param value
	 *            object to serialise
	 * @return Base-64 encoded data resulting from the serialisation
	 */
	public static String encode( Serializable value )
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream( );
		ObjectOutputStream oos;

		try {
			oos = new ObjectOutputStream( baos );
			try {
				oos.writeObject( value );
			} finally {
				try {
					oos.close( );
				} catch ( IOException e ) {
					throw new RuntimeException( e );
				}
			}
		} catch ( IOException e ) {
			throw new RuntimeException( e );
		}

		return Base64.encodeBase64String( baos.toByteArray( ) );
	}


	/**
	 * De-serialises an object from a Base-64 encoded string.
	 * 
	 * @param data
	 *            Base-64 encoded string containing the serialised object
	 * @return de-serialised object
	 */
	public static Serializable decode( String data )
	{
		ByteArrayInputStream bais = new ByteArrayInputStream( Base64.decodeBase64( data ) );
		ObjectInputStream ois;

		try {
			ois = new ObjectInputStream( bais );
			try {
				return (Serializable) ois.readObject( );
			} catch ( ClassNotFoundException e ) {
				throw new RuntimeException( e );
			} finally {
				try {
					ois.close( );
				} catch ( IOException e ) {
					throw new RuntimeException( e );
				}
			}
		} catch ( IOException e ) {
			throw new RuntimeException( e );
		}
	}

}
