package com.deepclone.lw.utils;


import java.util.regex.Pattern;



/**
 * This class provide password-related services, such as strength evaluation and hashing.
 * 
 * @author tseeker
 */
public class Password
{

	/** The various regular expressions used to evaluate a password's strength */
	private static final String regexps[] = {
			"[a-z]" , "[A-Z]" , "\\d" , ".*\\d.+\\d.+\\d" , "[^a-zA-Z\\d]" , "[^a-zA-Z\\d].*[^a-zA-Z\\d]" ,
			"([a-zA-Z].*\\d)|(\\d.*[a-zA-Z])" , "([a-z].*[A-Z])|([A-Z].*[a-z])" ,
			"([^a-zA-Z\\d][a-zA-Z\\d])|([a-zA-Z\\d][^a-zA-Z\\d])"
	};

	/** The scores associated with each regular expression */
	private static final int reScores[] = {
			1 , 5 , 5 , 5 , 5 , 5 , 2 , 2 , 2
	};

	/** Compiled regular expressions for strength evaluation */
	private static Pattern patterns[] = null;


	/**
	 * This method makes sure that strength evaluation RE's have been compiled and can be used.
	 */
	private static void initPatterns( )
	{
		synchronized ( Password.class ) {
			if ( Password.patterns != null ) {
				return;
			}

			Password.patterns = new Pattern[ Password.regexps.length ];
			for ( int i = 0 ; i < Password.regexps.length ; i++ ) {
				Password.patterns[ i ] = Pattern.compile( Password.regexps[ i ] );
			}
		}
	}

	/** The original password string */
	private String password;


	/** Initialise the instance with a specific password string */
	public Password( String password )
	{
		this.password = password;
	}


	/** @return the password's strength */
	public int getStrength( )
	{
		return this.getLengthStrength( ) + this.getPatternStrength( );
	}


	/** @return the password's SHA-1 hash (hex encoded) */
	public String getSha1( )
	{
		return DigestHelper.digest( "sha-1" , this.password );
	}


	/** @return the password's MD5 hash (hex encoded) */
	public String getMd5( )
	{
		return DigestHelper.digest( "md5" , this.password );
	}


	/** @return the password's strength based on regular expressions */
	private int getPatternStrength( )
	{
		int strength = 0;
		Password.initPatterns( );
		for ( int i = 0 ; i < Password.regexps.length ; i++ ) {
			if ( Password.patterns[ i ].matcher( this.password ).find( ) ) {
				strength += Password.reScores[ i ];
			} else {
			}
		}
		return strength;
	}


	/** @return the password's strength based on length */
	private int getLengthStrength( )
	{
		int len = this.password.length( );
		if ( len < 5 ) {
			return 3;
		} else if ( len < 8 ) {
			return 6;
		} else if ( len < 16 ) {
			return 12;
		}
		return 18;
	}

}
