package com.deepclone.lw.utils;


import java.util.HashSet;
import java.util.Set;



/**
 * This class implements a random string generator, for use by stuff like validation tokens, planet
 * names, etc. It can be used either directly or as a bean.
 * 
 * @author tseeker
 */
public class RandomStringGenerator
{

	/** The set of characters that can be used */
	private Character[] characters = {
			'0' , '1'
	};

	/** The length of the stings to generate */
	private int length = 2;


	/**
	 * Updates the instance's character set.
	 * 
	 * @param characters
	 *            a string that contains the new set of characters.
	 * @throws IllegalArgumentException
	 *             if there are less than 2 characters in the set
	 */
	public void setCharacterSet( String characters )
			throws IllegalArgumentException
	{
		Set< Character > chSet = new HashSet< Character >( );
		for ( Character c : characters.toCharArray( ) ) {
			chSet.add( c );
		}

		if ( chSet.size( ) < 2 ) {
			throw new IllegalArgumentException( );
		}

		this.characters = chSet.toArray( new Character[ 1 ] );
	}


	/**
	 * Sets the instance's string length.
	 * 
	 * @param length
	 *            the new string length
	 * @throws IllegalArgumentException
	 *             if the specified length is lower than 2
	 */
	public void setLength( int length )
			throws IllegalArgumentException
	{
		if ( length < 2 ) {
			throw new IllegalArgumentException( );
		}
		this.length = length;
	}


	/**
	 * Generates a random string. The string will have a fixed length and contain only characters
	 * from the generator's character set.
	 * 
	 * @return the generated string
	 */
	public String generate( )
	{
		String str = "";
		for ( int i = 0 ; i < this.length ; i++ ) {
			str += this.getRandomChar( );
		}
		return str;
	}


	/**
	 * @return a random character from the character set.
	 */
	private Character getRandomChar( )
	{
		Double d = Math.random( );
		d *= this.characters.length;
		return this.characters[ (int) Math.floor( d ) ];
	}

}
