package com.deepclone.lw.cmd.admin.adata;


import java.util.ArrayList;
import java.util.List;



public enum Privileges {

	SPAM( 0x00000001 , "Sending in-game spam" ) ,
	I18N( 0x00000002 , "Managing translations" ) ,
	CNST( 0x00000004 , "Modifying game constants" ) ,
	USER( 0x00000008 , "Managing user accounts" ) ,
	BANH( 0x00000010 , "Using the banhammer" ) ,
	TICK( 0x00000020 , "Pausing/restarting server-side tasks" ) ,
	NAME( 0x00000040 , "Validating empire and planet names" ) ,
	PREF( 0x00000080 , "Setting default preferences" ) ,
	LOGS( 0x00000100 , "Browsing server logs" ) ,
	BUGM( 0x00000200 , "Receiving automated error e-mail" ) ,
	BUGT( 0x00000400 , "Managing bugs reported through the bug tracker" ) ,
	MNTM( 0x00000800 , "Activating/disabling maintenance mode" ) ,
	SUPER( 0x80000000 , "Superuser (all privileges + admin management)" );

	private final int bits;
	private final String description;


	private Privileges( int bits , String description )
	{
		this.bits = bits;
		this.description = description;
	}


	public int getBits( )
	{
		return bits;
	}


	public String getDescription( )
	{
		return description;
	}


	public boolean hasPrivilege( int privs )
	{
		return ( privs & this.bits ) != 0 || ( privs & 0x80000000 ) != 0;
	}


	public static List< Privileges > getPrivileges( int privs )
	{
		List< Privileges > result = new ArrayList< Privileges >( );
		for ( Privileges p : Privileges.values( ) ) {
			if ( p.hasPrivilege( privs ) ) {
				result.add( p );
			}
		}
		return result;
	}


	public int grant( int privileges )
	{
		return privileges | this.bits;
	}

}
