package com.deepclone.lw.cmd.player.gdata.empire;


import com.deepclone.lw.cmd.player.gdata.AbstractResourceRecord;



/**
 * Resource economic record
 * 
 * <p>
 * This record describes a resource. It is used in the empire overview's economic information and
 * for mining settings.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 * 
 */
public class EmpireResourceRecord
		extends AbstractResourceRecord
{

	/**
	 * The serialisation version identifier
	 * 
	 * <ul>
	 * <li>Introduced in B6M2 with ID 1
	 * </ul>
	 */
	private static final long serialVersionUID = 1L;

	/** The income for this type of resource over 12h RT / 1 month GT */
	private long income;

	/** The planet upkeep for this type of resource over 12h RT / 1 month GT */
	private long planetUpkeep;

	/** The fleet upkeep for this type of resource over 12h RT / 1 month GT */
	private long fleetUpkeep;

	/** The amount of resources of this type possessed by the empire */
	private long stockpiled;

	/** The resource's mining priority, or <code>null</code> for basic resources */
	private Integer miningPriority;


	/** @return the income for this type of resource over a period of 12h RT / 1 month GT */
	public long getIncome( )
	{
		return this.income;
	}


	/**
	 * Set the income for this type of resource over a period of 12h RT / 1 month GT
	 * 
	 * @param income
	 *            the income for this type of resource over a period of 12h RT / 1 month GT
	 */
	public void setIncome( long income )
	{
		this.income = income;
	}


	/** @return the upkeep for this type of resource over a period of 12h RT / 1 month GT on planets */
	public long getPlanetUpkeep( )
	{
		return this.planetUpkeep;
	}


	/**
	 * Set the upkeep for this type of resource over a period of 12h RT / 1 month GT on planets
	 * 
	 * @param planetUpkeep
	 *            the upkeep for this type of resource over a period of 12h RT / 1 month GT on
	 *            planets
	 */
	public void setPlanetUpkeep( long planetUpkeep )
	{
		this.planetUpkeep = planetUpkeep;
	}


	/** @return the upkeep for this type of resource over a period of 12h RT / 1 month GT on planets */
	public long getFleetUpkeep( )
	{
		return this.fleetUpkeep;
	}


	/**
	 * Set the fleet upkeep for this type of resource over a period of 12h RT / 1 month GT
	 * 
	 * @param fleetUpkeep
	 *            the fleet upkeep for this type of resource over a period of 12h RT / 1 month GT
	 */
	public void setFleetUpkeep( long fleetUpkeep )
	{
		this.fleetUpkeep = fleetUpkeep;
	}


	/** @return the total upkeep (including both fleet and planet upkeep) */
	public long getUpkeep( )
	{
		return this.fleetUpkeep + this.planetUpkeep;
	}


	/** @return the amount of resources of this type the empire possesses */
	public long getStockpiled( )
	{
		return this.stockpiled;
	}


	/**
	 * Set the amount of resources of this type the empire possesses
	 * 
	 * @param stockpiled
	 *            the amount of resources of this type the empire possesses
	 */
	public void setStockpiled( long stockpiled )
	{
		this.stockpiled = stockpiled;
	}


	/**
	 * @return the mining priority for this type of resource, or <code>null</code> if it is a basic
	 *         resource
	 */
	public Integer getMiningPriority( )
	{
		return this.miningPriority;
	}


	/**
	 * Set the mining priority for this type of resource
	 * 
	 * @param miningPriority
	 *            the mining priority for this type of resource
	 */
	public void setMiningPriority( int miningPriority )
	{
		this.miningPriority = miningPriority;
	}

}
