package com.deepclone.lw.cmd.player.gdata.planets;


import com.deepclone.lw.cmd.player.gdata.AbstractResourceRecord;



/**
 * Planet resources information
 * 
 * <p>
 * This class carries information about a resource type, including all required strings, as well as
 * income, upkeep and investment. It also contains information about the planet's resource provider
 * if there is one.
 * 
 * <p>
 * The class is used by {@link PlanetOwnView} to represent all of a planet's resources.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class PlanetResourceRecord
		extends AbstractResourceRecord
{

	/**
	 * The serialisation version identifier
	 * 
	 * <ul>
	 * <li>Introduced in B6M2 with ID 1
	 * </ul>
	 */
	private static final long serialVersionUID = 1L;

	/** Income for 12h RT / 1 month GT */
	private long income;

	/** Upkeep for 12h RT / 1 month GT */
	private long upkeep;

	/** Quantity of that resource that will be used by the planet's queues */
	private long invested;

	/** Information about the planet's resource provider of that type, if there is one */
	private ResourceProviderRecord resourceProvider;


	/** @return the planet's income for that resource, over 12h RT / 1 month GT */
	public long getIncome( )
	{
		return this.income;
	}


	/**
	 * Set the planet's income for that resource
	 * 
	 * @param income
	 *            the planet's income for that resource, over 12h RT / 1 month GT
	 */
	public void setIncome( long income )
	{
		this.income = income;
	}


	/** @return the planet's upkeep for that resource, over 12h RT / 1 month GT */
	public long getUpkeep( )
	{
		return this.upkeep;
	}


	/**
	 * Set the planet's upkeep for that resource
	 * 
	 * @param upkeep
	 *            the planet's upkeep for that resource, over 12h RT / 1 month GT
	 */
	public void setUpkeep( long upkeep )
	{
		this.upkeep = upkeep;
	}


	/** @return the amount of that resource invested in the planet's build queues */
	public long getInvested( )
	{
		return this.invested;
	}


	/**
	 * Set the amount of that resource invested in the planet's build queues
	 * 
	 * @param invested
	 *            the amount of that resource invested in the planet's build queues
	 */
	public void setInvested( long invested )
	{
		this.invested = invested;
	}


	/**
	 * @return the resource provider data on that planet for the current resource type or
	 *         <code>null</code> if there is no resource provider of that type.
	 */
	public ResourceProviderRecord getResourceProvider( )
	{
		return this.resourceProvider;
	}


	/**
	 * Set the resource provider data on that planet for the current resource type
	 * 
	 * @param resourceProvider
	 *            the resource provider data on that planet for the current resource type
	 */
	public void setResourceProvider( ResourceProviderRecord resourceProvider )
	{
		this.resourceProvider = resourceProvider;
	}

}
