package com.deepclone.lw.cmd.player.planets;


import java.util.Map;



/**
 * Command that updates a planet's mining settings
 * 
 * <p>
 * This command must be sent with the identifier of a planet that belongs to the current empire. If
 * the planet doesn't exist, if its ownership changed, or if it's configured to use empire-wide
 * settings, the command will be ignored.
 * 
 * @author <a href="mailto:tseeker@legacyworlds.com">E. Benoît</a>
 */
public class UpdatePlanetMiningSettingsCommand
		extends ViewPlanetCommand
{

	/**
	 * Serialisation version identifier
	 * 
	 * <ul>
	 * <li>Introduced in B6M2 with ID 1
	 * </ul>
	 */
	private static final long serialVersionUID = 1L;

	/** The new mining settings */
	private final Map< String , Integer > settings;


	/**
	 * Initialise the command using a planet identifier and mining settings
	 * 
	 * @param planet
	 *            identifier of the planet to update 
	 * @param settings
	 *            a map that associates resource identifiers to priorities
	 */
	public UpdatePlanetMiningSettingsCommand( int planet , Map< String , Integer > settings )
	{
		super( planet );
		this.settings = settings;
	}


	/** @return the mining settings to apply */
	public Map< String , Integer > getSettings( )
	{
		return this.settings;
	}

}
