package com.deepclone.lw.session;


import java.io.Serializable;



/**
 * Session reference.
 * 
 * <p>
 * Session references are send from the server to the client for all session-related actions. They
 * contain information about the session itself (identifier and type of session) and about its state
 * (authentication and sub-type).
 * 
 * @author tseeker
 */
public final class SessionReference
		implements Serializable
{

	private static final long serialVersionUID = 1L;

	/** String identifier of the session */
	public final String identifier;

	/** Type of the session */
	public final String type;

	/** Whether the session has been authenticated or not */
	public final boolean authenticated;

	/**
	 * Extra state information.
	 * 
	 * <p>
	 * If the session needs authentication, this field contains a challenge to use when
	 * authenticating. If the session has been authenticated, it contains the session's sub-type.
	 */
	public final String extra;


	public SessionReference( String identifier , String type , boolean authenticated , String extra )
	{
		this.identifier = identifier;
		this.type = type;
		this.authenticated = authenticated;
		this.extra = extra;
	}

}
