package com.deepclone.lw.web.admin;


import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.AdminResponse;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.bans.BanType;
import com.deepclone.lw.cmd.admin.bans.BansSummaryResponse;
import com.deepclone.lw.cmd.admin.bans.ListBansResponse;
import com.deepclone.lw.cmd.admin.bans.RejectBanResponse;
import com.deepclone.lw.cmd.admin.bans.RequestBanResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class BanhammerPages
		extends PageControllerBase
{

	@RequestMapping( "/bans" )
	public String bansSummary( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BansSummaryResponse response = this.getSession( AdminSession.class , request ).getBansSummary( );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "bansSummary" , response );
	}


	@RequestMapping( "/bans-{type}" )
	public String listBans( HttpServletRequest request , Model model , @PathVariable( "type" ) String sType )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BanType type;
		try {
			type = BanType.valueOf( sType.toUpperCase( ) );
		} catch ( IllegalArgumentException e ) {
			return this.redirect( "bans" );
		}

		ListBansResponse response = this.getSession( AdminSession.class , request ).getBans( type );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}

		return this.render( model , "internal" , "en" , "bans" , response );
	}


	@RequestMapping( "/request-ban" )
	public String requestBan( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		AdminResponse response = this.getSession( AdminSession.class , request ).noOp( Privileges.BANH );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "banRequest" , response );
	}


	@RequestMapping( value = "/request-ban.action" , method = RequestMethod.POST )
	public String requestBan( HttpServletRequest request , Model model , @RequestParam( "eName" ) String eName ,
			@RequestParam( "eMail" ) String eMail , @RequestParam( "reason" ) String reason )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		boolean empire = "".equals( eMail );
		String user = empire ? eName : eMail;

		RequestBanResponse response = this.getSession( AdminSession.class , request ).requestBan( user , empire ,
				reason );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getError( ) == null ) {
			return this.redirect( "bans" );
		}
		return this.render( model , "internal" , "en" , "banRequest" , response );
	}


	@RequestMapping( value = "reject-ban-{id}" )
	public String rejectBan( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bans-pending" );
		}

		AdminResponse response = this.getSession( AdminSession.class , request ).noOp( Privileges.BANH );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}

		return this.renderMap( model , "internal" , "en" , "banReject" , "admin" , response.getAdmin( ) , "id" , id );
	}


	@RequestMapping( value = "reject-ban-{id}.action" , method = RequestMethod.POST )
	public String rejectBan( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId ,
			@RequestParam( "reason" ) String reason )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bans-pending" );
		}

		RejectBanResponse response = this.getSession( AdminSession.class , request ).rejectBan( id , reason );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( !response.isError( ) ) {
			return this.redirect( "bans-pending" );
		}

		return this.render( model , "internal" , "en" , "banReject" , response );
	}


	@RequestMapping( "confirm-ban-{id}.action" )
	public String confirmBan( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bans-pending" );
		}

		this.getSession( AdminSession.class , request ).confirmBan( id );
		return this.redirect( "bans-pending" );
	}


	@RequestMapping( "lift-ban-{id}.action" )
	public String liftBan( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bans-validated" );
		}

		this.getSession( AdminSession.class , request ).liftBan( id );
		return this.redirect( "bans-validated" );
	}

}
