package com.deepclone.lw.web.admin;


import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.adata.Administrator;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.bt.*;
import com.deepclone.lw.cmd.bt.data.BugEvent;
import com.deepclone.lw.cmd.bt.data.BugStatus;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.msgs.MessageFormatter;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.BugTrackerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class BugTrackerPages
		extends BugTrackerBase
{
	private static final int perPage = 20;

	private MessageFormatter formatter;


	@Autowired( required = true )
	public void setFormatter( MessageFormatter formatter )
	{
		this.formatter = formatter;
	}


	@RequestMapping( "/btracker" )
	public String getSummary( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugsSummaryResponse summary = this.getSession( AdminSession.class , request ).getBugsSummary( );
		if ( !summary.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "bugsSummary" , summary );
	}


	@RequestMapping( "/bugs" )
	public String listBugs( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );
		AdminSession aSession = this.getSession( AdminSession.class , request );
		ListBugsResponse response = aSession.listBugs( query.status , query.ownOnly , query.first , perPage );
		return this.render( model , "internal" , "en" , "bugsList" , response );
	}


	@RequestMapping( "/report-bug" )
	public String showReportForm( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );
		AdminSession aSession = this.getSession( AdminSession.class , request );
		Administrator admin = aSession.noOp( Privileges.BUGT ).getAdmin( );
		return this.renderMap( model , "internal" , "en" , "bugsReport" , "admin" , admin , "query" , query );
	}


	@RequestMapping( value = "/report-bug.action" , method = RequestMethod.POST )
	public String postReport( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @RequestParam( "title" ) String title ,
			@RequestParam( "description" ) String description ,
			@RequestParam( value = "public" , required = false , defaultValue = "0" ) String publicReport )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );
		AdminSession aSession = this.getSession( AdminSession.class , request );
		ReportBugResponse response = aSession.reportBug( title , description , "1".equals( publicReport ) );

		if ( response.getTitle( ) == null ) {
			// Successful post
			String rTo = "bug-" + response.getBugId( ) + this.makeGetParams( query );
			return this.redirect( rTo );
		}

		response = new ReportBugResponse( response , query );
		return this.render( model , "internal" , "en" , "bugsReport" , response );
	}


	@RequestMapping( value = "/bug-{id}" )
	public String viewBug( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugs" + this.makeGetParams( query ) );
		}

		ViewBugResponse response = this.getSession( AdminSession.class , request ).getBugReport( bugId );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getReport( ) == null ) {
			return this.redirect( "bugs" + this.makeGetParams( query ) );
		}

		response = new ViewBugResponse( response , query );
		return this.displayReport( model , response );
	}


	@RequestMapping( value = "/bug-{id}-comment.action" , method = RequestMethod.POST )
	public String commentBug( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId ,
			@RequestParam( "comment" ) String comment , @RequestParam( "visibility" ) String visibility )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		PostCommentResponse response = this.getSession( AdminSession.class , request ).postBugComment( bugId , comment ,
				"1".equals( visibility ) );
		if ( response.isPosted( ) ) {
			String rTo = "bug-" + bugId + this.makeGetParams( query );
			return this.redirect( rTo );
		} else if ( response.getReport( ) == null ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		response = new PostCommentResponse( response , query );
		return this.displayReport( model , response );
	}


	private String displayReport( Model model , ViewBugResponse response )
	{
		for ( BugEvent event : response.getEvents( ) ) {
			if ( event.getTitle( ) != null ) {
				event.setTitle( this.formatter.cleanMessage( event.getTitle( ) ) );
			}
			if ( event.getContents( ) != null ) {
				event.setContents( this.formatter.formatMessage( event.getContents( ) , false ) );
			}
		}
		return this.render( model , "internal" , "en" , "bugsView" , response );
	}


	@RequestMapping( value = "/bug-{id}-comment-moderation.action" , method = RequestMethod.POST )
	public String moderateComment( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId ,
			@RequestParam( "comment" ) String sComment , @RequestParam( "cAction" ) String validate )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		long commentId;
		try {
			commentId = Long.parseLong( sComment );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
		}

		this.getSession( AdminSession.class , request ).moderateBugComment( commentId , "1".equals( validate ) );
		return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
	}


	@RequestMapping( value = "/bug-{id}-validation.action" , method = RequestMethod.POST )
	public String moderateReport( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId ,
			@RequestParam( "nStatus" ) String sNewStatus , @RequestParam( "visibility" ) String visibility ,
			@RequestParam( "credits" ) String sCredits , @RequestParam( "snapshot" ) String keepSnapshot )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		BugStatus newStatus;
		try {
			newStatus = BugStatus.valueOf( sNewStatus );
		} catch ( IllegalArgumentException e ) {
			newStatus = BugStatus.PENDING;
		}

		boolean visible = "1".equals( visibility );
		boolean snapshot = "1".equals( keepSnapshot );
		int grantCredits;
		try {
			grantCredits = Integer.parseInt( sCredits );
		} catch ( NumberFormatException e ) {
			grantCredits = -1;
		}
		if ( grantCredits < 0 || grantCredits > 3 || newStatus == BugStatus.PENDING ) {
			return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
		}

		this.getSession( AdminSession.class , request ).validateReport( bugId , newStatus , visible , grantCredits ,
				snapshot );
		return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
	}


	@RequestMapping( value = "/bug-{id}-status.action" , method = RequestMethod.POST )
	public String setReportStatus( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId ,
			@RequestParam( "nStatus" ) String sNewStatus )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		BugStatus newStatus;
		try {
			newStatus = BugStatus.valueOf( sNewStatus );
		} catch ( IllegalArgumentException e ) {
			newStatus = BugStatus.PENDING;
		}

		if ( newStatus != BugStatus.PENDING ) {
			this.getSession( AdminSession.class , request ).setReportStatus( bugId , newStatus );
		}
		return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
	}


	@RequestMapping( value = "/bug-{id}-visibility.action" , method = RequestMethod.POST )
	public String toggleReportVisibility( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		this.getSession( AdminSession.class , request ).toggleReportVisibility( bugId );
		return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
	}


	@RequestMapping( value = "/bug-{id}-merge.action" , method = RequestMethod.POST )
	public String mergeReports( HttpServletRequest request , Model model ,
			@RequestParam( value = "status" , required = false ) String sStatus ,
			@RequestParam( value = "own" , required = false ) String sOwn ,
			@RequestParam( value = "first" , required = false ) String sFirst , @PathVariable( "id" ) String sId ,
			@RequestParam( "mergeId" ) String sMerge )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		BugQuery query = this.getBugQuery( sStatus , sOwn , sFirst );

		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bugtrack" + this.makeGetParams( query ) );
		}

		long mergeId;
		try {
			mergeId = Long.parseLong( sMerge );
		} catch ( NumberFormatException e ) {
			return this.redirect( "bug-" + bugId + this.makeGetParams( query ) );
		}

		MergeReportsResponse response = this.getSession( AdminSession.class , request ).mergeReports( bugId , mergeId );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getReport( ) == null ) {
			if ( response.getMergeError( ) == null ) {
				return this.redirect( "bug-" + mergeId + this.makeGetParams( query ) );
			} else {
				return this.redirect( "bugtrack" + this.makeGetParams( query ) );
			}
		}

		response = new MergeReportsResponse( response , query );
		return this.displayReport( model , response );
	}


	@RequestMapping( "/bug-{id}-xmlsnapshot" )
	public ResponseEntity< String > downloadSnapshot( HttpServletRequest request , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long bugId;
		try {
			bugId = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return new ResponseEntity< String >( "GTFO" , HttpStatus.NOT_FOUND );
		}

		GetSnapshotResponse response = this.getSession( AdminSession.class , request ).getSnapshot( bugId );
		if ( !response.isPrivilegeOk( ) ) {
			return new ResponseEntity< String >( "GTFO" , HttpStatus.FORBIDDEN );
		} else if ( response.getSnapshot( ) == null ) {
			return new ResponseEntity< String >( "GTFO" , HttpStatus.NOT_FOUND );
		}

		HttpHeaders rHeaders = new HttpHeaders( );
		rHeaders.setContentType( MediaType.APPLICATION_OCTET_STREAM );
		return new ResponseEntity< String >( response.getSnapshot( ) , rHeaders , HttpStatus.OK );
	}
}
