package com.deepclone.lw.web.admin;


import javax.servlet.http.HttpServletRequest;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.i18n.ChangeLanguageResponse;
import com.deepclone.lw.cmd.admin.i18n.GetLanguageResponse;
import com.deepclone.lw.cmd.admin.i18n.I18NString;
import com.deepclone.lw.cmd.admin.i18n.SetStringResponse;
import com.deepclone.lw.cmd.admin.i18n.ViewLanguagesResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.admin.i18ne.LanguageExport;
import com.deepclone.lw.web.admin.i18ne.StringExport;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;
import com.thoughtworks.xstream.XStream;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class I18NPages
		extends PageControllerBase
{

	@RequestMapping( "/i18n" )
	public String listLanguages( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ViewLanguagesResponse response = this.getSession( AdminSession.class , request ).listLanguages( );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "languages" , response );
	}


	@RequestMapping( "/i18n-{language}" )
	public String viewLanguage( HttpServletRequest request , Model model , @PathVariable( "language" ) String language )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		GetLanguageResponse response = this.getSession( AdminSession.class , request ).getLanguage( language );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getLanguage( ) == null ) {
			return this.redirect( "i18n" );
		}
		return this.render( model , "internal" , "en" , "language" , response );
	}


	@RequestMapping( "/i18n-export-{language}" )
	public ResponseEntity< String > exportLanguage( HttpServletRequest request ,
			@PathVariable( "language" ) String language )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		HttpHeaders rHeaders = new HttpHeaders( );
		GetLanguageResponse response = this.getSession( AdminSession.class , request ).getLanguage( language );

		if ( !response.isPrivilegeOk( ) ) {
			return new ResponseEntity< String >( "GTFO" , HttpStatus.FORBIDDEN );
		} else if ( response.getLanguage( ) == null ) {
			return new ResponseEntity< String >( "GTFO" , HttpStatus.NOT_FOUND );
		}

		LanguageExport lExport = new LanguageExport( );
		lExport.id = response.getLanguage( ).getId( );
		lExport.name = response.getLanguage( ).getName( );
		for ( I18NString str : response.getStrings( ) ) {
			StringExport sExport = new StringExport( );
			sExport.id = str.getId( );
			sExport.value = str.getText( );
			lExport.strings.add( sExport );
		}

		XStream xStream = new XStream( );
		xStream.autodetectAnnotations( true );
		rHeaders.setContentType( MediaType.APPLICATION_OCTET_STREAM );
		return new ResponseEntity< String >( xStream.toXML( lExport ) , rHeaders , HttpStatus.OK );
	}


	@RequestMapping( value = "/i18n-{language}-edit.action" , method = RequestMethod.POST )
	public String changeLanguage( HttpServletRequest request , Model model ,
			@PathVariable( "language" ) String language , @RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ChangeLanguageResponse response;
		response = this.getSession( AdminSession.class , request ).setLanguageName( language , name );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getLanguage( ) == null ) {
			return this.redirect( "i18n" );
		} else if ( !response.isNameError( ) ) {
			return this.redirect( "i18n-" + response.getLanguage( ).getId( ) );
		}
		return this.render( model , "internal" , "en" , "language" , response );
	}


	@RequestMapping( value = "/i18n-{language}-set-string.action" , method = RequestMethod.POST )
	public String setString( HttpServletRequest request , Model model , @PathVariable( "language" ) String language ,
			@RequestParam( "string" ) String string , @RequestParam( "value" ) String value )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		SetStringResponse response;
		response = this.getSession( AdminSession.class , request ).setTranslation( language , string , value );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getLanguage( ) == null ) {
			return this.redirect( "i18n" );
		} else if ( response.getEdited( ) == null ) {
			return this.redirect( "i18n-" + response.getLanguage( ).getId( ) );
		}
		return this.render( model , "internal" , "en" , "language" , response );
	}
}
