package com.deepclone.lw.web.admin;


import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.deepclone.lw.cmd.admin.logs.GetEntryResponse;
import com.deepclone.lw.cmd.admin.logs.LogLevel;
import com.deepclone.lw.cmd.admin.logs.LogType;
import com.deepclone.lw.cmd.admin.logs.ViewLogResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class LogPages
		extends PageControllerBase
{
	private final int pageSize = 30;


	@RequestMapping( value = "/logs" , method = RequestMethod.GET )
	public String viewLogs( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int first = this.getFirstIndex( request );
		LogType logType = this.getLogType( request );
		LogLevel logLevel = this.getLogLevel( request );
		String component = request.getParameter( "component" );
		if ( component == null ) {
			component = "";
		} else {
			component = component.trim( ).toLowerCase( );
		}
		boolean excOnly = "1".equals( request.getParameter( "excOnly" ) );

		AdminSession aSession = this.getSession( AdminSession.class , request );
		ViewLogResponse response = aSession.viewLog( logType , first , pageSize , logLevel , component , excOnly );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}

		Map< String , Object > params = new HashMap< String , Object >( );
		params.put( "type" , logType );
		params.put( "level" , logLevel );
		params.put( "component" , component );
		params.put( "excOnly" , (Boolean) excOnly );
		params.put( "first" , first );

		Map< String , Object > data = new HashMap< String , Object >( );
		data.put( "admin" , response.getAdmin( ) );
		data.put( "entries" , response.getEntries( ) );
		data.put( "count" , response.getCount( ) );
		data.put( "params" , params );

		return this.render( model , "internal" , "en" , "logs" , data );
	}


	private int getFirstIndex( HttpServletRequest request )
	{
		String sIndex = request.getParameter( "first" );
		if ( sIndex == null ) {
			return 0;
		}
		int f;
		try {
			f = Integer.parseInt( sIndex );
		} catch ( NumberFormatException e ) {
			return 0;
		}
		return ( f > 0 ) ? f : 0;
	}


	private LogType getLogType( HttpServletRequest request )
	{
		LogType logType;
		String sLogType = request.getParameter( "logType" );
		if ( sLogType == null ) {
			logType = LogType.SYSTEM;
		} else {
			try {
				logType = LogType.valueOf( sLogType );
			} catch ( IllegalArgumentException e ) {
				logType = LogType.SYSTEM;
			}
		}
		return logType;
	}


	private LogLevel getLogLevel( HttpServletRequest request )
	{
		LogLevel logLevel;
		String sLogLevel = request.getParameter( "logLevel" );
		if ( sLogLevel == null ) {
			logLevel = LogLevel.INFO;
		} else {
			try {
				logLevel = LogLevel.valueOf( sLogLevel );
			} catch ( IllegalArgumentException e ) {
				logLevel = LogLevel.INFO;
			}
		}
		return logLevel;
	}


	@RequestMapping( "/view-log-entry-{id}" )
	public String viewLogEntry( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long id;
		try {
			id = Long.parseLong( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "logs" );
		}

		GetEntryResponse response = this.getSession( AdminSession.class , request ).getLogEntry( id );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getEntry( ) == null ) {
			return this.redirect( "logs" );
		}

		return this.render( model , "internal" , "en" , "logEntry" , response );
	}
}
