package com.deepclone.lw.web.admin;


import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.mntm.*;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class MaintenancePages
		extends PageControllerBase
{

	@RequestMapping( "/maintenance" )
	public String getMaintenanceStatus( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		MaintenanceStatusResponse response = this.getSession( AdminSession.class , request ).getMaintenanceStatus( );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "maintenance" , response );
	}


	@RequestMapping( value = "/maintenance-start.action" , method = RequestMethod.POST )
	public String enableMaintenance( HttpServletRequest request , Model model ,
			@RequestParam( "reason" ) String reason , @RequestParam( "duration" ) String sDuration )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int duration;
		try {
			duration = Integer.parseInt( sDuration );
		} catch ( NumberFormatException e ) {
			duration = -1;
		}

		AdminSession aSession = this.getSession( AdminSession.class , request );
		MaintenanceChangeResponse response = aSession.enableMaintenance( reason , duration );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getNewReason( ) == null ) {
			return this.redirect( "maintenance" );
		}

		return this.render( model , "internal" , "en" , "maintenance" , response );
	}


	@RequestMapping( value = "/maintenance-extend.action" , method = RequestMethod.POST )
	public String extendMaintenance( HttpServletRequest request , Model model ,
			@RequestParam( "duration" ) String sDuration )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int duration;
		try {
			duration = Integer.parseInt( sDuration );
		} catch ( NumberFormatException e ) {
			duration = -1;
		}

		this.getSession( AdminSession.class , request ).extendMaintenance( duration );
		return this.redirect( "maintenance" );
	}


	@RequestMapping( value = "/maintenance-end.action" , method = RequestMethod.POST )
	public String endMaintenance( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.getSession( AdminSession.class , request ).endMaintenance( );
		return this.redirect( "maintenance" );
	}

}
