package com.deepclone.lw.web.admin;


import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.naming.*;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class NamesPages
		extends PageControllerBase
{

	@RequestMapping( "/names" )
	public String listNameTypes( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		NamesSummaryResponse response = this.getSession( AdminSession.class , request ).getNameSummary( );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "namesSummary" , response );
	}


	@RequestMapping( "/names-{type}" )
	public String listNames( HttpServletRequest request , Model model , @PathVariable( "type" ) String sType )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		NameType type;
		try {
			type = NameType.valueOf( sType.toUpperCase( ).replace( "-" , "_" ) );
		} catch ( IllegalArgumentException e ) {
			return this.redirect( "names" );
		}

		GetNamesResponse response = this.getSession( AdminSession.class , request ).getNames( type );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "names" , response );
	}


	@RequestMapping( value = "/names-{type}.action" , method = RequestMethod.POST )
	public String namesAction( HttpServletRequest request , Model model , @PathVariable( "type" ) String sType ,
			@RequestParam( "action" ) String sAction , @RequestParam( value = "ids" , required = false ) String[] sIds )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		// Parse type
		NameType type;
		try {
			type = NameType.valueOf( sType.toUpperCase( ).replace( "-" , "_" ) );
		} catch ( IllegalArgumentException e ) {
			return this.redirect( "names" );
		}

		// Parse action
		NameAction action;
		try {
			action = NameAction.valueOf( sAction );
		} catch ( IllegalArgumentException e ) {
			return this.redirect( "names-" + type.toString( ).toLowerCase( ).replace( "_" , "-" ) );
		}

		// Parse identifiers
		List< Integer > lIds = new LinkedList< Integer >( );
		if ( sIds != null ) {
			for ( String s : sIds ) {
				try {
					lIds.add( Integer.parseInt( s ) );
				} catch ( NumberFormatException e ) {
					// EMPTY
				}
			}
		}
		if ( lIds.isEmpty( ) ) {
			return this.redirect( "names-" + type.toString( ).toLowerCase( ).replace( "_" , "-" ) );
		}
		int[] ids = new int[ lIds.size( ) ];
		int i = 0;
		for ( Integer v : lIds ) {
			ids[ i++ ] = v.intValue( );
		}

		this.getSession( AdminSession.class , request ).namesAction( type , action , ids );
		return this.redirect( "names-" + type.toString( ).toLowerCase( ).replace( "_" , "-" ) );
	}
}
