package com.deepclone.lw.web.admin;


import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.AdminResponse;
import com.deepclone.lw.cmd.admin.adata.Privileges;
import com.deepclone.lw.cmd.admin.su.AddAdministratorResponse;
import com.deepclone.lw.cmd.admin.su.ListAdministratorsResponse;
import com.deepclone.lw.cmd.admin.su.ViewAdministratorResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class SuperUserPages
		extends PageControllerBase
{

	@RequestMapping( "/admins" )
	public String listAdministrators( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ListAdministratorsResponse response = this.getSession( AdminSession.class , request ).listAdmins( );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "admins" , response );
	}


	@RequestMapping( "/add-admin" )
	public String addAdministrator( HttpServletRequest request , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		AdminResponse response = this.getSession( AdminSession.class , request ).noOp( Privileges.SUPER );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "addAdmin" , response );
	}


	@RequestMapping( value = "/add-admin.action" , method = RequestMethod.POST )
	public String addAdministrator( HttpServletRequest request , Model model ,
			@RequestParam( "address" ) String address , @RequestParam( "name" ) String name ,
			@RequestParam( value = "privileges" , required = false ) String[] sPrivs )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		List< Privileges > privs = this.getPrivileges( sPrivs );
		AddAdministratorResponse response;
		response = this.getSession( AdminSession.class , request ).addAdmin( address , name , privs );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( !response.isError( ) ) {
			return this.redirect( "admins" );
		}

		return this.render( model , "internal" , "en" , "addAdmin" , response );
	}


	@RequestMapping( "/admin-{id}" )
	public String viewAdministrator( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "admins" );
		}

		ViewAdministratorResponse response = this.getSession( AdminSession.class , request ).viewAdministrator( id );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getView( ) == null ) {
			return this.redirect( "admins" );
		}
		return this.render( model , "internal" , "en" , "viewAdmin" , response );
	}


	@RequestMapping( "/reset-admin-{id}.action" )
	public String resetAdministrator( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "admins" );
		}

		ViewAdministratorResponse response = this.getSession( AdminSession.class , request ).viewAdministrator( id );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getView( ) == null || response.getView( ).getPrivileges( ).isEmpty( ) ) {
			return this.redirect( "admins" );
		}
		return this.render( model , "internal" , "en" , "resetAdmin" , response );
	}


	@RequestMapping( value = "/do-reset-admin-{id}.action" , method = RequestMethod.POST )
	public String confirmResetAdministrator( HttpServletRequest request , Model model ,
			@PathVariable( "id" ) String sId , @RequestParam( value = "cancel" , required = false ) String sCancel )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "admins" );
		}

		if ( sCancel != null ) {
			return this.redirect( "admin-" + id );
		}

		ViewAdministratorResponse response = this.getSession( AdminSession.class , request ).resetAdministrator( id );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getView( ) == null ) {
			return this.redirect( "admins" );
		}
		return this.redirect( "admin-" + id );
	}


	@RequestMapping( value = "/admin-privileges-{id}.action" , method = RequestMethod.POST )
	public String setPrivileges( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId ,
			@RequestParam( value = "privileges" , required = false ) String[] sPrivs )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "admins" );
		}

		List< Privileges > privs = this.getPrivileges( sPrivs );
		ViewAdministratorResponse response;
		response = this.getSession( AdminSession.class , request ).setAdminPrivileges( id , privs );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getView( ) == null ) {
			return this.redirect( "admins" );
		}
		return this.redirect( "admin-" + id );
	}


	private List< Privileges > getPrivileges( String[] sPrivs )
	{
		List< Privileges > privs = new LinkedList< Privileges >( );
		if ( sPrivs != null ) {
			for ( String sPriv : sPrivs ) {
				try {
					privs.add( Privileges.valueOf( sPriv ) );
				} catch ( IllegalArgumentException e ) {
					// EMPTY
				}
			}
		}
		return privs;
	}

}
