package com.deepclone.lw.web.admin;


import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import com.deepclone.lw.cmd.admin.users.AccountStatus;
import com.deepclone.lw.cmd.admin.users.ListAccountsResponse;
import com.deepclone.lw.cmd.admin.users.ListSessionsResponse;
import com.deepclone.lw.cmd.admin.users.ViewAccountResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.AdminSession;



@Controller
@SessionRequirement( value = true , subType = "main" , redirectTo = "admin-session" )
public class UsersPages
		extends PageControllerBase
{

	@RequestMapping( "/users" )
	public String listUsers( HttpServletRequest request , Model model ,
			@RequestParam( value = "online" , required = false ) String sOnline )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		AdminSession aSession = this.getSession( AdminSession.class , request );
		ListAccountsResponse list;
		if ( "1".equals( sOnline ) ) {
			list = aSession.listOnlineUsers( );
		} else {
			list = aSession.listUsers( );
		}
		if ( !list.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}

		return this.render( model , "internal" , "en" , "users" , list );
	}


	@RequestMapping( "/users-by-status" )
	public String listUsers( HttpServletRequest request , Model model , @RequestParam( "status" ) String sStatus ,
			@RequestParam( value = "online" , required = false ) String sOnline )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		AccountStatus status;
		try {
			status = AccountStatus.valueOf( sStatus );
		} catch ( IllegalArgumentException e ) {
			return this.redirect( "users" + ( "1".equals( sOnline ) ? "?online=1" : "" ) );
		}

		AdminSession aSession = this.getSession( AdminSession.class , request );
		ListAccountsResponse list;
		if ( "1".equals( sOnline ) ) {
			list = aSession.listOnlineUsers( status );
		} else {
			list = aSession.listUsers( status );
		}
		if ( !list.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		}
		return this.render( model , "internal" , "en" , "users" , list );
	}


	@RequestMapping( "/user-{id}" )
	public String viewUser( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "users" );
		}

		ViewAccountResponse response = this.getSession( AdminSession.class , request ).viewUser( id );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getAccount( ) == null ) {
			return this.redirect( "users" );
		}
		return this.render( model , "internal" , "en" , "user" , response );
	}


	@RequestMapping( "/user-{id}-sessions" )
	public String viewUserSessions( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "users" );
		}

		ListSessionsResponse response = this.getSession( AdminSession.class , request ).viewUserSessions( id );
		if ( !response.isPrivilegeOk( ) ) {
			return this.redirect( "main" );
		} else if ( response.getAccount( ) == null ) {
			return this.redirect( "users" );
		}
		return this.render( model , "internal" , "en" , "userSessions" , response );
	}


	@RequestMapping( value = "/user-{id}.action" , method = RequestMethod.POST )
	public String grantCredits( HttpServletRequest request , Model model , @PathVariable( "id" ) String sId ,
			@RequestParam( "credits" ) String sCredits )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( sId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "users" );
		}

		int credits;
		try {
			credits = Integer.parseInt( sCredits );
		} catch ( NumberFormatException e ) {
			credits = 0;
		}
		if ( credits > 0 ) {
			this.getSession( AdminSession.class , request ).giveCredits( id , credits );
		}
		return this.redirect( "user-" + id );
	}

}
