package com.deepclone.lw.web.beans.intercept;


import java.io.IOException;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.deepclone.lw.cmd.MaintenanceResponse;
import com.deepclone.lw.cmd.ext.ListLanguagesResponse;
import com.deepclone.lw.session.CommandResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.session.ClientSessionReference;
import com.deepclone.lw.web.beans.session.SessionClient;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.csess.ExternalSession;



public class LanguageInterceptorBean
		extends HandlerInterceptorAdapter

{

	private final Logger logger = Logger.getLogger( SessionInterceptorBean.class );
	private SessionClient client;


	@Autowired( required = true )
	public void setSessionClient( SessionClient client )
	{
		this.client = client;
	}


	@Override
	public boolean preHandle( HttpServletRequest request , HttpServletResponse response , Object handler )
			throws IOException
	{
		HttpSession session = request.getSession( );
		String language = (String) session.getAttribute( "language" );

		if ( language != null ) {
			return true;
		}

		this.logger.debug( "no language configured, trying to look it up" );

		ExternalSession eSession;
		CommandResponse cResponse;
		ClientSessionReference sReference = new ClientSessionReference( null );
		try {
			InetAddress addr = InetAddress.getByName( request.getRemoteAddr( ) );
			eSession = this.client.getSession( sReference , ExternalSession.class , addr );
			cResponse = eSession.listLanguages( );
		} catch ( SessionException e ) {
			this.logger.error( "session error while initialising session language" );
			throw new RuntimeException( e );
		} catch ( SessionServerException e ) {
			this.logger.warn( "server offline while initialising session language, defaulting" );
			session.setAttribute( "language" , "en" );
			return true;
		} catch ( SessionMaintenanceException e ) {
			this.logger.warn( "server under maintenance while initialising session language, defaulting" );
			session.setAttribute( "language" , "en" );
			return true;
		}

		if ( cResponse instanceof MaintenanceResponse ) {
			this.logger.warn( "server under maintenance while initialising session language, defaulting" );
			session.setAttribute( "language" , "en" );
			return true;
		}

		ListLanguagesResponse languages = (ListLanguagesResponse) cResponse;
		String accept = request.getHeader( "Accept-Language" );
		if ( accept == null ) {
			this.logger.info( "Accept-Language not found, defaulting" );
			session.setAttribute( "language" , "en" );
			return true;
		}

		Set< String > supported = new HashSet< String >( );
		for ( ListLanguagesResponse.Language l : languages.getLanguages( ) ) {
			supported.add( l.getId( ).toLowerCase( ) );
			this.logger.trace( "supported language code '" + l.getId( ) + "'" );
		}

		List< String > lAccept = this.makeLangList( accept.replaceAll( "\\s+" , "" ) );
		for ( String lId : lAccept ) {
			this.logger.debug( "accepted language code '" + lId + "'" );
			if ( supported.contains( lId.toLowerCase( ) ) ) {
				this.logger.info( "using language " + lId );
				session.setAttribute( "language" , lId );
				return true;
			}
		}

		this.logger.info( "no language found in Accept-Language, defaulting" );
		session.setAttribute( "language" , "en" );
		return true;
	}


	private List< String > makeLangList( String acceptString )
	{
		List< Double > qValues = new ArrayList< Double >( );
		Map< Double , Set< String > > found = new HashMap< Double , Set< String > >( );

		for ( String part : acceptString.split( "," ) ) {

			Double qValue;
			String language;
			if ( part.contains( ";q=" ) ) {
				try {
					String r[] = part.split( ";q=" );
					language = r[ 0 ];
					qValue = Double.parseDouble( r[ 1 ] );
				} catch ( NumberFormatException e ) {
					continue;
				}
			} else {
				qValue = 1.0;
				language = part;
			}

			Set< String > accepts = found.get( qValue );
			if ( accepts == null ) {
				accepts = new HashSet< String >( );
				qValues.add( qValue );
				found.put( qValue , accepts );
			}
			accepts.add( language );
		}

		Collections.sort( qValues );
		Collections.reverse( qValues );
		List< String > result = new LinkedList< String >( );
		for ( Double v : qValues ) {
			result.addAll( found.get( v ) );
		}
		return result;
	}
}
