package com.deepclone.lw.web.beans.intercept;


import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.deepclone.lw.web.beans.session.ClientSessionReference;



public class SessionInterceptorBean
		extends HandlerInterceptorAdapter
{

	private final Logger logger = Logger.getLogger( SessionInterceptorBean.class );


	@Override
	public boolean preHandle( HttpServletRequest request , HttpServletResponse response , Object handler )
			throws IOException
	{
		HttpSession session = request.getSession( );
		ClientSessionReference sReference = (ClientSessionReference) session.getAttribute( "sReference" );
		if ( sReference == null ) {
			sReference = new ClientSessionReference( null );
			session.setAttribute( "sReference" , sReference );
		}

		Class< ? > handlerType = handler.getClass( );
		SessionRequirement requirement = handlerType.getAnnotation( SessionRequirement.class );
		if ( requirement == null ) {
			this.logger.debug( "no session requirement" );
			return true;
		}

		if ( ( requirement.value( ) && sReference.isNull( ) ) || ( !requirement.value( ) && !sReference.isNull( ) ) ) {
			this.logger.debug( "requirement fail, redirecting to /" + requirement.redirectTo( ) );
			response.sendRedirect( requirement.redirectTo( ) );
			return false;
		}
		if ( requirement.value( ) && requirement.subType( ).length == 1 ) {
			String rType = requirement.subType( )[ 0 ];
			if ( !rType.equals( sReference.getReference( ).extra ) ) {
				this.logger.debug( "sub-type requirement fail, redirecting to /" + requirement.redirectTo( ) );
				response.sendRedirect( requirement.redirectTo( ) );
				return false;
			}
		}
		this.logger.debug( "session requirement satisfied" );

		return true;
	}
}
