package com.deepclone.lw.web.csess;


import java.util.List;

import com.deepclone.lw.cmd.CreateAuthChallengeCommand;
import com.deepclone.lw.cmd.CreateAuthChallengeResponse;
import com.deepclone.lw.cmd.admin.*;
import com.deepclone.lw.cmd.admin.adata.*;
import com.deepclone.lw.cmd.admin.bans.*;
import com.deepclone.lw.cmd.admin.bt.*;
import com.deepclone.lw.cmd.admin.constants.*;
import com.deepclone.lw.cmd.admin.i18n.*;
import com.deepclone.lw.cmd.admin.logs.*;
import com.deepclone.lw.cmd.admin.mntm.*;
import com.deepclone.lw.cmd.admin.msg.*;
import com.deepclone.lw.cmd.admin.naming.*;
import com.deepclone.lw.cmd.admin.prefs.*;
import com.deepclone.lw.cmd.admin.su.*;
import com.deepclone.lw.cmd.admin.tick.*;
import com.deepclone.lw.cmd.admin.users.*;
import com.deepclone.lw.cmd.bt.*;
import com.deepclone.lw.cmd.bt.data.BugStatus;
import com.deepclone.lw.cmd.msgdata.MessageType;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.utils.DigestHelper;
import com.deepclone.lw.web.beans.session.Session;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.session.SessionType;



@SessionType( "admin" )
public class AdminSession
		extends Session
{

	/* Common commands */

	public AdminOverviewResponse getOverview( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AdminOverviewResponse) this.execute( new AdminOverviewCommand( ) );
	}


	public AdminResponse noOp( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AdminResponse) this.execute( new NoOperationCommand( ) );
	}


	public AdminResponse noOp( Privileges privilege )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AdminResponse) this.execute( new NoOperationCommand( privilege ) );
	}


	public SetPasswordResponse changePassword( String current , String password , String passwordConfirm )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		String challenge = ( (CreateAuthChallengeResponse) this.execute( new CreateAuthChallengeCommand( ) ) )
				.getChallenge( );

		String sha1 , md5;
		sha1 = DigestHelper.digest( "sha-1" , challenge + " " + DigestHelper.digest( "sha-1" , current ) );
		md5 = DigestHelper.digest( "md5" , challenge + " " + DigestHelper.digest( "md5" , current ) );

		return (SetPasswordResponse) this.execute( new SetPasswordCommand( sha1 , md5 , password , passwordConfirm ) );
	}


	/* Constants administration */

	public GetConstantsResponse getConstants( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetConstantsResponse) this.execute( new GetConstantsCommand( ) );
	}


	public SetConstantResponse setConstant( String name , double value )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (SetConstantResponse) this.execute( new SetConstantCommand( name , value ) );
	}


	/* Administrators administration */

	public ListAdministratorsResponse listAdmins( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListAdministratorsResponse) this.execute( new ListAdministratorsCommand( ) );
	}


	public AddAdministratorResponse addAdmin( String address , String name , List< Privileges > privs )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AddAdministratorResponse) this.execute( new AddAdministratorCommand( address , name , privs ) );
	}


	public ViewAdministratorResponse viewAdministrator( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewAdministratorResponse) this.execute( new ViewAdministratorCommand( id ) );
	}


	public ViewAdministratorResponse resetAdministrator( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewAdministratorResponse) this.execute( new ResetAdminPasswordCommand( id ) );
	}


	public ViewAdministratorResponse setAdminPrivileges( int id , List< Privileges > privs )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewAdministratorResponse) this.execute( new SetPrivilegesCommand( id , privs ) );
	}


	/* I18N administration */

	public ViewLanguagesResponse listLanguages( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewLanguagesResponse) this.execute( new ViewLanguagesCommand( ) );
	}


	public GetLanguageResponse getLanguage( String language )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetLanguageResponse) this.execute( new GetLanguageCommand( language ) );
	}


	public ChangeLanguageResponse setLanguageName( String language , String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ChangeLanguageResponse) this.execute( new ChangeLanguageCommand( language , name ) );
	}


	public SetStringResponse setTranslation( String language , String string , String value )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (SetStringResponse) this.execute( new SetStringCommand( language , string , value ) );
	}


	/* Names */

	public NamesSummaryResponse getNameSummary( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (NamesSummaryResponse) this.execute( new NamesSummaryCommand( ) );
	}


	public GetNamesResponse getNames( NameType type )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetNamesResponse) this.execute( new GetNamesCommand( type ) );
	}


	public void namesAction( NameType type , NameAction action , int[] ids )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new NamesActionCommand( type , action , ids ) );
	}


	/* Banhammer */

	public BansSummaryResponse getBansSummary( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (BansSummaryResponse) this.execute( new BansSummaryCommand( ) );
	}


	public ListBansResponse getBans( BanType type )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListBansResponse) this.execute( new ListBansCommand( type ) );
	}


	public RequestBanResponse requestBan( String user , boolean empire , String reason )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (RequestBanResponse) this.execute( new RequestBanCommand( user , empire , reason ) );
	}


	public RejectBanResponse rejectBan( int id , String reason )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (RejectBanResponse) this.execute( new RejectBanCommand( id , reason ) );
	}


	public void confirmBan( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ConfirmBanCommand( id ) );
	}


	public void liftBan( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new LiftBanCommand( id ) );
	}


	/* Ticker */

	public TickerStatusResponse getTickerStatus( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (TickerStatusResponse) this.execute( new TickerStatusCommand( ) );
	}


	public void toggleTicker( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ToggleTickerCommand( ) );
	}


	public void setTickerTaskStatus( int task , boolean run )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new SetTaskStatusCommand( task , run ) );
	}


	public void setTickerTaskStatus( int task , long delay )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new SetTaskStatusCommand( task , delay ) );
	}


	/* Logs */

	public ViewLogResponse viewLog( LogType logType , int first , int pageSize , LogLevel logLevel , String component ,
			boolean excOnly )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewLogResponse) this.execute( new ViewLogCommand( logType , first , pageSize , logLevel , component ,
				excOnly ) );
	}


	public GetEntryResponse getLogEntry( long id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetEntryResponse) this.execute( new GetEntryCommand( id ) );
	}


	/* Spam! */

	public AdminResponse sendSpam( String title , String body )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AdminResponse) this.execute( new SendSpamCommand( title , body ) );
	}


	/* Messages */

	public GetMessagesResponse getMessages( boolean inbox )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetMessagesResponse) this.execute( new GetMessagesCommand( inbox ) );
	}


	public ReadMessageResponse readMessage( boolean inbox , long id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ReadMessageResponse) this.execute( new ReadMessageCommand( inbox , id ) );
	}


	public void deleteMessages( boolean inbox , long[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new MessageBoxCommand( inbox , selection ) );
	}


	public void markRead( long[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new MessageBoxCommand( selection , true ) );
	}


	public void markUnread( long[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new MessageBoxCommand( selection , false ) );
	}


	public ComposeMessageResponse initNewMessage( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ComposeMessageResponse) this.execute( PrepareMessageCommand.newMessage( ) );
	}


	public ComposeMessageResponse replyTo( boolean inbox , long id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ComposeMessageResponse) this.execute( new PrepareMessageCommand( id , inbox ) );
	}


	public ComposeMessageResponse messageTo( MessageType type , int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ComposeMessageResponse) this.execute( PrepareMessageCommand.newMessageTo( type , id ) );
	}


	public ComposeMessageResponse sendMessage( MessageType type , String toName , String title , String contents )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ComposeMessageCommand command = new ComposeMessageCommand( );
		command.setType( type );
		command.setTarget( toName );
		command.setSubject( title );
		command.setContents( contents );
		return (ComposeMessageResponse) this.execute( command );
	}


	public ComposeMessageResponse sendReply( boolean inbox , long rtId , MessageType type , String toName ,
			String title , String contents )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ComposeMessageCommand command = new ComposeMessageCommand( inbox , rtId );
		command.setType( type );
		command.setTarget( toName );
		command.setSubject( title );
		command.setContents( contents );
		return (ComposeMessageResponse) this.execute( command );
	}


	/* Users */

	public ListAccountsResponse listUsers( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListAccountsResponse) this.execute( new ListAccountsCommand( ) );
	}


	public ListAccountsResponse listOnlineUsers( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListAccountsResponse) this.execute( new ListAccountsCommand( null , true ) );
	}


	public ListAccountsResponse listUsers( AccountStatus status )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListAccountsResponse) this.execute( new ListAccountsCommand( status ) );
	}


	public ListAccountsResponse listOnlineUsers( AccountStatus status )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListAccountsResponse) this.execute( new ListAccountsCommand( status , true ) );
	}


	public ViewAccountResponse viewUser( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewAccountResponse) this.execute( new ViewAccountCommand( id ) );
	}


	public ListSessionsResponse viewUserSessions( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListSessionsResponse) this.execute( new ListSessionsCommand( id ) );
	}


	public void giveCredits( int id , int credits )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new GiveCreditsCommand( id , credits ) );
	}


	/* Default preferences */

	public PrefDefaultsResponse getPrefDefaults( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (PrefDefaultsResponse) this.execute( new GetPrefDefaultsCommand( ) );
	}


	public void setPrefDefault( String pref , String value )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new SetPrefDefaultCommand( pref , value ) );
	}


	/* Bug tracking system */

	public BugsSummaryResponse getBugsSummary( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (BugsSummaryResponse) this.execute( new BugsSummaryCommand( ) );
	}


	public ListBugsResponse listBugs( BugStatus status , boolean ownOnly , long first , int count )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListBugsResponse) this.execute( new ListBugsCommand( status , ownOnly , first , count ) );
	}


	public ReportBugResponse reportBug( String title , String description , boolean publicReport )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ReportBugResponse) this.execute( new ReportBugCommand( title , description , publicReport ) );
	}


	public ViewBugResponse getBugReport( long bugId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewBugResponse) this.execute( new ViewBugCommand( bugId ) );
	}


	public PostCommentResponse postBugComment( long bugId , String comment , boolean publicComment )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (PostCommentResponse) this.execute( new PostCommentCommand( bugId , comment , publicComment ) );
	}


	public void moderateBugComment( long commentId , boolean validate )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ModerateCommentCommand( commentId , validate ) );
	}


	public void validateReport( long bugId , BugStatus newStatus , boolean visible , int grantCredits , boolean snapshot )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ValidateReportCommand( bugId , newStatus , visible , grantCredits , snapshot ) );

	}


	public void setReportStatus( long bugId , BugStatus newStatus )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ReportStatusCommand( bugId , newStatus ) );
	}


	public void toggleReportVisibility( long bugId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ReportVisibilityCommand( bugId ) );
	}


	public MergeReportsResponse mergeReports( long bugId , long mergeId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (MergeReportsResponse) this.execute( new MergeReportsCommand( bugId , mergeId ) );
	}


	public GetSnapshotResponse getSnapshot( long bugId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetSnapshotResponse) this.execute( new GetSnapshotCommand( bugId ) );
	}


	/* Maintenance mode */

	public MaintenanceStatusResponse getMaintenanceStatus( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (MaintenanceStatusResponse) this.execute( new MaintenanceStatusCommand( ) );
	}


	public MaintenanceChangeResponse enableMaintenance( String reason , int duration )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (MaintenanceChangeResponse) this.execute( new EnableMaintenanceCommand( reason , duration ) );
	}


	public void extendMaintenance( int duration )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ExtendMaintenanceCommand( duration ) );
	}


	public void endMaintenance( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new EndMaintenanceCommand( ) );
	}

}
