package com.deepclone.lw.web.csess;


import java.util.Map;

import com.deepclone.lw.cmd.CreateAuthChallengeCommand;
import com.deepclone.lw.cmd.CreateAuthChallengeResponse;
import com.deepclone.lw.cmd.bt.*;
import com.deepclone.lw.cmd.bt.data.BugStatus;
import com.deepclone.lw.cmd.msgdata.MessageType;
import com.deepclone.lw.cmd.player.*;
import com.deepclone.lw.cmd.player.account.*;
import com.deepclone.lw.cmd.player.alliances.*;
import com.deepclone.lw.cmd.player.battles.*;
import com.deepclone.lw.cmd.player.bt.*;
import com.deepclone.lw.cmd.player.elist.*;
import com.deepclone.lw.cmd.player.fleets.*;
import com.deepclone.lw.cmd.player.gdata.*;
import com.deepclone.lw.cmd.player.planets.*;
import com.deepclone.lw.cmd.player.tech.GetResearchCommand;
import com.deepclone.lw.cmd.player.tech.GetResearchResponse;
import com.deepclone.lw.cmd.player.tech.ImplementTechCommand;
import com.deepclone.lw.cmd.player.tech.UpdateResearchPrioritiesCommand;
import com.deepclone.lw.cmd.player.msgs.*;
import com.deepclone.lw.session.Command;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.utils.DigestHelper;
import com.deepclone.lw.web.beans.session.*;



@SessionType( "player" )
public class PlayerSession
		extends Session
{

	/* General & account management commands */

	public String getLanguage( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Command command = new GetLanguageCommand( );
		return ( (GetLanguageResponse) this.execute( command ) ).getLanguage( );
	}


	public AccountValidationResponse startValidation( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AccountValidationResponse) this.execute( new AccountValidationCommand( ) );
	}


	public AccountValidationResponse validate( String token , String empire , String planet )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Command command = new AccountValidationCommand( token , empire , planet );
		return (AccountValidationResponse) this.execute( command );
	}


	public AccountReactivationResponse reactivate( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AccountReactivationResponse) this.execute( new AccountReactivationCommand( ) );
	}


	public BanDetailsResponse getBanDetails( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (BanDetailsResponse) this.execute( new BanDetailsCommand( ) );
	}


	/* Empire commands */

	public EmpireResponse getOverview( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (EmpireResponse) this.execute( new OverviewCommand( ) );
	}


	/**
	 * Request research information
	 * 
	 * @return the research information response
	 */
	public GetResearchResponse getResearch( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetResearchResponse) this.execute( new GetResearchCommand( ) );
	}


	/**
	 * Update research priorities
	 * 
	 * @param priorities
	 *            the map of research identifiers to priority values
	 */
	public void updateResearchPriorities( Map< String , Integer > priorities )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new UpdateResearchPrioritiesCommand( priorities ) );
	}


	/**
	 * Implement a technology
	 * 
	 * @param technology
	 *            the technology's identifier
	 */
	public void implementTechnology( String technology )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ImplementTechCommand( technology ) );
	}


	/**
	 * Send an {@link UpdateEmpireMiningSettingsCommand} to the server
	 * 
	 * @param miningSettings
	 *            the mining settings
	 */
	public void updateEmpireMiningSettings( Map< String , Integer > miningSettings )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new UpdateEmpireMiningSettingsCommand( miningSettings ) );
	}


	/* Planet list */

	public ListPlanetsResponse listPlanets( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListPlanetsResponse) this.execute( new ListPlanetsCommand( ) );
	}


	/* Map */

	public ViewMapResponse viewMap( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewMapResponse) this.execute( new ViewMapCommand( ) );
	}


	public ViewMapResponse viewMap( int x , int y , MapSize size )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewMapResponse) this.execute( new ViewMapCommand( x , y , size ) );
	}


	/* Planet commands */

	public ViewPlanetResponse getPlanetView( int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewPlanetResponse) this.execute( new ViewPlanetCommand( id ) );
	}


	public ViewPlanetResponse constructBuildings( int planetId , int type , int amount )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Command cmd = new BuildingActionCommand( planetId , type , amount , false );
		return (ViewPlanetResponse) this.execute( cmd );
	}


	public ViewPlanetResponse destroyBuildings( int planetId , int type , int amount )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Command cmd = new BuildingActionCommand( planetId , type , amount , true );
		return (ViewPlanetResponse) this.execute( cmd );
	}


	public ViewPlanetResponse buildShips( int planetId , int type , int amount )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Command cmd = new BuildShipsCommand( planetId , type , amount );
		return (ViewPlanetResponse) this.execute( cmd );
	}


	public ViewPlanetResponse flushQueue( int planetId , boolean military )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Command cmd = new FlushQueueCommand( planetId , military );
		return (ViewPlanetResponse) this.execute( cmd );
	}


	/**
	 * Send a {@link ToggleMiningSettingsCommand} to the server
	 * 
	 * @param pId
	 *            the planet whose mining settings are to be switched between empire-wide and
	 *            planet-specific
	 */
	public void toggleMiningSettingsFor( int pId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ToggleMiningSettingsCommand( pId ) );
	}


	/**
	 * Send a {@link UpdatePlanetMiningSettingsCommand} to the server
	 * 
	 * @param pId
	 *            the identifier of the planet to udpate
	 * @param settings
	 *            the map of settings, as associations between resource identifiers and priority
	 *            values
	 */
	public void updatePlanetMiningSettings( int pId , Map< String , Integer > settings )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new UpdatePlanetMiningSettingsCommand( pId , settings ) );
	}


	public ViewPlanetResponse rename( int planetId , String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewPlanetResponse) this.execute( new RenamePlanetCommand( planetId , name ) );
	}


	public ViewPlanetResponse abandon( int planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewPlanetResponse) this.execute( new AbandonPlanetCommand( planetId , false ) );
	}


	public ViewPlanetResponse cancelAbandon( int planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewPlanetResponse) this.execute( new AbandonPlanetCommand( planetId , true ) );
	}


	/* Alliance commands */

	public AllianceStatusResponse getAllianceStatus( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new AllianceStatusCommand( ) );
	}


	public AllianceStatusResponse viewAlliance( String tag )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new ViewAllianceCommand( tag ) );
	}


	public AllianceStatusResponse joinAlliance( String tag )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new JoinAllianceCommand( tag ) );
	}


	public AllianceStatusResponse cancelJoinAlliance( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new CancelJoinCommand( ) );
	}


	public AllianceStatusResponse createAlliance( String tag , String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new CreateAllianceCommand( tag , name ) );
	}


	public AllianceStatusResponse manageRequests( boolean accept , int[] members )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new ManageRequestsCommand( members , accept ) );
	}


	public AllianceStatusResponse kickMembers( int[] members )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new KickMembersCommand( members ) );
	}


	public AllianceStatusResponse transferLeadership( int to )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new TransferLeadershipCommand( to ) );
	}


	public AllianceStatusResponse leaveAlliance( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (AllianceStatusResponse) this.execute( new LeaveAllianceCommand( ) );
	}


	/* Enemy list */

	public EnemyListResponse getEnemyList( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (EnemyListResponse) this.execute( new EnemyListCommand( ) );
	}


	public EnemyListResponse addEnemy( boolean alliance , String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (EnemyListResponse) this.execute( new AddEnemyCommand( alliance , name ) );
	}


	public EnemyListResponse removeEnemies( boolean alliance , int[] ids )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (EnemyListResponse) this.execute( new RemoveEnemiesCommand( alliance , ids ) );
	}


	/* Fleets */

	public ViewFleetsResponse getFleets( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewFleetsResponse) this.execute( new ViewFleetsCommand( ) );
	}


	public MoveFleetsResponse moveFleets( long[] ids )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (MoveFleetsResponse) this.execute( new MoveFleetsCommand( ids ) );
	}


	public MoveFleetsResponse moveFleets( long[] ids , String destination , boolean attack )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (MoveFleetsResponse) this.execute( new MoveFleetsCommand( ids , destination , attack ) );
	}


	public SetFleetsModeResponse setFleetsMode( long[] ids , boolean attack , boolean forReal )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (SetFleetsModeResponse) this.execute( new SetFleetsModeCommand( ids , attack , forReal ) );
	}


	public RenameFleetsResponse renameFleets( long[] ids )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (RenameFleetsResponse) this.execute( new RenameFleetsCommand( ids ) );
	}


	public RenameFleetsResponse renameFleets( long[] ids , String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (RenameFleetsResponse) this.execute( new RenameFleetsCommand( ids , name ) );
	}


	public SplitFleetResponse splitFleet( long id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (SplitFleetResponse) this.execute( new SplitFleetCommand( id ) );
	}


	public SplitFleetResponse splitFleet( long id , Map< Integer , Integer > ships , int nFleets , String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (SplitFleetResponse) this.execute( new SplitFleetCommand( id , ships , nFleets , name ) );
	}


	public DisbandFleetsResponse disbandFleets( long[] ids , boolean confirm )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (DisbandFleetsResponse) this.execute( new DisbandFleetsCommand( ids , confirm ) );
	}


	public MergeFleetsResponse mergeFleets( long[] ids )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (MergeFleetsResponse) this.execute( new MergeFleetsCommand( ids ) );
	}


	/* Account management */

	public GetAccountResponse getAccount( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetAccountResponse) this.execute( new GetAccountCommand( ) );
	}


	public void setLanguage( String language )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new SetLanguageCommand( language ) );
	}


	public String createAuthenticationChallenge( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return ( (CreateAuthChallengeResponse) this.execute( new CreateAuthChallengeCommand( ) ) ).getChallenge( );
	}


	public SetPasswordResponse setPassword( String currentPassword , String challenge , String newPass1 ,
			String newPass2 )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		String sha1 , md5;
		sha1 = DigestHelper.digest( "sha-1" , challenge + " " + DigestHelper.digest( "sha-1" , currentPassword ) );
		md5 = DigestHelper.digest( "md5" , challenge + " " + DigestHelper.digest( "md5" , currentPassword ) );

		return (SetPasswordResponse) this.execute( new SetPasswordCommand( sha1 , md5 , newPass1 , newPass2 ) );
	}


	public SetAddressResponse setAddress( String currentPassword , String challenge , String newAddr1 , String newAddr2 )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		String sha1 , md5;
		sha1 = DigestHelper.digest( "sha-1" , challenge + " " + DigestHelper.digest( "sha-1" , currentPassword ) );
		md5 = DigestHelper.digest( "md5" , challenge + " " + DigestHelper.digest( "md5" , currentPassword ) );

		return (SetAddressResponse) this.execute( new SetAddressCommand( sha1 , md5 , newAddr1 , newAddr2 ) );
	}


	public ValidateSetAddressResponse cancelAddressChange( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ValidateSetAddressResponse) this.execute( new ValidateSetAddressCommand( ) );
	}


	public ValidateSetAddressResponse confirmAddressChange( String code )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ValidateSetAddressResponse) this.execute( new ValidateSetAddressCommand( code ) );
	}


	public void loadDefaultPreferences( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new SetPreferencesCommand( ) );
	}


	public void setPreferences( Map< String , String > prefs )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new SetPreferencesCommand( prefs ) );
	}


	public void setQuit( String reason )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new QuitGameCommand( reason ) );
	}


	public void cancelQuit( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new CancelQuitCommand( ) );
	}


	public void toggleVacation( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new ToggleVacationCommand( ) );
	}


	/* Battles */

	public GetBattleResponse getBattle( long battle )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetBattleResponse) this.execute( new GetBattleCommand( battle ) );
	}


	public GetBattleResponse getBattle( long battle , long tick )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetBattleResponse) this.execute( new GetBattleCommand( battle , tick ) );
	}


	public ListBattlesResponse getBattles( int page )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListBattlesResponse) this.execute( new ListBattlesCommand( page ) );
	}


	public GetNewPlanetResponse getNewPlanet( String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetNewPlanetResponse) this.execute( new GetNewPlanetCommand( name ) );
	}


	/* Messages */

	public GetMessagesResponse getMessages( boolean inbox )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (GetMessagesResponse) this.execute( new GetMessagesCommand( inbox ) );
	}


	public ReadMessageResponse readMessage( boolean inbox , long id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ReadMessageResponse) this.execute( new ReadMessageCommand( inbox , id ) );
	}


	public void deleteMessages( boolean inbox , long[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new MessageBoxCommand( inbox , selection ) );
	}


	public void markRead( long[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new MessageBoxCommand( selection , true ) );
	}


	public void markUnread( long[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		this.execute( new MessageBoxCommand( selection , false ) );
	}


	public ComposeMessageResponse initNewMessage( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ComposeMessageResponse) this.execute( PrepareMessageCommand.newMessage( ) );
	}


	public ComposeMessageResponse messageTo( MessageType messageType , int id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ComposeMessageResponse) this.execute( PrepareMessageCommand.newMessageTo( messageType , id ) );
	}


	public ComposeMessageResponse sendMessage( MessageType type , String toName , String title , String contents )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ComposeMessageCommand command = new ComposeMessageCommand( );
		command.setType( type );
		command.setTarget( toName );
		command.setSubject( title );
		command.setContents( contents );
		return (ComposeMessageResponse) this.execute( command );
	}


	public ComposeMessageResponse sendReply( boolean inbox , long rtId , MessageType type , String toName ,
			String title , String contents )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		ComposeMessageCommand command = new ComposeMessageCommand( inbox , rtId );
		command.setType( type );
		command.setTarget( toName );
		command.setSubject( title );
		command.setContents( contents );
		return (ComposeMessageResponse) this.execute( command );
	}


	public ComposeMessageResponse replyTo( boolean inbox , long id )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ComposeMessageResponse) this.execute( new PrepareMessageCommand( id , inbox ) );
	}


	public ListTargetsResponse listMessageTargets( )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListTargetsResponse) this.execute( new ListTargetsCommand( ) );
	}


	/* Bug tracking system */

	public ListBugsResponse listBugs( BugStatus status , boolean ownOnly , long first , int count )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ListBugsResponse) this.execute( new ListBugsCommand( status , ownOnly , first , count ) );
	}


	public ReportBugResponse reportBug( String title , String description )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ReportBugResponse) this.execute( new ReportBugCommand( title , description ) );
	}


	public ViewBugResponse getBugReport( long bugId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (ViewBugResponse) this.execute( new ViewBugCommand( bugId ) );
	}


	public PostCommentResponse postBugComment( long bugId , String comment )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return (PostCommentResponse) this.execute( new PostCommentCommand( bugId , comment ) );
	}

}
