<#macro render_fleet_ships rType id ships>
	<table>
		<tr>
			<th class="ships-type">Ship class</th>
			<th class="ships-amount">Quantity</th>
			<th class="ships-power">Power</th>
		</tr>
		<#list ships as ship>
			<tr>
				<td class="ships-type">${ship.name}</td>
				<td class="ships-amount">${ship.amount?string(',##0')}</td>
				<td class="ships-power">${ship.power?string(',##0')}</td>
			</tr>
		</#list>
	</table>
</#macro>
<#macro render_static_fleet_details rType fleet>
	<tr class="${rType} fleet-ships" id="fleet-${fleet.id}-ships">
		<td>&nbsp;</td>
		<td colspan="5">
			<div style="float:right">
				<@render_fleet_ships rType="own-fleet" id=fleet.id ships=fleet.ships />
			</div>
			<div style="width:">
				<@dt_main>
					<#if fleet.status != 'AVAILABLE'>
						<@dt_entry title="Penalty">
							<@duration rTime=fleet.penalty gTime=fleet.gamePenalty />
						</@dt_entry>
					</#if>
					<@dt_entry abbr="OOFT" title="Orbit-to-orbit flight time">
						<@duration rTime=fleet.flightTime gTime=fleet.gameFlightTime />
					</@dt_entry>
				</@dt_main>
			</div>
		</td>
	</tr> 
</#macro>
<#macro render_fleets_at location owners>
	<#list owners as owner>
		<#switch owner.relation>
			<#case 'OWN'>
				<#local rType = 'own-fleet'>
				<#local mode = location.attacking >
				<#break>
			<#case 'ALLIED'>
				<#local rType = 'allied-fleet'>
				<#local mode = location.attacking >
				<#break>
			<#case 'ENEMY'>
				<#local rType = 'enemy-fleet'>
				<#local mode = ! location.attacking >
				<#break>
		</#switch>
		<#list owner.fleets as fleet>
			<tr class="${rType} fleet-details" id="fleet-${fleet.id}">
				<td class="selector">
					<#if owner.relation == 'OWN' && data.page.special! != 'v'>
						<input type="checkbox" name="selection" value="${fleet.id}" />
					<#else>
						&nbsp;
					</#if>
				</td>
				<td class="name">
					<#if fleet.name?has_content && fleet.name != "">
						${fleet.name?xhtml}
					<#else>
						<em>(unnamed fleet)</em>
					</#if>
				</td>
				<td>
					<#if owner.relation = 'OWN'>
						${owner.name?xhtml}
					<#else>
						<a href="msg-empire-${owner.id}">${owner.name?xhtml}</a>
					</#if>
				</td>
				<td class="mode">
					<#if mode><abbr title="Attacking">A</abbr><#else><abbr title="Defending">D</abbr></#if>
				</td>
				<td class="power">${fleet.power?string(',##0')}</td>
				<td class="status">
					<#if fleet.status == 'AVAILABLE'>
						<abbr title="This fleet is available, it can be given any order.">AVL</abbr>
					<#else>
						<#switch fleet.status>
							<#case 'DEPLOYING'><abbr title="This fleet is undergoing its initial deployment from the space docks.">DPL</abbr><#break>
							<#case 'REDEPLOYING'><abbr title="This fleet is re-deploying after arrival or mode change.">RDP</abbr><#break>
						</#switch>
					</#if>
				</td>
			</tr>
			<@render_static_fleet_details rType=rType fleet=fleet />
		</#list>
	</#list>
</#macro>
<#macro render_static_fleets locations>
	<#list locations as location>
		<table class="fleets-planet">
			<tr>
				<td class="pic"><img src="img/pp/s/${location.picture}.png" alt="${location.name?xhtml}" /></td>
				<th class="planet-name"><a href="planet-${location.id}">${location.name?xhtml}</a> (<strong>${location.x},${location.y}</strong>;<strong>${location.orbit}</strong>)</th>
				<td class="planet-data">Population: <strong>${location.population?string(",##0")}</strong></td>
				<td class="planet-data">
					<#if location.attacking>
						<span class="enemy-fleet">
					<#elseif location.own>
						<span class="own-fleet">
					<#else>
						<span class="allied-fleet">
					</#if>
						Defence: <strong>${location.defence?string(",##0")}</strong>
					</span>
				</td>
			</tr>
			<tr>
				<td colspan="4">
					<#if location.fleetOwners?size == 0>
						<p>No fleets at this location.</p>
					<#else>
						<table class="fleets-list">
							<tr class="header">
								<td class="selector">&nbsp;</td>
								<th class="name">Fleet name</th>
								<th class="owner">Owner</th>
								<th class="mode"><abbr title="Fleet mode">M</abbr></th>
								<th class="power"><abbr title="Fleet power">P</abbr></th>
								<th class="status"><abbr title="Fleet status">S</abbr></th>
							</tr>
							<@render_fleets_at location=location owners=location.fleetOwners />
						</table>
					</#if>
				</td>
			</tr>
			<#if location.onVacation!false>
				<tr><td colspan="4" style="padding:2px">Vacation mode enabled</td></tr>
			<#elseif location.battle?has_content && location.battle.hostile gt 0>
				<tr><td colspan="4" style="padding:2px">
					Battle - <span class="own-fleet">${location.battle.friendly?string(',##0')}</span> vs. <span class="enemy-fleet">${location.battle.hostile?string(',##0')}</span> -
					<a href="battle-${location.battle.id}-latest">Details</a>
				</td></tr>
			</#if>
		</table>
	</#list>
</#macro>
<#macro render_moving_fleet_details fleet>
	<tr class="own-fleet fleet-ships" id="fleet-${fleet.id}-ships">
		<td>&nbsp;</td>
		<td colspan="6">
			<div style="float:right">
				<@render_fleet_ships rType="own-fleet" id=fleet.id ships=fleet.ships />
			</div>
			<div style="width:">
				<@dt_main>
					<@dt_entry title="Source"><a href="planet-${fleet.source.id}">${fleet.source.name?xhtml}</a></@dt_entry>
					<@dt_entry title="Status">
						<#if fleet.status == 'AVAILABLE'>
							<abbr title="This fleet is available, it can be given any order.">AVL</abbr>
						<#else>
							<abbr title="This fleet is changing course.">RDR</abbr> -
							<@duration rTime=fleet.penalty gTime=fleet.gamePenalty />
						</#if>
					</@dt_entry>
					<@dt_entry title="Time to target"><@duration rTime=fleet.timeLeft gTime=fleet.gameTimeLeft /></@dt_entry>
					<@dt_entry title="Coordinates">(${fleet.currentX?string("0.00")};${fleet.currentY?string("0.00")})</@dt_entry>
					<@dt_entry title="Nearest planet">
						<#if fleet.nearest?has_content>
							<a href="planet-${fleet.nearest.id}">${fleet.nearest.name?xhtml}</a>
						<#else>
							<em>in outer space</em>
						</#if>
					</@dt_entry>
					<@dt_entry abbr="OOFT" title="Orbit-to-orbit flight time">
						<@duration rTime=fleet.flightTime gTime=fleet.gameFlightTime />
					</@dt_entry>
				</@dt_main>
			</div>
		</td>
	</tr> 
</#macro>
<#macro render_moving_fleet fleet>
	<tr class="own-fleet fleet-details" id="fleet-${fleet.id}">
		<td class="selector">
			<#if data.page.special! != 'v'>
				<input type="checkbox" name="selection" value="${fleet.id}" />
			<#else>
				&nbsp;
			</#if>
		</td>
		<td class="name" colspan="2">
			<#if fleet.name?has_content && fleet.name != "">
				${fleet.name?xhtml}
			<#else>
				<em>(unnamed fleet)</em>
			</#if>
		</td>
		<td class="mode">
			<#if fleet.attacking><abbr title="Attacking">A</abbr><#else><abbr title="Defending">D</abbr></#if>
		</td>
		<td class="power">${fleet.power?string(',##0')}</td>
		<td class="timeleft"><@duration rTime=fleet.eta gTime=fleet.gameEta /></td>
		<td class="destination"><a href="planet-${fleet.destination.id}">${fleet.destination.name?xhtml}</a></td>
	</tr>
	<@render_moving_fleet_details fleet=fleet />
</#macro>
<#macro render_moving_fleets fleets>
	<#if fleets?size != 0>
		<table class="fleets-moving">
			<tr>
				<th class="mv-header">Moving fleets</th>
			</tr>
			<tr>
				<td>
					<table class="fleets-list">
						<tr>
							<td class="selector">&nbsp;</td>
							<th class="name" colspan="2">Fleet name</th>
							<th class="mode"><abbr title="Fleet mode">M</abbr></th>
							<th class="power"><abbr title="Fleet power">P</abbr></th>
							<th class="timeleft"><abbr title="Estimated time to arrival">ETA</abbr></th>
							<th class="destination">Destination</th>
						</tr>
						<#list fleets as fleet>
							<@render_moving_fleet fleet=fleet />
						</#list>
					</table>
				</td>
			</tr>
		</table>
	</#if>
</#macro>
<#macro render>
<@page title="Fleets">
	<#local locations = data.fleets.locations>
	<#local mFleets = data.fleets.moving>
	<#if locations?size == 0 && mFleets?size == 0>
		<p>You do not own any fleets or planets.</p>
	<#else>
		<div style="font-style: italic; text-align:center; padding: 0 0 10px 0">Click a fleet's name to see composition and details</div>
		<form method="post" action="fleets.action">
			<@render_static_fleets locations=locations />
			<@render_moving_fleets fleets=mFleets />
			<#if data.page.special! != 'v'>
				<div id="fleet-actions">
					<input type="submit" class="input" name="rename" value="Rename" title="Rename selected fleets" />
					&nbsp;
					<input type="submit" class="input" name="move" value="Move / redirect" title="Move or redirect selected fleets, if they are available." />
					&nbsp;
					<input type="submit" class="input" name="setAttack" value="Attack" title="Set selected fleets to attack mode." />
					<input type="submit" class="input" name="setDefend" value="Defend" title="Set selected fleets to defence mode." />
					&nbsp;
					<input type="submit" class="input" name="merge" value="Merge" title="Merge selected fleets, if they are available." />
					<input type="submit" class="input" name="split" value="Split" title="Split selected fleet into two or more fleets." />
					&nbsp;
					<input type="submit" class="input" name="disband" value="Disband" title="Destroy selected fleets." />
				</div>
			</#if>
		</form>
	</#if>
</@page>
</#macro>