<#macro render>
<#if ! data.basic?has_content>
<@page title="Planet not found">
	<p>This planet does not exist.</p>
</@page>
<#return>
</#if>
<@page title="Planet ${data.basic.name}">
	<#if data.ownershipError>
		<@standalone_error>Unfortunately, we no longer control this planet.</@standalone_error>
	</#if>

	<@tabs>

		<@tab id="general" title="General">
			<div>
				<div style="float:right; padding: 3px">
					<img src="img/pp/l/${data.basic.picture}.png" style="border: 1px solid white" alt="${data.basic.name?xhtml}" />
				</div>
	
				<@left_column width=250>
					<@dt_main>
						<@dt_entry title="Coordinates">(<strong>${data.basic.x},${data.basic.y}</strong>;<strong>${data.basic.orbit}</strong>)</@dt_entry>
						<#if data.basic.alliance?has_content>
							<@dt_entry title="Alliance"><strong>${data.basic.alliance}</strong></@dt_entry>
						<#else>
							<@dt_blank />
						</#if>
	
						<#if data.orbit?has_content>
							<@dt_entry title="Population"><strong>${data.orbit.population?string(",##0")}</strong></@dt_entry>
							<@dt_entry title="Stationary defence"><strong>${data.orbit.defencePoints?string(",##0")}</strong></@dt_entry>
						</#if>
	
						<#if data.own?has_content>
							<@dt_entry title="Happiness"><@happiness value=data.own.happiness /> <@happiness_change value=data.own.hChange /></@dt_entry>
							<@dt_entry title="Income"><strong>${data.own.income?string(",##0")}</strong> <@abbr_bgc/></@dt_entry>
							<@dt_entry title="Upkeep"><strong>${data.own.upkeep?string(",##0")}</strong> <@abbr_bgc/></@dt_entry>
						</#if>
					</@dt_main>
				</@left_column>
	
				<@right_column width=250>
					<#if data.orbit?has_content>
						<@dt_main>
	
							<#if data.orbit.ownFleet gt 0>
								<@dt_entry title="Own fleet power"><span style="color: #afffaf">${data.orbit.ownFleet?string(",##0")}</span></@dt_entry>
							<#else>
								<@dt_blank />
							</#if>
	
							<#if data.orbit.friendlyFleet gt 0>
								<@dt_entry title="Friendly fleet power"><span style="color: #afafff">${data.orbit.friendlyFleet?string(",##0")}</span></@dt_entry>
							<#else>
								<@dt_blank />
							</#if>
	
							<#if data.orbit.hostileFleet gt 0>
								<@dt_entry title="Hostile fleet power"><span style="color: #ffafaf">${data.orbit.hostileFleet?string(",##0")}</span></@dt_entry>
							<#else>
								<@dt_blank />
							</#if>
	
							<#if data.orbit.battle?has_content>
								<@dt_entry title=""><a href="battle-${data.orbit.battle}-latest">View battle</a></@dt_entry>
							<#else>
								<@dt_blank />
							</#if>
	
							<@dt_blank /><@dt_blank /><@dt_blank /><@dt_blank /><@dt_blank />
	
						</@dt_main>
					</#if>
				</@right_column>
			</div>

			<#if data.own?has_content>
				<#if data.own.status.renamePossible && data.page.special! != 'v'>
					<#switch data.renameError!>
						<#case "EMPTY"><@standalone_error>Please specify a name.</@standalone_error><#break>
						<#case "INVALID"><@standalone_error>Invalid planet name.</@standalone_error><#break>
						<#case "UNAVAILABLE"><@standalone_error>This planet name already exists.</@standalone_error><#break>
						<#case "BANNED"><@standalone_error>This name is banned.</@standalone_error><#break>
					</#switch>
					<@lineform action="planet-${data.id}-rename" name="rename" hash="general">
						<#if data.renamingTo?has_content>
							<#assign newName = data.renamingTo!>
						</#if>
						Rename planet to <@ff_text name="name" id="rename-name" maxLength=20 size=21 value=newName! />
						<@ff_submit label="Rename planet" /> 
					</@lineform>
				</#if>
				
				<#if data.own.status.abandonPossible && data.page.special! != 'v'>
					<@lineform action="planet-${data.id}-abandon" name="abandon" hash="general">
						<@ff_submit label="Abandon planet" />
					</@lineform>
				<#elseif data.own.status.abandonTime gt 0>
					<@lineform action="planet-${data.id}-cancel-abandon" name="abandon" hash="general">
						Abandoning planet in <strong><@duration rTime=data.own.status.abandonTime gTime=data.own.status.abandonGameTime /></strong>
						&nbsp;
						<#if data.page.special! != 'v'>
							<@ff_submit label="Cancel" />
						</#if>
					</@lineform>
				</#if>
			</#if>

		</@tab>

		<#if data.own?has_content && data.own.resources?size gt 0>
		
			<@tab id="resources" title="Economy">
			
				<@listview>
					<#local previousCategory="">

					<@lv_line headers=true>
						<@lv_column width=30>&nbsp;</@lv_column>
						<@lv_column width="x">Resource</@lv_column>
						<@lv_column width=100 centered=true><@over_time "Income" /></@lv_column>
						<@lv_column width=100 centered=true><@over_time "Upkeep" /></@lv_column>
						<@lv_column width=100 centered=true>Invested</@lv_column>
					</@lv_line>
					
					<#list data.own.resources as resource>

						<#if previousCategory?has_content && !resource.category?has_content
								|| resource.category?has_content && !previousCategory?has_content
								|| resource.category?has_content && previousCategory?has_content
									&& resource.category != previousCategory>
							<@lv_line>
								<#if resource.category?has_content>
									<td colspan="5"><strong>${resource.category?xhtml}</strong></td>
								<#else>
									<td colspan="5"><hr /></td>
								</#if>
							</@lv_line>
							<#local previousCategory=resource.category>
						</#if>
						
						<@lv_line>
							<@lv_column>&nbsp;</@lv_column>
							<@lv_column>${resource.title?xhtml}
								<div class="auto-hide">${resource.description?xhtml}</div>
							</@lv_column>
							<@lv_column centered=true>${resource.income?string(",##0")}</@lv_column>
							<@lv_column centered=true>${resource.upkeep?string(",##0")}</@lv_column>
							<@lv_column centered=true>${resource.invested?string(",##0")}</@lv_column>
						</@lv_line>

					</#list>

				</@listview>
			
			</@tab>

		</#if>
	
		<#if data.orbit?has_content>
		
			<#if data.orbit.buildings?size gt 0 || data.own?has_content>
				<@tab id="buildings" title="Buildings">

					<#if data.orbit.buildings?size gt 0>
						<@listview>
							<@lv_line headers=true>
								<@lv_column width="x">Building</@lv_column>
								<@lv_column width=50 centered=true>Amount</@lv_column>
								<#if data.own?has_content>
									<@lv_column width=150 centered=true>Production</@lv_column>
									<@lv_column width=50 centered=true>Jobs</@lv_column>
									<@lv_column width=100 centered=true>Upkeep</@lv_column>
								</#if> 
							</@lv_line>
	
							<#list data.orbit.buildings as building>
								<@lv_line>
									<@lv_column>
										${building.name?xhtml}
										<#if data.own?has_content>
											<div class="auto-hide">${building.description?xhtml}</div>
										</#if>
									</@lv_column>
									<@lv_column centered=true>${building.amount?string(",##0")}</@lv_column>
									<#if data.own?has_content>
										<@lv_column centered=true>
											${building.output?string(",##0")}
											<#switch building.produces>
												<#case "CASH">ind. production<#break>
												<#case "DEF">defence<#break>
												<#case "WORK">mil. output<#break>
												<#case "POP">growth<#break>
												<#case "MINE">res. extraction<#break>
											</#switch>
										</@lv_column>
										<@lv_column centered=true>${building.jobs?string(",##0")}</@lv_column>
										<@lv_column centered=true>${building.upkeep?string(",##0")} <@abbr_bgc/></@lv_column>
									</#if> 
								</@lv_line>
							</#list>
						</@listview>
					<#else>
						<@dt_main><@dt_status>No buildings on this planet.</@dt_status></@dt_main>
					</#if>

					<#if data.own?has_content>
						<#if data.page.special! != 'v'>
							<#if data.own.civQueue.appendPossible>
								<@lineform name="build-civ" action="planet-${data.id}-build-civ" hash="buildings">
									Build <@ff_text name="amount" id="civ-build-amount" size=5 maxLength=4 value="" />
									<@ff_select name="type" id="civ-build-type">
										<@form_option value="0" text="(building type)" />
										<#list data.own.bBuildings as building>
											<@form_option value=building.id text=building.name />
										</#list>
									</@ff_select>
									<@ff_submit label="Add to queue" />
								</@lineform>
								<div id="buildings-types">
									<#list data.own.bBuildings as building>
										<div class="building-type" id="building-type-${building.id}">
											<@dt_main>
												<@dt_status>${building.description?xhtml}</@dt_status>
												<@dt_entry title="Cost">${building.cost?string(",##0")} <@abbr_bgc/></@dt_entry>
												<@dt_entry title="Upkeep">${building.upkeep?string(",##0")} <@abbr_bgc/></@dt_entry>
												<@dt_entry title="Time to build">
													<#if building.time?has_content>
														<@duration rTime=building.time gTime=building.gameTime />
													<#else>
														<span style="color:#ffafaf;font-weight:bold" title="Don't even try.">N/A</span>
													</#if>
												</@dt_entry>
												<@dt_entry title="Jobs">${building.workers?string(",##0")}</@dt_entry>
												<@dt_entry title="Production">${building.output?string(",##0")}
													<#switch building.prodType>
														<#case "CASH">ind. production<#break>
														<#case "DEF">defence<#break>
														<#case "WORK">mil. output<#break>
														<#case "POP">growth<#break>
														<#case "MINE">res. extraction<#break>
													</#switch>
												</@dt_entry>
											</@dt_main>
										</div>
									</#list>
								</div>
	
								<#if data.orbit.buildings?size gt 0>
									<#if data.destructionFailed!false>
										<@standalone_error>Trying to destroy too many buildings</@standalone_error>
									</#if>
									<@lineform name="destroy" action="planet-${data.id}-destroy" hash="buildings">
										Destroy <@ff_text name="amount" id="civ-destroy-amount" size=5 maxLength=4 value="" />
										<@ff_select name="type" id="civ-destroy-type">
											<@form_option value="0" text="(building type)" />
											<#list data.orbit.buildings as building>
												<@form_option value=building.id text=building.name />
											</#list>
										</@ff_select>  
										<@ff_submit label="Add to queue" />
									</@lineform>
								</#if>
							</#if>
						</#if>

						<#if data.own.civQueue.items?size gt 0>
							<#if data.page.special! != 'v'>
								<@lineform name="flush-civ-queue" action="planet-${data.id}-flush-civ" hash="buildings">
									Remove all items from the build queue (investments will be lost)
									<@ff_submit label="Flush" />
								</@lineform>
							</#if>

							<@listview>
								<@lv_line headers=true>
									<@lv_column width=80 centered=true>Amount</@lv_column>
									<@lv_column width="x">Nature</@lv_column>
									<@lv_column width=200 centered=true>Time</@lv_column>
									<@lv_column width=70 centered=true>Investment</@lv_column>
								</@lv_line> 
								<#list data.own.civQueue.items as qItem>
									<@lv_line>
										<@lv_column centered=true>${qItem.amount?string(",##0")}</@lv_column>
										<@lv_column>${qItem.name} <#if qItem.destroy>(destruction)<#else>(construction)</#if></@lv_column>
										<@lv_column centered=true>
											<#if qItem.timeLeft?has_content>
												<@duration rTime=qItem.timeLeft gTime=qItem.gameTimeLeft />
											<#else>
												<span style="color:#ffafaf;font-weight:bold" title="The human race will be long gone when this finishes building.">N/A</span>
											</#if>
										</@lv_column>
										<@lv_column centered=true>${qItem.invested?string(",##0")} <@abbr_bgc/></@lv_column>
									</@lv_line>
								</#list>
							</@listview>
						</#if>
					</#if>

				</@tab>
			</#if>

			<#if data.own?has_content>

				<@tab id="ships" title="Shipyards">
					<#if data.page.special! != 'v'>
						<#if data.own.milQueue.appendPossible>
							<@lineform name="build-mil" action="planet-${data.id}-build-mil" hash="ships">
								Build <@ff_text name="amount" id="mil-build-amount" size=5 maxLength=4 value="" />
								<@ff_select name="type" id="mil-build-type">
									<@form_option value="0" text="(ship type)" />
									<#list data.own.bShips as ship>
										<@form_option value=ship.id text=ship.name />
									</#list>
								</@ff_select>
								<@ff_submit label="Add to queue" />
							</@lineform>
							<div id="ships-types">
								<#list data.own.bShips as ship>
									<div class="ship-type" id="ship-type-${ship.id}">
										<@dt_main>
											<@dt_status>${ship.description?xhtml}</@dt_status>
											<@dt_entry title="Cost">${ship.cost?string(",##0")} <@abbr_bgc/></@dt_entry>
											<@dt_entry title="Upkeep">${ship.upkeep?string(",##0")} <@abbr_bgc/></@dt_entry>
											<@dt_entry title="Time to build">
												<#if ship.time?has_content>
													<@duration rTime=ship.time gTime=ship.gameTime />
												<#else>
													<span style="color:#ffafaf;font-weight:bold" title="Don't even try.">N/A</span>
												</#if>
											</@dt_entry>
											<@dt_entry title="Power">${ship.power?string(",##0")}</@dt_entry>
											<@dt_entry abbr="OOFT" title="Orbit-to-orbit flight time">
												<@duration rTime=ship.flightTime gTime=ship.gameFlightTime />
											</@dt_entry>
										</@dt_main>
									</div>
								</#list>
							</div>
						</#if>
					</#if>

					<#if data.own.milQueue.items?size gt 0>
						<#if data.page.special! != 'v'>
							<@lineform name="flush-mil-queue" action="planet-${data.id}-flush-mil" hash="ships">
								Remove all items from the build queue (investments will be lost)
								<@ff_submit label="Flush" />
							</@lineform>
						</#if>

						<@listview>
							<@lv_line headers=true>
								<@lv_column width=80 centered=true>Amount</@lv_column>
								<@lv_column width="x">Ship</@lv_column>
								<@lv_column width=200 centered=true>Time</@lv_column>
								<@lv_column width=70 centered=true>Investment</@lv_column>
							</@lv_line> 
							<#list data.own.milQueue.items as qItem>
								<@lv_line>
									<@lv_column centered=true>${qItem.amount?string(",##0")}</@lv_column>
									<@lv_column>${qItem.name}</@lv_column>
									<@lv_column centered=true>
										<#if qItem.timeLeft?has_content>
											<@duration rTime=qItem.timeLeft gTime=qItem.gameTimeLeft />
										<#else>
											<span style="color:#ffafaf;font-weight:bold" title="The human race will be long gone when this finishes building.">N/A</span>
										</#if>
									</@lv_column>
									<@lv_column centered=true>${qItem.invested?string(",##0")} <@abbr_bgc/></@lv_column>
								</@lv_line>
							</#list>
						</@listview>
					</#if>
				</@tab>

			</#if>
			
			<#include "planet/natres.ftl" />
			<@RenderNaturalResources />
		</#if>
		
	</@tabs>
</@page>
</#macro>