<#macro show_alliance alliance>
	<@dt_main>
		<@dt_entry title="Tag ">${alliance.tag?xhtml}</@dt_entry>
		<@dt_entry title="Nom ">${alliance.name?xhtml}</@dt_entry>
		<@dt_entry title="Dirigeant ">${alliance.leaderName?xhtml}</@dt_entry>
		<@dt_entry title="Planètes ">${alliance.planets?string(",##0")}</@dt_entry>
		<@dt_entry title=""><a href="msg-alliance-${alliance.id}">Envoyer un message</a></@dt_entry>
	</@dt_main>
</#macro>
<#macro alliance_info_tab>

	<@tab id="info" title="Autres alliances">

		<@form action="alliance-info" name="alliance-info" hash="info">
			<@form_text name="tag" id="ai-tag" label="Tag de l'alliance" value=data.requested! maxLength=5 />
			<@form_submit label="Obtenir les informations" />
		</@form>
		
		<#if data.info?has_content>
			<@show_alliance alliance=data.info />
		</#if>

	</@tab>

</#macro>
<#macro alliance_creation_tab>

	<#if data.creation?has_content>
		<#assign tag = data.creation.tag!>
		<#assign tagError = data.creation.tagError!>
		<#assign name = data.creation.name!>
		<#assign nameError = data.creation.nameError!>
	<#else>
		<#assign tag = "">
		<#assign tagError = "">
		<#assign name = "">
		<#assign nameError = "">
	</#if>

	<@tab id="create-alliance" title="Créer">
		<@form action="create-alliance" name="create-alliance" hash="create-alliance">
			<#switch tagError>
				<#case "EMPTY"><@form_error>Veuillez indiquer le tag de votre alliance.</@form_error><#break>
				<#case "INVALID"><@form_error>Ce tag est invalide.</@form_error><#break>
				<#case "UNAVAILABLE"><@form_error>Ce tag est utilisé par une autre alliance.</@form_error></break>
			</#switch>
			<@form_text name="tag" id="ca-tag" label="Tag de l'alliance " value=tag maxLength=5 />

			<#switch nameError>
				<#case "EMPTY"><@form_error>Veuillez spécifier le nom de votre alliance.</@form_error><#break>
				<#case "INVALID"><@form_error>Ce nom est invalide.</@form_error><#break>
			</#switch>
			<@form_text name="name" id="ca-name" label="Nom de l'alliance " value=name maxLength=5 />

			<@form_submit label="Créer une alliance" />
		</@form>
	</@tab>

</#macro>
<#macro alliance_join_tab>

	<@tab id="join-alliance" title="Rejoindre">
		<@form action="join-alliance" name="join-alliance" hash="join-alliance">
			<#if data.joinFailure?has_content>
				<@form_error>Alliance non trouvée.</@form_error>
			</#if>
			<@form_text name="tag" id="ja-tag" label="Tag de l'alliance" value=data.joinFailure! maxLength=5 />
			<@form_submit label="Rejoindre cette alliance" />
		</@form>
	</@tab>

</#macro>
<#macro no_alliance>

	<#if data.page.special! != 'v'>
		<@alliance_join_tab />
		<@alliance_creation_tab />
	</#if>

</#macro>
<#macro alliance_joining_tab>

	<@tab id="join-alliance" title="Requête envoyée">
		<@lineform action="cancel-join" hash="join-alliance">
			Une requête pour rejoindre <strong>${alliance.main.tag?xhtml}</strong> a été envoyée.
			<#if data.page.special! != 'v'>
				<@ff_submit label="Annuler" />
			</#if>
		</@lineform>
		<@show_alliance alliance=alliance.main />
	</@tab>

</#macro>
<#macro alliance_planets pList>
	<@tab id="planets" title="Planètes">
	
		<#if pList?size == 0>
			<p>Il n'y a pas de planètes dans cette alliance.</p>
			<#return>
		</#if>
		
		<@listview>
			<@lv_line headers=true>
				<@lv_column width=80 centered=true>Coordonnées</@lv_column>
				<@lv_column width=150>Planètes</@lv_column>
				<@lv_column width=150>Propriétaire</@lv_column>
			</@lv_line>

			<#list pList as planet>
				<#if planet.battle>
					<#local bStart = "<span class='enemy-fleet' style='font-weight:bold'>">
					<#local bEnd = "</span>">
				<#else>
					<#local bStart = " ">
					<#local bEnd = " ">
				</#if>

				<@lv_line>
					<@lv_column centered=true>${bStart}(${planet.x},${planet.y};${planet.orbit})${bEnd}</@lv_column>
					<@lv_column><a href="planet-${planet.id}">${bStart}${planet.name?xhtml}${bEnd}</a></@lv_column>
					<@lv_column>${bStart}${planet.owner?xhtml}${bEnd}</@lv_column>
				</@lv_line>
				
				<#if planet.battle>
					<@lv_line>
						<td>&nbsp;</td>
						<td colspan="2">
							Situation militaire:
							<strong class="own-fleet">${planet.defence?string(",##0")}</strong>
							contre
							<strong class="enemy-fleet">${planet.attack?string(",##0")}</strong>
						</td>
					</@lv_line>
				</#if>
			</#list>
		</@listview>
	
	</@tab>
</#macro>
<#macro alliance_members mList isLeader empty="">

	<#if mList?size == 0 && empty != "">
		<p>${empty?xhtml}</p>
		<#return>
	</#if>

	<@listview>
		<@lv_line headers=true>
			<#if isLeader && data.page.special! != 'v'>
				<@lv_column width=40 centered=true>&nbsp;</@lv_column>
			</#if>
			<@lv_column width="x">Empire</@lv_column>
		</@lv_line>

		<#list mList as member>
			<@lv_line>
				<#if isLeader && data.page.special! != 'v'>
					<@lv_column centered=true>
						<#if member.name == data.page.empire>
							&nbsp;
						<#else>
							<input type="checkbox" name="members" value="${member.id}" />
						</#if>
					</@lv_column>
				</#if>
				<@lv_column>
					<#if member.name == data.page.empire>
						${member.name?xhtml}
					<#else>
						<a href="msg-empire-${member.id}">${member.name?xhtml}</a>
					</#if>
				</@lv_column>
			</@lv_line>
		</#list>
	</@listview>

</#macro>
<#macro alliance_member_page>
	<#local leader = alliance.leader?has_content>

	<@tab id="in-alliance" title="Alliance">
		<@lineform action="leave-alliance" hash="join-alliance">
			Vous êtes
			<#if alliance.leader?has_content>
				le dirigeant
			<#else>
				un membre
			</#if>
			de <strong>${alliance.main.tag?xhtml}</strong>.
			<#if data.page.special! != 'v'>
				<@ff_submit label="Quitter l'alliance" />
			</#if>
		</@lineform>
		<@show_alliance alliance=alliance.main />
	</@tab>

	<@alliance_planets pList=alliance.member.planets />

	<@tab id="members" title="Members">
		<#if leader && alliance.member.members?size gt 1 && data.page.special! != 'v'>
			<form action="kick-members.action#members" method="post">
		</#if>
		<@alliance_members mList=alliance.member.members isLeader=leader />
		<#if leader && alliance.member.members?size gt 1 && data.page.special! != 'v'>
				<div class="form-container">
					<table><tr><td>
						<@ff_submit label="Exclure les membres sélectionnés" />
					</td></tr></table>
				</div>
			</form>
			
			<@lineform action="transfer-leadership" hash="members">
				Trasnférer la direction à
				<@ff_select id="leadership" name="leadership">
					<#list alliance.member.members as member>
						<#if member.name != data.page.empire>
							<@form_option value=member.id text=member.name /> 
						</#if>
					</#list>
				</@ff_select>
				<@ff_submit label="Confirmation" />
			</@lineform>
		</#if>
	</@tab>

	<#if leader>
		<@tab id="requests" title="Requêtes">
			<#if alliance.leader.requests?size gt 0 && data.page.special! != 'v'>
				<form action="manage-requests.action#requests" method="post">
			</#if>
			<@alliance_members mList=alliance.leader.requests isLeader=true empty="Aucune requête en suspens" />
			<#if alliance.leader.requests?size gt 0 && data.page.special! != 'v'>
					<div class="form-container">
						<table><tr><td>
							<@ff_select id="requests-select" name="action">
								<@form_option value="1" text="Accepter" />
								<@form_option value="0" text="Rejecter" />
							</@ff_select> postulants sélectionnés
							<@ff_submit label="Confirmation" />
						</td></tr></table>
					</div>
				</form>
			</#if>
		</@tab>
	</#if>

</#macro>
<#macro render>
<@page title="Alliance">

	<#assign alliance = data.alliance>
	<@tabs>
	
		<#if alliance.main?has_content>
			<#if alliance.member?has_content>
				<@alliance_member_page />
			<#else>
				<@alliance_joining_tab />
			</#if>
		<#else>
			<@no_alliance />
		</#if>
		<@alliance_info_tab />	

	</@tabs>

</@page>
</#macro>