<#macro render_fleet fleet>
	<td class="selector"><input type="checkbox" name="selection" value="${fleet.id}" checked="checked" /></td>
	<td class="name">
		<#if fleet.name?has_content && fleet.name != "">
			${fleet.name?xhtml}
		<#else>
			<em>(flotte sans nom)</em>
		</#if>
	</td>	
	<td class="power">${fleet.power?string(',##0')}</td>
	<td class="fltime"><@duration rTime=fleet.flightTime gTime=fleet.gameFlightTime /></td>
	<td class="status">
		<#switch fleet.status>
			<#case 'DEPLOYING'><abbr title="Cette flotte est en train d'effectuer son déploiement initial depuis les quais spaciaux.">DPL</abbr><#break>
			<#case 'REDEPLOYING'><abbr title="Cette flotte est en train de se re-déployer après son arrivée ou un changement de mode.">RDP</abbr><#break>
			<#case 'REDIRECTING'><abbr title="Cette flotte est en train de changer de destination.">RDP</abbr><#break>
			<#case 'AVAILABLE'><abbr title="Cette flotte est disponible, il est possible de lui donner n'importe quel ordre.">DIS</abbr><#break>
		</#switch>
	</td>
	<td class="cloc">
		(${fleet.x?string("0.00")};${fleet.y?string("0.00")})<br/>
		<#if fleet.nearest?has_content>
			<#if fleet.atPlanet>À<#else>Près de</#if>
			<a href="planet-${fleet.nearest.id}">${fleet.nearest.name?xhtml}</a>
		<#else>
			Dans l'espace profond
		</#if>
	</td>
</#macro>
<#macro render_selected_fleets fleets>
	<h3>Flottes sélectionnées</h3>
	<table class="selected-fleets">
		<tr>
			<td class="selector">&nbsp;</td>
			<th class="name">Nom</th>
			<th class="power"><abbr title="Puissance de la flotte">P</abbr></th>
			<th class="fltime"><abbr title="Temps de vol orbite-à-orbite">TV</abbr></th>
			<th class="status"><abbr title="Statut de la flotte">S</abbr></th>
			<th class="cloc">Localisation</th>
		</tr>
		<#list fleets as fleet>
			<tr>
				<@render_fleet fleet=fleet />
			</tr>
		</#list>
	</table>
</#macro>
<#macro render_move_fleets>
	<h3>Nouveaux ordres</h3>
	<#if data.error>
		<@standalone_error>Cette planète n'existe pas.</@standalone_error>
	</#if>
	<p>
		Diriger les flottes sélectionnées vers
		<@ff_text name="destination" id="destination" value="${data.destination!?xhtml}" maxLength=20 size=21 />
		en
		<@ff_select name="mode" id="mode">
			<@form_option text="défense" selected=( !data.mode!false ) value="0" />
			<@form_option text="attaque" selected=( data.mode!false ) value="1" />
		</@ff_select>
		&nbsp;
		<@ff_submit label="Ok" />
		<@ff_submit label="Annuler" name="cancel" />
	</p>
</#macro>
<#macro render_rename_fleets>
	<h3>Renommer les flottes</h3>
	<#if data.error>
		<@standalone_error>Ce nom est invalide.</@standalone_error>
	</#if>
	<p>
		Renommer les flottes sélectionnées en
		<@ff_text name="name" id="name" value="${data.name!?xhtml}" maxLength=40 size=21 />
		&nbsp;
		<@ff_submit label="Ok" />
		<@ff_submit label="Annuler" name="cancel" />
	</p>
</#macro>
<#macro render_set_fleets_mode>
	<h3>Définir le mode des flottes</h3>
	<p>
		Mettre les flottes sélectionnées en
		<#if data.attack>attaque<#else>défense</#if>?
		&nbsp;
		<@ff_submit label="Oui" />
		<@ff_submit label="Non" name="cancel" />
	</p>
	<input type="hidden" name="attack" value="<#if data.attack>1<#else>0</#if>" />
</#macro>
<#macro render_disband_fleets>
	<h3>Dissudre les flottes</h3>
	<p>
		Êtes-vous sûr?
		&nbsp;
		<@ff_submit label="Oui" />
		<@ff_submit label="Non" name="cancel" />
	</p>
</#macro>
<#macro render>
<#switch dataType>
	<#case 'MoveFleetsResponse'>
		<#local title="Déplacer les flottes">
		<#local action="move-fleets.action">
		<#break>
	<#case 'RenameFleetsResponse'>
		<#local title="Renommer les flottes">
		<#local action="rename-fleets.action">
		<#break>
	<#case 'SetFleetsModeResponse'>
		<#local title="Définir le mode des flottes">
		<#local action="set-fleets-mode.action">
		<#break>
	<#case 'DisbandFleetsResponse'>
		<#local title="Dissoudre les flottes">
		<#local action="disband-fleets.action">
		<#break>
</#switch>
<@page title=title>
	<form action="${action}" method="post">
		<@render_selected_fleets fleets=data.fleets />
		<#switch dataType>
			<#case 'MoveFleetsResponse'>
				<@render_move_fleets />
				<#break>
			<#case 'RenameFleetsResponse'>
				<@render_rename_fleets />
				<#break>
			<#case 'SetFleetsModeResponse'>
				<@render_set_fleets_mode />
				<#break>
			<#case 'DisbandFleetsResponse'>
				<@render_disband_fleets />
				<#break>
		</#switch>
	</form>
</@page>
</#macro>