package com.deepclone.lw.web.main;


import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.ModelAndView;

import com.deepclone.lw.cmd.MaintenanceResponse;
import com.deepclone.lw.session.SessionCommandException;
import com.deepclone.lw.session.SessionIdentifierException;
import com.deepclone.lw.web.beans.session.MaintenanceStatus;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;



public class ErrorHandlerBean
		implements HandlerExceptionResolver
{

	private MaintenanceStatus maintenanceStatus;


	@Autowired( required = true )
	public void setMaintenanceStatus( MaintenanceStatus maintenanceStatus )
	{
		this.maintenanceStatus = maintenanceStatus;
	}


	private String getLanguage( HttpServletRequest request )
	{
		HttpSession session = request.getSession( );
		String l = (String) session.getAttribute( "language" );
		return ( l == null ) ? "en" : l;
	}


	@Override
	public ModelAndView resolveException( HttpServletRequest request , HttpServletResponse response , Object handler ,
			Exception ex )
	{
		if ( ex instanceof SessionServerException ) {
			MaintenanceResponse maintenance = this.maintenanceStatus.getStatus( );
			if ( maintenance != null ) {
				return this.maintenance( request , maintenance );
			}
			return this.offline( request );
		} else if ( ex instanceof SessionMaintenanceException ) {
			MaintenanceResponse maintenance = ( (SessionMaintenanceException) ex ).getMaintenance( );
			this.maintenanceStatus.storeStatus( maintenance );
			return this.maintenance( request , maintenance );
		} else if ( ex instanceof SessionIdentifierException || ex instanceof SessionCommandException ) {
			return new ModelAndView( "redirect:player-session" );
		}

		// Other exceptions are not handled
		return null;
	}


	private ModelAndView maintenance( HttpServletRequest request , MaintenanceResponse maintenance )
	{
		ModelAndView mav = new ModelAndView( "ROOT" );
		mav.addObject( "container" , "offline" );
		mav.addObject( "language" , this.getLanguage( request ) );
		mav.addObject( "type" , "maintenance" );
		mav.addObject( "data" , maintenance );
		return mav;
	}


	private ModelAndView offline( HttpServletRequest request )
	{
		ModelAndView mav = new ModelAndView( "ROOT" );
		mav.addObject( "container" , "offline" );
		mav.addObject( "language" , this.getLanguage( request ) );
		mav.addObject( "type" , "offline" );
		return mav;
	}

}
