package com.deepclone.lw.web.main.game;


import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.player.alliances.AllianceStatusResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class AlliancePage
		extends PageControllerBase
{

	@RequestMapping( "/alliance" )
	public String view( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "alliance" , pSession.getAllianceStatus( ) );
	}


	@RequestMapping( value = "/alliance-info.action" , method = RequestMethod.POST )
	public String info( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@RequestParam( "tag" ) String tag )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "alliance" , pSession.viewAlliance( tag ) );
	}


	@RequestMapping( value = "/join-alliance.action" , method = RequestMethod.POST )
	public String join( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@RequestParam( "tag" ) String tag )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "alliance" , pSession.joinAlliance( tag ) );
	}


	@RequestMapping( value = "/cancel-join.action" , method = RequestMethod.POST )
	public String cancelJoin( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "alliance" , pSession.cancelJoinAlliance( ) );
	}


	@RequestMapping( value = "/create-alliance.action" , method = RequestMethod.POST )
	public String create( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@RequestParam( "tag" ) String tag , @RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "alliance" , pSession.createAlliance( tag , name ) );
	}


	@RequestMapping( value = "/manage-requests.action" , method = RequestMethod.POST )
	public String manageRequests( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "members" , required = false ) String[] members ,
			@RequestParam( "action" ) String action )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		boolean accept = "1".equals( action );
		List< Integer > identifiers = new LinkedList< Integer >( );
		if ( members != null ) {
			for ( String member : members ) {
				try {
					identifiers.add( Integer.parseInt( member ) );
				} catch ( NumberFormatException e ) {
					// EMPTY
				}
			}
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		AllianceStatusResponse response;
		if ( identifiers.isEmpty( ) ) {
			response = pSession.getAllianceStatus( );
		} else {
			int[] ids = new int[ identifiers.size( ) ];
			int i = 0;
			for ( Integer l : identifiers ) {
				ids[ i++ ] = l.intValue( );
			}
			response = pSession.manageRequests( accept , ids );
		}
		return this.render( model , "game" , language , "alliance" , response );
	}


	@RequestMapping( value = "/kick-members.action" , method = RequestMethod.POST )
	public String kickMembers( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "members" , required = false ) String[] members )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		List< Integer > identifiers = new LinkedList< Integer >( );
		if ( members != null ) {
			for ( String member : members ) {
				try {
					identifiers.add( Integer.parseInt( member ) );
				} catch ( NumberFormatException e ) {
					// EMPTY
				}
			}
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		AllianceStatusResponse response;
		if ( identifiers.isEmpty( ) ) {
			response = pSession.getAllianceStatus( );
		} else {
			int[] ids = new int[ identifiers.size( ) ];
			int i = 0;
			for ( Integer l : identifiers ) {
				ids[ i++ ] = l.intValue( );
			}
			response = pSession.kickMembers( ids );
		}
		return this.render( model , "game" , language , "alliance" , response );
	}


	@RequestMapping( value = "/transfer-leadership.action" , method = RequestMethod.POST )
	public String transferLeadership( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( "leadership" ) String leadership )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		AllianceStatusResponse response;
		try {
			response = pSession.transferLeadership( Integer.parseInt( leadership ) );
		} catch ( NumberFormatException e ) {
			response = pSession.getAllianceStatus( );
		}
		return this.render( model , "game" , language , "alliance" , response );
	}


	@RequestMapping( value = "/leave-alliance.action" , method = RequestMethod.POST )
	public String leaveAlliance( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "alliance" , pSession.leaveAlliance( ) );
	}
}
