package com.deepclone.lw.web.main.game;


import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.player.elist.EnemyListResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class EnemiesPage
		extends PageControllerBase
{

	@RequestMapping( "/enemies" )
	public String overview( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "enemies" , pSession.getEnemyList( ) );
	}


	@RequestMapping( value = "/add-enemy-empire.action" , method = RequestMethod.POST )
	public String addEmpire( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "enemies" , pSession.addEnemy( false , name ) );
	}


	@RequestMapping( value = "/add-enemy-alliance.action" , method = RequestMethod.POST )
	public String addAlliance( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "enemies" , pSession.addEnemy( true , name ) );
	}


	@RequestMapping( value = "/remove-enemy-empires.action" , method = RequestMethod.POST )
	public String removeEmpires( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "id" , required = false ) String[] sIds )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.remove( request , language , model , sIds , false );
	}


	@RequestMapping( value = "/remove-enemy-alliances.action" , method = RequestMethod.POST )
	public String removeAlliances( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "id" , required = false ) String[] sIds )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		return this.remove( request , language , model , sIds , true );
	}


	private String remove( HttpServletRequest request , String language , Model model , String[] sIds , boolean alliance )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		List< Integer > identifiers = new LinkedList< Integer >( );
		if ( sIds != null ) {
			for ( String member : sIds ) {
				try {
					identifiers.add( Integer.parseInt( member ) );
				} catch ( NumberFormatException e ) {
					// EMPTY
				}
			}
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		EnemyListResponse response;
		if ( identifiers.isEmpty( ) ) {
			response = pSession.getEnemyList( );
		} else {
			int[] ids = new int[ identifiers.size( ) ];
			int i = 0;
			for ( Integer l : identifiers ) {
				ids[ i++ ] = l.intValue( );
			}
			response = pSession.removeEnemies( alliance , ids );
		}

		return this.render( model , "game" , language , "enemies" , response );
	}

}
