package com.deepclone.lw.web.main.game;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.player.fleets.MoveFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.MultiFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.RenameFleetsResponse;
import com.deepclone.lw.cmd.player.fleets.SplitFleetResponse;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class FleetsPage
		extends PageControllerBase
{

	@RequestMapping( "/fleets" )
	public String fleetsList( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "fleets" , pSession.getFleets( ) );
	}


	@RequestMapping( "/fleets.action" )
	public String fleetsAction( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "selection" , required = false ) String[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Map< String , Object > input = this.getInput( request );
		long fleetIds[] = this.getSelected( selection );
		if ( fleetIds.length == 0 ) {
			return this.redirect( "fleets" );
		}

		// "Split" is special - it needs a single fleet, no less, no more
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		if ( input.containsKey( "split" ) ) {
			if ( fleetIds.length != 1 ) {
				return this.redirect( "fleets" );
			}
			SplitFleetResponse response = pSession.splitFleet( fleetIds[ 0 ] );
			if ( response.getInitialFleet( ) == null ) {
				return this.redirect( "fleets" );
			}
			return this.render( model , "game" , language , "splitFleet" , response );
		}

		// Handle "multiple" commands and responses
		MultiFleetsResponse response = null;
		if ( input.containsKey( "rename" ) ) {
			response = pSession.renameFleets( fleetIds );
		} else if ( input.containsKey( "move" ) ) {
			response = pSession.moveFleets( fleetIds );
		} else if ( input.containsKey( "setAttack" ) ) {
			response = pSession.setFleetsMode( fleetIds , true , false );
		} else if ( input.containsKey( "setDefend" ) ) {
			response = pSession.setFleetsMode( fleetIds , false , false );
		} else if ( input.containsKey( "merge" ) ) {
			response = pSession.mergeFleets( fleetIds );
		} else if ( input.containsKey( "disband" ) ) {
			response = pSession.disbandFleets( fleetIds , false );
		} else {
			return this.redirect( "fleets" );
		}

		// Empty response - we're done, redirect to the fleets page
		if ( response == null || response.getFleets( ).isEmpty( ) ) {
			return this.redirect( "fleets" );
		}

		return this.render( model , "game" , language , "fleetsCommand" , response );
	}


	@RequestMapping( "/move-fleets.action" )
	public String moveFleets( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@RequestParam( value = "selection" , required = false ) String[] selection ,
			@RequestParam( "destination" ) String destination , @RequestParam( "mode" ) String mode )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Map< String , Object > input = this.getInput( request );
		long fleetIds[] = this.getSelected( selection );
		if ( fleetIds.length == 0 || input.containsKey( "cancel" ) ) {
			return this.redirect( "fleets" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		MoveFleetsResponse response = pSession.moveFleets( fleetIds , destination , "1".equals( mode ) );

		if ( response.getFleets( ).isEmpty( ) ) {
			return this.redirect( "fleets" );
		}
		return this.render( model , "game" , language , "fleetsCommand" , response );
	}


	@RequestMapping( "/rename-fleets.action" )
	public String renameFleets( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "selection" , required = false ) String[] selection ,
			@RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Map< String , Object > input = this.getInput( request );
		long fleetIds[] = this.getSelected( selection );
		if ( fleetIds.length == 0 || input.containsKey( "cancel" ) ) {
			return this.redirect( "fleets" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		RenameFleetsResponse response = pSession.renameFleets( fleetIds , name );

		if ( response.getFleets( ).isEmpty( ) ) {
			return this.redirect( "fleets" );
		}
		return this.render( model , "game" , language , "fleetsCommand" , response );
	}


	@RequestMapping( "/set-fleets-mode.action" )
	public String setFleetsMode( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "selection" , required = false ) String[] selection ,
			@RequestParam( "attack" ) String sAttack )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Map< String , Object > input = this.getInput( request );
		long fleetIds[] = this.getSelected( selection );
		if ( fleetIds.length == 0 || input.containsKey( "cancel" ) ) {
			return this.redirect( "fleets" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		pSession.setFleetsMode( fleetIds , "1".equals( sAttack ) , true );
		return this.redirect( "fleets" );
	}


	@RequestMapping( "/disband-fleets.action" )
	public String disbandFleets( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @RequestParam( value = "selection" , required = false ) String[] selection )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Map< String , Object > input = this.getInput( request );
		long fleetIds[] = this.getSelected( selection );
		if ( fleetIds.length == 0 || input.containsKey( "cancel" ) ) {
			return this.redirect( "fleets" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		pSession.disbandFleets( fleetIds , true );
		return this.redirect( "fleets" );
	}


	@RequestMapping( "/split-fleet.action" )
	public String splitFleet( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@RequestParam( "fleet" ) String idStr , @RequestParam( "nFleets" ) String nbStr ,
			@RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		long id;
		try {
			id = Long.parseLong( idStr );
		} catch ( NumberFormatException e ) {
			return this.redirect( "fleets" );
		}

		int nb;
		try {
			nb = Integer.parseInt( nbStr );
		} catch ( NumberFormatException e ) {
			nb = 0;
		}

		Map< String , Object > input = this.getInput( request );
		Map< Integer , Integer > ships = new HashMap< Integer , Integer >( );

		// Get ship counts
		for ( Map.Entry< String , Object > contents : input.entrySet( ) ) {
			String k = contents.getKey( );
			if ( "cancel".equals( k ) ) {
				return this.redirect( "fleets" );
			}

			if ( !k.startsWith( "ships_" ) ) {
				continue;
			}

			try {
				int shipType = Integer.parseInt( k.substring( 6 ) );
				String[] value = (String[]) contents.getValue( );
				ships.put( shipType , Integer.parseInt( value[ 0 ] ) );
			} catch ( Throwable t ) {
				continue;
			}
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		SplitFleetResponse response = pSession.splitFleet( id , ships , nb , name );
		if ( response.getInitialFleet( ) == null ) {
			return this.redirect( "fleets" );
		}

		return this.render( model , "game" , language , "splitFleet" , response );
	}


	private long[] getSelected( String[] selection )
	{
		if ( selection == null ) {
			return new long[] { };
		}

		ArrayList< Long > selected = new ArrayList< Long >( );
		for ( String sel : selection ) {
			try {
				selected.add( Long.parseLong( sel ) );
			} catch ( NumberFormatException e ) {
				// EMPTY - ignore errors
			}
		}

		long[] values = new long[ selected.size( ) ];
		for ( int i = 0 ; i < values.length ; i++ ) {
			values[ i ] = selected.get( i ).longValue( );
		}

		return values;
	}
}
