package com.deepclone.lw.web.main.game;


import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.cmd.player.ViewMapResponse;
import com.deepclone.lw.cmd.player.gdata.MapSize;
import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class MapPage
		extends PageControllerBase
{

	@RequestMapping( "/map" )
	public String viewMap( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		ViewMapResponse map = this.getMap( pSession , request.getSession( ) );
		return this.render( model , "game" , language , "map" , map );
	}


	@RequestMapping( value = "/move-map.action" , method = RequestMethod.POST )
	public String viewMap( HttpServletRequest request , @RequestParam( "x" ) String xStr ,
			@RequestParam( "y" ) String yStr , @RequestParam( "sz" ) String szStr )
	{
		int x , y , szOrd;
		x = this.getInt( xStr );
		y = this.getInt( yStr );
		szOrd = this.getInt( szStr );

		MapSize sz;
		try {
			sz = MapSize.values( )[ szOrd ];
		} catch ( ArrayIndexOutOfBoundsException e ) {
			sz = MapSize.SMALL;
		}

		HttpSession session = request.getSession( );
		session.setAttribute( "mapX" , x );
		session.setAttribute( "mapY" , y );
		session.setAttribute( "mapSize" , sz );

		return this.redirect( "map" );
	}


	private int getInt( String str )
	{
		try {
			return Integer.parseInt( str );
		} catch ( NumberFormatException e ) {
			return 0;
		}
	}


	private ViewMapResponse getMap( PlayerSession pSession , HttpSession session )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		Integer mx = (Integer) session.getAttribute( "mapX" );
		Integer my = (Integer) session.getAttribute( "mapY" );
		MapSize size = (MapSize) session.getAttribute( "mapSize" );

		ViewMapResponse vmr;
		if ( mx == null || my == null || size == null ) {
			vmr = pSession.viewMap( );
		} else {
			vmr = pSession.viewMap( mx , my , size );
		}

		session.setAttribute( "mapX" , vmr.getX( ) );
		session.setAttribute( "mapY" , vmr.getY( ) );
		session.setAttribute( "mapSize" , vmr.getSize( ) );

		return vmr;
	}
}
