package com.deepclone.lw.web.main.game;


import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.deepclone.lw.session.SessionException;
import com.deepclone.lw.web.beans.intercept.SessionRequirement;
import com.deepclone.lw.web.beans.session.SessionMaintenanceException;
import com.deepclone.lw.web.beans.session.SessionServerException;
import com.deepclone.lw.web.beans.view.PageControllerBase;
import com.deepclone.lw.web.csess.PlayerSession;



@Controller
@SessionRequirement( value = true , redirectTo = "player-session" , subType = "game" )
@SessionAttributes( "language" )
public class PlanetPage
		extends PageControllerBase
{

	@RequestMapping( "/planet-{planetId}" )
	public String view( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable String planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int id;
		try {
			id = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.getPlanetView( id ) );
	}


	@RequestMapping( value = "/planet-{planetId}-rename.action" , method = RequestMethod.POST )
	public String rename( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable String planetId , @RequestParam( "name" ) String name )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.rename( pId , name ) );
	}


	@RequestMapping( value = "/planet-{planetId}-abandon.action" , method = RequestMethod.POST )
	public String abandon( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable String planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.abandon( pId ) );
	}


	@RequestMapping( value = "/planet-{planetId}-cancel-abandon.action" , method = RequestMethod.POST )
	public String cancelAbandon( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @PathVariable String planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.cancelAbandon( pId ) );
	}


	@RequestMapping( value = "/planet-{planetId}-build-civ.action" , method = RequestMethod.POST )
	public String construct( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable String planetId , @RequestParam( "type" ) String type ,
			@RequestParam( "amount" ) String amount )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		int tId;
		try {
			tId = Integer.parseInt( type );
		} catch ( NumberFormatException e ) {
			return this.redirect( "planet-" + pId + "#buildings" );
		}

		int x;
		try {
			x = Integer.parseInt( amount );
		} catch ( NumberFormatException e ) {
			x = 0;
		}
		if ( x <= 0 ) {
			return this.redirect( "planet-" + pId + "#buildings" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.constructBuildings( pId , tId , x ) );
	}


	@RequestMapping( value = "/planet-{planetId}-destroy.action" , method = RequestMethod.POST )
	public String destroy( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable String planetId , @RequestParam( "type" ) String type ,
			@RequestParam( "amount" ) String amount )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		int tId;
		try {
			tId = Integer.parseInt( type );
		} catch ( NumberFormatException e ) {
			return this.redirect( "planet-" + pId + "#buildings" );
		}

		int x;
		try {
			x = Integer.parseInt( amount );
		} catch ( NumberFormatException e ) {
			x = 0;
		}
		if ( x <= 0 ) {
			return this.redirect( "planet-" + pId + "#buildings" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.destroyBuildings( pId , tId , x ) );
	}


	@RequestMapping( value = "/planet-{planetId}-build-mil.action" , method = RequestMethod.POST )
	public String buildShips( HttpServletRequest request , @ModelAttribute( "language" ) String language , Model model ,
			@PathVariable String planetId , @RequestParam( "type" ) String type ,
			@RequestParam( "amount" ) String amount )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		int tId;
		try {
			tId = Integer.parseInt( type );
		} catch ( NumberFormatException e ) {
			return this.redirect( "planet-" + pId + "#ships" );
		}

		int x;
		try {
			x = Integer.parseInt( amount );
		} catch ( NumberFormatException e ) {
			x = 0;
		}
		if ( x <= 0 ) {
			return this.redirect( "planet-" + pId + "#ships" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.buildShips( pId , tId , x ) );
	}


	@RequestMapping( value = "/planet-{planetId}-flush-civ.action" , method = RequestMethod.POST )
	public String flushCivQueue( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @PathVariable String planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.flushQueue( pId , false ) );
	}


	@RequestMapping( value = "/planet-{planetId}-flush-mil.action" , method = RequestMethod.POST )
	public String flushMilQueue( HttpServletRequest request , @ModelAttribute( "language" ) String language ,
			Model model , @PathVariable String planetId )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession pSession = this.getSession( PlayerSession.class , request );
		return this.render( model , "game" , language , "planet" , pSession.flushQueue( pId , true ) );
	}


	/**
	 * Handler for commands that modify planet mining settings
	 * 
	 * <p>
	 * This method handles all mining settings commands, including both switching between empire-
	 * and planet-specific settings and updating all priorities.
	 * 
	 * @param request
	 *            the HTTP request
	 * @param model
	 *            the model
	 * @param planetId
	 *            the planet's identifier string from the path
	 * @param toggle
	 *            this parameter from the form's <code>toggle-settings</code> value will be set if
	 *            the "Switch to empire-wide/planet-specific settings" button was clicked.
	 * 
	 * @return a redirect to the overview page if the planet identifier was invalid, or a redirect
	 *         to the planet page's natural resources tab otherwise.
	 * 
	 * @throws SessionException
	 *             if some error occurs on the server
	 * @throws SessionServerException
	 *             if the server is unreachable
	 * @throws SessionMaintenanceException
	 *             if the game is under maintenance
	 */
	@RequestMapping( value = "/planet-{planetId}-update-mset.action" , method = RequestMethod.POST )
	public String updateMiningSettings( HttpServletRequest request , Model model , @PathVariable String planetId ,
			@RequestParam( value = "toggle-settings" , required = false ) String toggle )
			throws SessionException , SessionServerException , SessionMaintenanceException
	{
		int pId;
		try {
			pId = Integer.parseInt( planetId );
		} catch ( NumberFormatException e ) {
			return this.redirect( "overview" );
		}

		PlayerSession session = this.getSession( PlayerSession.class , request );
		if ( toggle == null ) {
			Map< String , Integer > settings = this.getMiningSettings( request );
			if ( settings != null ) {
				session.updatePlanetMiningSettings( pId , settings );
			}
		} else {
			session.toggleMiningSettingsFor( pId );
		}

		return this.redirect( "planet-" + Integer.toString( pId ) + "#natres" );
	}


	/**
	 * Extract mining priorities from the HTTP request
	 * 
	 * <p>
	 * Look for all submitted fields that begin with "pms-" then try to extract their values into a
	 * map that associates resource identifiers to priorities.
	 * 
	 * @param request
	 *            the HTTP request
	 * 
	 * @return the map containing the submitted mining settings, or <code>null</code> if one of the
	 *         values was incorrect.
	 */
	private Map< String , Integer > getMiningSettings( HttpServletRequest request )
	{
		Map< String , Object > input = this.getInput( request );
		Map< String , Integer > miningSettings = new HashMap< String , Integer >( );
		for ( Entry< String , Object > entry : input.entrySet( ) ) {
			// Ignore items which are not mining settings
			String name = entry.getKey( );
			if ( !name.startsWith( "pms-" ) ) {
				continue;
			}
			name = name.substring( 4 );

			// Get values
			if ( ! ( entry.getValue( ) instanceof String[] ) ) {
				continue;
			}
			String[] values = (String[]) entry.getValue( );
			if ( values.length < 1 ) {
				continue;
			}

			// Pre-validate them
			int value;
			try {
				value = Integer.parseInt( values[ 0 ] );
			} catch ( NumberFormatException e ) {
				value = -1;
			}
			if ( value < 0 || value > 4 ) {
				return null;
			}

			miningSettings.put( name , value );
		}
		return miningSettings;
	}
}
